/* -*- linux-c -*- (Autoset linux indent style when using emacs) */

/*
 * Copyright 1999 by Digi International (www.digi.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *----------------------------------------------------------------------
 *  Include File:  dxbisdn.h
 *
 *  Purpose:
 *
 *      STREAMS putmsg/getmsg patterned structure to pass DxB command
 *      and data to the host driver.
 *
 *
 *  Revision History:
 *      $Log: dgdm_isdn.h,v $
 *      Revision 1.4  2001/04/05 15:46:34  randall
 *      - Updates to support Linux 2.4
 *
 *      Revision 1.3  1999/12/11 00:20:39  randall
 *      - Complete code cleanup:
 *        - All functions prototyped.
 *        - Global includes, #defines, and structs all moved to the new
 *          dgdm_driver.h
 *        - eqm.c, audiotap.c and mgmt.c renamed with "dgdm_" prefixes
 *        - Global functions renamed to have "dgdm_" prefixes.
 *        - Unused functions removed.
 *        - #if 0 code also removed.
 *        - Corrected all instances of #include "file.c"
 *        - Makefile modified to stop including libc.a in final linking
 *        - Makefile modified to link in the tracing only when tracing
 *          enabled in the Makefile.
 *      - Tracebuffer changed to use dynamically allocated memory, thus
 *        allowing arbitrary sizes without recompilation.
 *      - Fastbaud functionality removed from the ioctls in dgdm_tty.c
 *        as per Functional Specification changes.
 *
 *      Revision 1.2  1999/09/03 14:54:45  dickl
 *      Added the GNU Public License to pretty much everything
 *      Minor Makefile changes
 *
 *      Revision 1.1  1999/06/23 22:08:29  randall
 *      The great renaming from digi_dxb to dgdm as per Rick Richardson's request.
 *
 *      Revision 1.2  1999/05/12 13:59:01  dickl
 *      Minor reformatting
 *
 *
 *----------------------------------------------------------------------
 */

#if !defined(DXBISDNH)
#define DXBISDNH

#if !defined(KERNEL_VERSION)
# include <linux/version.h>
#endif

/*
 * This structure is a rip-off of the STREAMS strbuf definition used
 * to pass control or data information in a putmsg or getmsg call.
 */
typedef struct
{
	int     maxlen;                 /* Max. length of buffer. */
	int     len;                    /* Bytes used in buffer. */
	int     buffer_offset;          /* Buffer address. */
} STRBUF;

typedef struct
{
	int     command;                /* DxB Command. */
					/* Use DxB commands from dxbglbl.h. */
	int     priority;               /* Priority value setting. */
	STRBUF  control;                /* Control message buffer. */
	STRBUF  data;                   /* Data message buffer. */
} DXB_STRM_MESSAGE;

/*
 * Lifted from SUNOS.
 */
# define I_STR   	(('S' << 8) | 010)
# define I_VERYRAW	(('S' << 8) | 017)

typedef struct
{
	int     ic_cmd;
	int     ic_timout;
	int     ic_len;
	char    *ic_dp;
} strioctl;

struct strbuf
{
	int	maxlen;
	int	len;
	char	*buf;
};
#endif /* DXBISDNH */
