/*****************************************************************************
 * Copyright (c) 1988-95 Digi International Inc., All Rights Reserved
 * 
 * This  software  contains  proprietary  and  confidential   information   of
 * Digi International  Inc.   By  accepting  transfer  of this copy, Recipient
 * agrees to retain this software in  confidence,  to  prevent  disclosure  to
 * others,  and  to make no  use of this software other than that for which it
 * was delivered.  This is an unpublished copyright work  Digi   International
 * Inc.    Except   as   permitted   by  federal  law,  17 USC 117, copying is
 * strictly prohibited.
 * 
 * Digi  International  Inc.  CONFIDENTIAL  -   (Digi    International    Inc.
 * Confidential  when   combined   with   the   aggregated  modules  for  this
 * product) OBJECT CODE ONLY SOURCE MATERIALS
 * 
 * (C) COPYRIGHT Digi International Inc. 1995
 * 
 * All Rights Reserved
 * 
 * US   Government   Users   Restricted   Rights   -   Use,   duplication   or
 * disclosure  restricted by GSA ADP Schedule Contract with Digi International
 * Inc.
 * 
 *****************************************************************************/

/*
 * $Id: bchan_proto.h,v 1.2 2000/03/06 21:28:24 ron Exp $
 *
 * This file contains the protocol structures for the B-channel driver
 */

#ifndef _BCHAN_PROTO_H_
#define _BCHAN_PROTO_H_

#define XN_CONNECT_B	0x01      /* Uses message type `xn_b_call_t' */
#define XN_CONNACK_B	0x02      /* Uses message type `xn_b_call_t' */
#define XN_DISCONNECT_B	0x03      /* Uses message type `xn_b_call_t' */
#define XN_DISCONNACK_B	0x04      /* Uses message type `xn_b_call_t' */
#define XN_CACTIVE_B3	0x05      /* Uses message type `xn_b_call_t' */
#define XN_B_PROTOCOL	0x06      /* Uses message type `xn_sel_b_proto_t' */
#define	XN_B_DATA	0x07      /* Uses message type `xn_b_data_t' */

#define	XN_BAD_B_MSG	0xFF   /* Uses message type `xn_bad_b_msg_ind_t' */

/*
 * Bad message (XN_BAD_B_MSG) reason codes.
 */
#define	BCHAN_REASON_MESSAGE_SIZE    0x01  /* Message size wrong for command */
#define	BCHAN_REASON_BAD_COMMAND     0x02  /* Command not recognized */
#define	BCHAN_REASON_BAD_STATE       0x03  /* Inappropriate message */
#define	BCHAN_REASON_BAD_CHAN_TYPE   0x04  /* Channel type not supported */
#define	BCHAN_REASON_INVALID_CALL_ID 0x05  /* Call ID cannot possibly exist */
#define	BCHAN_REASON_UNUSED_CALL_ID  0x06  /* Call ID not in use */

/*
 * Error (XN_B_DATA) codes.
 */
#define ERROR_NONE		0x00000000	/* Good data */
#define ERROR_HDLC_UNDERRUN	0x00000001
#define ERROR_HDLC_SHORT	0x00000020
#define ERROR_HDLC_FORMAT	0x00000040
#define ERROR_HDLC_OVERRUN	0x00000080	/* Includes long frame */
#define ERROR_HDLC_ABORTED	0x00000100
#define ERROR_HDLC_CRC		0x00000200
#define ERROR_HDLC_LENGTH	0x00000400	/* Good long frame */
#define ERROR_HDLC_MARK		0x00000800	/* Mark spacing follows */

/******************************************************************************
 * Redefinitions of ISDN message primitives that are easier to understand
 *****************************************************************************/

/* b-channel control */
#define XN_SELECT_B_PROTO_REQ	XN_B_PROTOCOL
#define XN_SELECT_B_PROTO_CONF	XN_B_PROTOCOL
#define XN_CONN_B_REQ		XN_CONNECT_B
#define XN_CONN_B_CONF		XN_CONNACK_B
#define XN_CONN_B_IND		XN_CONNECT_B
#define XN_CONN_B_RESP		XN_CONNACK_B
#define XN_CONN_B3_ACT_IND	XN_CACTIVE_B3
#define XN_CONN_B3_ACT_RESP	XN_CACTIVE_B3
#define XN_DISCONN_B_REQ	XN_DISCONNECT_B
#define XN_DISCONN_B_CONF	XN_DISCONNACK_B
#define XN_DISCONN_B_IND	XN_DISCONNECT_B
#define XN_DISCONN_B_RESP	XN_DISCONNACK_B
#define XN_BAD_B_MSG_IND	XN_BAD_B_MSG

/* bchannel hotpath control primitives */
#define XN_FLUSH_REQ		XN_FLUSH
#define XN_MARK_REQ		XN_MARK
#define XN_SYNC_REQ		XN_SYNC
#define XN_SYNC_CONF		XN_SYNC
#define	XN_DATA_REQ		XN_B_DATA
#define	XN_DATA_IND		XN_B_DATA

/******************************************************************************
 * B protocol definitions and structures 
 *****************************************************************************/

/*
 * B1 protocol definitions
 */
#define B1_HDLC64       0      /* 64 kbit/s w/HDLC framing (default) */
#define B1_RAW64        1      /* 64 kbit/s bit-transparent */
#define B1_V110_ASYNC	2      /* V.110 async w/start-stop framing */
#define B1_V110_SYNC	3      /* V.110 sync w/HDLC framing */
#define B1_T30		4      /* T.30 Modem for Group 3 FAX */
#define B1_HDLC64_INV	5      /* 64 kbit/s inverted w/HDLC framing */
#define B1_RAW56        6      /* 56 kbit/s bit-transparent */
#define B1_MODEM_FN	7      /* Modem w/full negotiation */
#define B1_MODEM_SSBF	8      /* Modem async w/start-stop framing */
#define B1_MODEM_HDLC	9      /* Modem sync w/ HDLC framing */

#define B1_CIP_DEFAULT  256
#define B1_HDLC56       257
#define B1_ULAW         258    /* Not Supported */
#define B1_ALAW         259    /* Not Supported */
#define B1_NONE         260
#define B1_HDLC_ID      261    /* Layer 1 auto frame detection mode */

/* B1 protocol/layer 1 config map 

 * B1_HDLC64    L1_HDLC64
 * B1_RAW64     L1_RAW64
 * B1_V110_SYNC L1_V110         
 * B1_RAW56     L1_RAW56
 * ?            B1_CIP_DEFAULT 
 * ?            B1_HDLC56
 * ?            B1_ULAW
 * ?            B1_ALAW
 * ?            B1_V120 
 * ?            B1_NONE
 * ?            B1_HDLC_ID
 */

/*
 * B2 protocol definitions
 */
#define B2_X75SLP       0      /* ISO 7776 (X75 SLP) (default) */
#define B2_RAW		1      /* Transparent */
#define B2_SDLC		2      /* SDLC */
#define B2_LAPD		3      /* LAPD (Q.921) for D-channel X.25 on SAPI 16 */
#define B2_T30		4      /* T.30 for Group 3 FAX */
#define B2_PPP		5      /* Point-to-Point Protocol (PPP) */
#define B2_RAW_E	6      /* Transparent (ignore framing errors of B1 ) */
#define B2_MODEM_FN	7      /* Modem w/full neg. (e.g. V.42 bis, MNP 5) */
#define B2_X75SLP_V42	8      /* ISO 7776 w/V.42 bis compression */
#define B2_V120		9      /* V.120 asynchronous mode */
#define B2_V120_V42	10     /* V.120 asynchronous mode w/V.42 bis comp. */
#define B2_V120_RAW	11     /* V.120 bit-transparent mode */
#define B2_LAPD_SAPI	12     /* LAPD (Q.921) for DCH X.25 on SAPI chosen */

/*
 * B3 protocol definitions
 */
#define B3_RAW		0      /* Transparent (default) */
#define B3_T90NL	1      /* T.90NL w/T.70NL */
#define B3_X25_DTE	2      /* ISO 8208 (X25 DTE-DTE) */
#define B3_X25_DCE	3      /* X25 DCE */
#define B3_T30		4      /* T.30 for Group 3 FAX */
#define B3_T30_EXT	5      /* T.30 for Group 3 FAX Extended */
#define B3_RESERVED	6      /* Reserved */
#define B3_MODEM	7      /* Modem */

/*
 * Mode definitions
 */
#define	BMODE_STATIC	0      /* Connect req. -- no disc. indications */
#define	BMODE_ACTIVE	1      /* Connect req. -- allow disc. indications */
#define	BMODE_PASSIVE	2      /* Connect ind. -- allow disc. indications */

typedef struct
{
    dxb_uint16 max_bit_rate;
    dxb_uint16 bits_per_character;
    dxb_uint16 parity;
    dxb_uint16 stop_bits;
    dxb_uint16 options;
    dxb_uint16 speed_negotiation;
    dxb_uint32 max_frame_size; /* maximum frame size for HDLC  */
}
b1_config_t;

typedef struct
{
    dxb_uint8 address_a;
    dxb_uint8 address_b;
    dxb_uint8 modulo_mode;
    dxb_uint8 window_size;
    dxb_uint16 direction;
    dxb_uint16 code_words;
    dxb_uint16 max_string;
}
b2_config_t;

typedef struct
{
    dxb_uint16 options;
    dxb_uint16 resolution;
    dxb_uint16 format;
    char station_id[20];
    char headline[80];
}
b3_t30_config_t;

typedef struct
{
    dxb_uint16 lic;
    dxb_uint16 hic;
    dxb_uint16 ltc;
    dxb_uint16 htc;
    dxb_uint16 loc;
    dxb_uint16 hoc;
    dxb_uint16 modulo_mode;
    dxb_uint16 window_size;
}
b3_config_t;

typedef struct
{
    dxb_uint16 mode ;
    dxb_uint16 b1_protocol;
    dxb_uint16 b2_protocol;
    dxb_uint16 b3_protocol;
    b1_config_t b1_config;
    b2_config_t b2_config;
    b3_config_t b3_config;
}
b_proto_t;

/* typedef struct ncpi_x25 */
typedef struct
{
    dxb_uint8 options;
    dxb_uint8 group;
    dxb_uint8 channel;
    dxb_uint8 contents[1];
}
ncpi_x25_t;

/* typedef struct ncpi_t30 */
typedef struct 
{
    dxb_uint16 rate;
    dxb_uint16 options;
    dxb_uint16 format;
    dxb_uint16 pages;
    dxb_uint32 rx_id;
}
ncpi_t30_t;

/* typedef struct ncpi_modem */
typedef struct 
{
    dxb_uint16 rate;
    dxb_uint16 protocol;
}
ncpi_modem_t;

/*
 * XN_BAD_B_MSG_IND
 *
 * Firmware indication of the receipt of a bad message from the user
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_BAD_B_MSG_IND             */
    dxb_uint32 id;	       /* call or listen id            */
    dxb_uint32 message;	       /* errored message              */
    dxb_uint32 reason;	       /* Cause for error message      */
}
xn_bad_b_msg_ind_t;

/*
 * XN_SELECT_B_PROTO_REQ / XN_SELECT_B_PROTO_CONF
 *
 * User request to change B protocol
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_SELECT_B_PROTO_REQ / CONF */
    dxb_uint32 call_id;	       /* call id                      */
    b_proto_t b_protocol;      /* B-channel protocol config    */
}
xn_sel_b_proto_t;

/******************************************************************************
 * Data API primitive definitions and structures 
 *****************************************************************************/

/*
 * XN_DATA_REQ
 *
 * User request to transmit B channel data
 *
 * XN_DATA_IND
 *
 * Firmware indication of received B channel data 
 *
 * Note: These are control_data messages. The structure below is in the control
 * portion and the data is in the data portion.
 *
 * Note: Many protocols will just use plain data messages.
 * Note: The secondary call id is for protocols supporting virtual calls (X.25)
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_DATA                           */
    dxb_uint32 call_id;	       /* call id                           */
    dxb_uint32 call_id_2;      /* secondary call id                 */
    dxb_uint32 error;	       /* error message type                */
    dxb_uint32 flags;	       /* CAPI flags for X.25 and V.120     */
}
xn_b_data_t;

/*
 * XN_CONN_B_REQ, XN_CONN_B_CONF, XN_CONN_B_IND, XN_CONN_B_RESP
 * XN_DISCONN_B_REQ, XN_DISCONN_B_CONF, XN_DISCONN_B_IND, XN_DISCONN_B_RESP
 * XN_CONN_B3_ACT_IND, XN_CONN_B3_ACT_RESP
 * XN_CONNECT_B, XN_CONNACK_B, XN_DISCONNECT_B, XN_DISCONNACK_B
 *
 * B protocol connect and disconnect messages.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_B_*                       */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 call_id_2;      /* secondary call id            */
    dxb_uint32 info;	       /* info, reason or reject       */
    ncpi_x25_t x25;            /* x25 protocol specific info   */
    ncpi_t30_t t30;            /* t30 protocol specific info   */
    ncpi_modem_t modem;        /* modem protocol specific info */
}
xn_b_call_t;

/*
 * XN_FLUSH
 *
 * Flush inbound and/or outbound data.  This command does NOT affect API
 * commands, including XN_SYNC.  Any of the flags may be used in any
 * combination.  This message may be sent as normal priority or high
 * priority.  No response is generated.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_FLUSH           */
    dxb_uint32 call_id;	       /* call id            */
    dxb_uint32 call_id_2;      /* secondary call id  */
    dxb_uint32 flags;	       /* See below          */
}
xn_flush_t;

typedef xn_flush_t xn_flsh_t;

#define	FLUSH_INPUT_BUFFER	0x01	/* Flush read-side data         */
#define	FLUSH_OUTPUT_BUFFER	0x02	/* Flush write-side data  */
#define	FLUSH_ABORT_TX_FRAME	0x04	/* Abort current outbound frame */

/*
 * XN_MARK
 *
 * Set mark idle detect mode, or issue mark idle interframe fill orders.
 * This command should be issued during active calls only.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_MARK            */
    dxb_uint32 call_id;	       /* call id            */
    dxb_uint32 call_id_2;      /* secondary call id  */
    dxb_uint32 mode;	       /* Operational mode -- see below */
}
xn_mark_t;

/* Choices for xn_mark_t mode field */
#define MARK_TX_DEACTIVATE      1
#define MARK_TX_ACTIVATE        2
#define MARK_TX_ONCE            3
#define MARK_RX_DETECT_ON       4
#define MARK_RX_DETECT_OFF      5

/*
 * XN_SYNC
 *
 * Synchronization message.  Message is sent by user, and firmware bounces
 * message back to user when all data before the message is transmitted,
 * if sync message is sent normal priority.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_SYNC                    */
    dxb_uint32 call_id;	       /* call id                    */
    dxb_uint32 sync_id;	       /* Optional unique identifier */
    dxb_uint32 flags;	       /* See below                  */
}
xn_sync_t;

#define	SYNC_RESHIPRI		0x02	/* Make sync response high priority */

/******************************************************************************
 * Generic B-channel API primitive
 *****************************************************************************/

/*
 * Message header
 */
typedef struct
{
    dxb_uint32 command;
    dxb_uint32 call_id;
}
xn_b_msg_t;

#ifndef __cplusplus
union bchan_primitives
{
    dxb_uint32 command;	      
    xn_b_msg_t msg;

    xn_bad_b_msg_ind_t bad_msg;
    xn_sel_b_proto_t sel_b;
    xn_data_t data;
    xn_b_call_t b_stuff;
    xn_flush_t flush;
    xn_mark_t mark;
    xn_sync_t sync;
};

#define	BCHAN_API_MAX_MSG_SIZE	(sizeof(union bchan_primitives))

#else
typedef struct union_bchan_api {
private:
	// pointer is stored here
	volatile void *location ;

public:
	// methods to present casts of that pointer
	inline xn_bad_b_msg_ind_t *badmsg() { return ((xn_bad_b_msg_ind_t *) location) ; }
	inline xn_sel_b_proto_t *selb() { return ((xn_sel_b_proto_t *) location) ; }
	inline xn_b_data_t *data() { return ((xn_b_data_t *) location) ; }
	inline xn_b_call_t *bcall() { return ((xn_b_call_t *) location) ; }
	inline xn_flush_t *flush() { return ((xn_flush_t *) location) ; }
	inline xn_mark_t *mark() { return ((xn_mark_t *) location) ; }
	inline xn_sync_t *sync() { return ((xn_sync_t *) location) ; }
	inline uint32 command() { return (uint32) *((dxb_uint32 *) location) ; }
	inline uint32 call_id() { return (uint32) ((xn_b_msg_t *) location)->call_id ; }

	// Constructor #1
	union_bchan_api(void *ptr) {
		location = ptr ;
	}

	// Constructor #2
	union_bchan_api(uint8 *ptr) {
		location = (void *) ptr ;
	}

	// Constructor #3
	union_bchan_api(char *ptr) {
		location = (void *) ptr ;
	}

	// Dummy null constructor
	union_bchan_api() {}

	// Eventually, we should come up with a way of quickly giving them
	// the right size.
	inline uint32 max_size() {
		return 512 ;
	}
} bchan_api_t ;

#define	BCHAN_API_MAX_MSG_SIZE	(((bchan_api_t *) 0)->max_size())

#endif /* __cplusplus */

#endif	/* _BCHAN_PROTO_H */

