/*
 * 	DXB library
 *
 *	TODO:
 *	-	This file needs to be split up into pieces:
 *		slaveside, openside, rxbuf, dxbuf, and common.
 *	-	dxbuf option is only partially implemented.
 *
 *	$Id: dxblib.c,v 1.59 2000/06/27 17:43:02 garyp Exp $
 */
#define	DXB_LIBSOURCE	1	/* Needed to properly handle inlines */

#include "dxbtypes.h"
#include "triglbl.h"
#include "dxbglbl.h"
#include "fepglbl.h"
#include "dxblib.h"

#if	!defined(DXB_HOST) && !defined(DXB_ADAPTER)
#define	DXB_HOST	1
#define	DXB_ADAPTER	1
#endif

#if	defined(DXB_HOST) && defined(DXB_ADAPTER) && !defined(DXB_BOTH)
#define	DXB_BOTH	1
#endif

/*
 *	The PROCESS_SIDE macro is used in command processing, where
 *	the function to be called varies depending on whether we are
 *	the "openside" or not. Using the macro makes it easier to
 *	exclude code not required for a single-side implementation.
 *
 *	The first argument is the name of the function to call when
 *	dxb->openside is true.
 *
 *	The second argument is the name of the function to call when
 *	dxb->openside is false.
 *
 *	The third argument is the parameters to pass to either function.
 *	Note that parenthesis are always required, even if there should
 *	only be a single argument, ie. PROCESS_SIDE(funca, funcb, (arg));
 */

#if	defined(DXB_BOTH)
#define	PROCESS_SIDE(a,b,x) if (dxb->openside) a x; else b x
#elif	defined(DXB_HOST)
#define	PROCESS_SIDE(a,b,x) a x
#elif	defined(DXB_ADAPTER)
#define	PROCESS_SIDE(a,b,x) b x
#else
#error	Impossible define error
#endif


/*
 *	Special definitions for multi-processor environment.
 *
 *	NOTE: the DXB_MP flag is only used in conjunction with firmware
 *	builds. It is NOT the beginnings of multi-processor host driver
 *	support.
 */

#ifndef	DXB_MP
#define	GET_OPENMAP_LOCK(d)
#define	FREE_OPENMAP_LOCK(d)
#endif	/* DXB_MP */


/*
 *	For debugging purposes, a few global variables are exposed
 */
DXB	*LibdxbDxb;		/* Addr of last dxb instance initted */
DXBCHAN	*LibdxbChan;		/* last dxb->chan location */
int	LibdxbChanOffset;	/* Offset of dxb->chan from dxb */

/*
 *	Gag me with a spoon.  A couple of forward declarations are
 *	needed.  Department of redundancy department, Kern-again chapter.
 */
static void uninit_rxbuf(DXBCHAN *chan);
static void uninit_dxbuf(DXBCHAN *chan);


/************************************************************************/
/*                                                                      */
/*      General utility routines					*/
/*                                                                      */
/************************************************************************/

#define	MYMIN(A,B) ( (A) < (B) ? (A) : (B) )

#define	X1(WHAT)	dxb_printf(#WHAT " = %x\n", WHAT);

/*
 * BOGUS_VALUE:
 *
 * Used to verify that dxb_data() / dxb_cdata() / etc. are called
 * before attempting to write buffer blocks and/or execute dxb_write_done().
 */
#define	BOGUS_VALUE	((uint32) -1)

/*
 *	Return index of rightmost one in a word
 */
static int
rightmost_one(uint32 val)
{
	int	i;
	uint32	mask = 1;

	for (i = 0; i < 32; ++i)
	{
		if (val & mask) return (i);
		mask = mask << 1;
	}
	return (32);
}


#if	defined(DXB_HOST)
/*
 *	Find the first zero bit in a set, which is the first available
 *	channel number which can be opened.
 */
static int
find_openmap_zero(uint32 *openmap)
{
	int	i, j;

	for (i = 0; i < DXB_MAXCHAN/32; ++i)
	{
		uint32 word = openmap[i];
		uint32 mask = 1;

		if (word == 0xffffffff)
			continue;
		for (j = 0; j < 32; ++j, mask <<= 1)
			if ( (word & mask) == 0)
				return (i*32 + j);
	}
	return (-1);
}
#endif	/* DXB_HOST */


DXB_INLINE(
void
set_openmap_bit(uint32 *openmap, unsigned int bitnum),
{
	openmap[bitnum/32] |= 1 << (bitnum & 31);
}
)


DXB_INLINE(
void
clear_openmap_bit(uint32 *openmap, unsigned int bitnum),
{
	openmap[bitnum/32] &= ~ (1 << (bitnum & 31));
}
)


/*
 *	Determine the range of open channels and set min_channel
 *	and max_channel approriately.  Also, constrain first_channel
 *	to be within that range.
 */
static void
find_openmap_limits(DXB *dxb)
{
	uint32	*openmap = dxb->openmap;
	int	i, j;

	dxb->min_channel = 0;
	dxb->max_channel = 0;

	for (i = 0; i < DXB_MAXCHAN/32; ++i)
	{
		uint32 word = openmap[i];
		uint32 mask = 1;

		if (word == 0)
			continue;
		for (j = 0; j < 32; ++j, mask <<= 1)
			if (word & mask)
			{
				dxb->min_channel = i*32 + j;
				break;
			}
		break;
	}

	for (i = (DXB_MAXCHAN/32) - 1; i >= 0; --i)
	{
		uint32 word = openmap[i];
		uint32 mask = 0x80000000;

		if (word == 0)
			continue;
		for (j = 31; j >= 0; --j, mask >>= 1)
			if (word & mask)
			{
				dxb->max_channel = i*32 + j;
				break;
			}
		break;
	}

	if (dxb->first_channel < dxb->min_channel
		|| dxb->first_channel > dxb->max_channel)
	{
		dxb->first_channel = dxb->max_channel;
	}
}


DXB_INLINE(
uint32
dxb_swaplong(uint32 val),
{
        uint32 d = (val << 16) | (val >> 16);
        return ((d >> 8) & 0x00ff00ff) | ((d << 8) & 0xff00ff00);
}
)

/*
 *	Roundup a value to the next multiple of 4
 */
#define	roundup4(VAL)	( ((VAL) + 3) & ~3 )

/*
 *	Roundup a buffer offset to next word boundary and wrap it
 */
#define	roundup_and_wrap(VAL, Q)	( ((VAL) + 3) & Q->buf.mask4)

/*
 *	Given a buffer and an offset, compute its virtual address
 */
static void *
dxb_buf_vaddr(const DXBBUF *buf, int offset)
{
	int	page;
	int	pageoffset;

	page = offset >> buf->pageshift;
	pageoffset = offset & buf->pagemask;

	return (buf->page[page].virt + pageoffset);
}

/*
 *	for debugging
 */
static int
dxb_data0(const DXBBUF *buf, int offset)
{
	uint32	*p;

	p = (uint32 *) dxb_buf_vaddr(buf, offset);
	return *p;
}

/*
 *	DMA sync wrapper dxb_copy_to_buf
 *
 *	This macro wrapper is an inobtrusive way for people who need 
 *	DMA sync on writes to use that functionality without adding 
 *	any overhead to the 95% of the callers who aren't interested.  
 *      
 *	We probably should do the same thing for the read sync, but 
 *	we took a more global approach to sync on reads which did not
 *	require a dxb pointer in dxb_copy_from_buf.
 */
#if defined(DXB_USE_SYNCWRITE)
#	define dxb_copy_to_buf(buf, off, data, len, dxb) \
	 dxb_copy_to_buf_sync(buf, off, data, len, dxb)
#else
#	define dxb_copy_to_buf(buf, off, data, len, dxb) \
	 dxb_copy_to_buf_no_sync(buf, off, data, len)
#endif

/*
 *	Copy data from the user into a DXB buffer, respecting
 *	page boundaries and the buffer wrap.
 *
 *	Does not require a DXB pointer.
 *	Returns the new DXB buffer offset.
 */
int
dxb_copy_to_buf_no_sync(DXBBUF *buf, int offset, const void *data, uint len)
{
	uint8	*data8 = (uint8 *) data;	/* Compiler fodder */

	while (len)
	{
		int	amt;
		int	page;
		int	pageoffset;

		page = offset >> buf->pageshift;
		pageoffset = offset & buf->pagemask;

		amt = MYMIN(len, buf->pagesize - pageoffset);
		memcpy(buf->page[page].virt + pageoffset, data8, amt);

		offset += amt;
		offset &= buf->mask;
		len -= amt;
		data8 += amt;
	}

	return (offset);
}

/*
 *	DMA-sync conscious copy_to_buf
 *
 *	Copy data from the user into a DXB buffer, respecting
 *	page boundaries and the buffer wrap, syncing as necessary.
 *
 *	Requires dxb pointer to be passed in
 *	Returns the new DXB buffer offset.
 *
 *	A more performance-sensitive treatment of cache-consistency 
 *	might actually try to do the DMA syncs only when needed (when 
 *	q->in is updated), but other factors such as possible non-virtual 
 *	contiguity of DMA make that implementation more trouble than is 
 *	currently worth contemplating.  
 */
int
dxb_copy_to_buf_sync(DXBBUF *buf, int offset, const void *data, uint len, 
	DXB *dxb)
{
	uint8	*data8 = (uint8 *) data;	/* Compiler fodder */

	while (len)
	{
		int	amt;
		int	page;
		int	pageoffset;

		page = offset >> buf->pageshift;
		pageoffset = offset & buf->pagemask;

		amt = MYMIN(len, buf->pagesize - pageoffset);
		memcpy(buf->page[page].virt + pageoffset, data8, amt);
		dxb_syncwrite(buf->page[page].virt + pageoffset, 	
			amt, dxb->syncdata);

		offset += amt;
		offset &= buf->mask;
		len -= amt;
		data8 += amt;
	}

	return (offset);
}

/*
 *	Copy data from a DXB buffer into the user's buffer, respecting
 *	page boundaries and the buffer wrap.  No matter what, pretend
 *	we copied "buf_len" bytes out of the DXB buffer, but never store
 *	more than "data_size" bytes into the user's buffer.
 *
 *	return the new DXB buffer offset.
 */
int
dxb_copy_from_buf(void *data, uint data_size,
			const DXBBUF *buf, int offset, uint buf_len)
{
	uint	len;
	uint8	*data8 = (uint8 *) data;	/* Compiler fodder */

	/*
	 *	Copy as much data as we have or have space for
	 */
	len = MYMIN(data_size, buf_len);

	while (len)
	{
		int	amt;
		int	page;
		int	pageoffset;

		page = offset >> buf->pageshift;
		pageoffset = offset & buf->pagemask;

		amt = MYMIN(len, buf->pagesize - pageoffset);
		memcpy(data8, buf->page[page].virt + pageoffset, amt);

		offset += amt;
		offset &= buf->mask;
		len -= amt;
		data8 += amt;
	}

	/*
	 *	Skip the rest
	 */
	if (buf_len > data_size)
	{
		offset += buf_len - data_size;
		offset &= buf->mask;
	}

	return (offset);
}

/*
 *	Return the absolute number of bytes free in a DXB queue
 */
DXB_INLINE(
int
dxb_bytes_free(const DXBQ *q),
{
	return (q->out - q->in - 4) & q->buf.mask;
}
)

/*
 *	Return the absolute number of bytes queued in a DXB queue
 */
DXB_INLINE(
int
dxb_bytes_queued(const DXBQ *q),
{
	return (q->in - q->out) & q->buf.mask;
}
)


/************************************************************************/
/*                                                                      */
/*      API routines used to determine if a message can be sent		*/
/*                                                                      */
/************************************************************************/

/*
 *	dxb_data_payload_space:
 *
 *	Return the number of bytes available for sending DATA
 *	message payload on a particular "channel".  Typically
 *	used by ASYNC channels where the data is unbounded.
 */
int
dxb_data_payload_space(DXB *dxb, int channel)
{
	DXBQ	*q = dxb->outq;
	int	space;

	space = dxb_bytes_free(q) - DXB_MSG_DATA_SIZE;

	space = MYMIN(space, dxb->chan[channel].credits);

	space -= dxb->chan[channel].his_fixed_cost;

	return (space > 0) ? space : 0;
}

/*
 *	dxb_cdata_payload_space:
 *
 *	Return the number of bytes available for sending CDATA
 *	message payload on a particular "channel".  Typically
 *	used by ASYNC channels where the data is unbounded.
 *	"ctrl_len" is the amount of space needed by the control
 *	portion.  "highpri" is 1 if this message is not subject
 *	to flow control.
 */
int
dxb_cdata_payload_space(DXB *dxb, int channel, int ctrl_len, int highpri)
{
	DXBQ	*q = dxb->outq;
	int	space;

	space = dxb_bytes_free(q) - DXB_MSG_CDATA_SIZE;

	space -= roundup4(ctrl_len);

	if (!highpri)
	{
		space = MYMIN(space, dxb->chan[channel].credits);
		space -= dxb->chan[channel].his_fixed_cost;
	}

	return (space > 0) ? space : 0;
}

/*
 *	dxb_space_avail:
 *
 *	Return 0 if there is space available to send a message on a
 *	"channel" of the given "ctrl_len" and "data_len".  "highpri"
 *	is 1 if the message is not subject to flow control.
 *	Else return ENOSPACE or ENOCRED.
 *
 *	Typically used by non-ASYNC channels where the data is
 *	a bounded message (packet).
 */
int
dxb_space_avail(DXB *dxb, int channel,
		int ctrl_len, int data_len,
		int highpri)
{
	int	need;

	/*
	 *	See if there is space available for this message
	 *	plus one additional message with no data in it.
	 */
	need = roundup4(ctrl_len) + roundup4(data_len);

	if ( (need + 2*DXB_MSG_GENERIC_SIZE) > dxb_bytes_free(dxb->outq))
		return (DXB_ENOSPACE);

	/*
	 *	Priority messages not subject to flow control
	 */
	if (highpri)
		return (0);

	/*
	 *	See if we have enough credits
	 */
	need = dxb->chan[channel].his_fixed_cost + ctrl_len + data_len;

	if (need <= dxb->chan[channel].credits)
		return (0);
	else
		return (DXB_ENOCRED);
}

/*
 *	Return the amount of free space in the DXB queue itself.
 *
 *	This routine returns a pessimistic amount, intentionally.
 */
int
dxb_queue_space(DXB *dxb)
{
	int	space = dxb_bytes_free(dxb->outq) - 2*DXB_MSG_GENERIC_SIZE;

	return (space > 0) ? space : 0;
}

/************************************************************************/
/*									*/
/*	API Data message sending functions				*/
/*									*/
/************************************************************************/

/*
 *	dxb_data:	Enqueue a DATA message header.
 *
 *	"channel" is the channel number, "data_len" is the length in
 *	bytes of the data portion that will be enqueued using
 *	dxb_write_data_block().  Space for the message must have
 *	been previously guaranteed using dxb_space_avail(), or by
 *	using dxb_data_payload_space().
 *
 *	Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *	Typical use of API for sending a DATA message:
 *
 *	for (;;)
 *	{
 *		DXB_LOCK();
 *		if (dxb_space_avail(dxb, chan, 0, data_len, 0) == DXB_OK)
 *			break;
 *		DXB_UNLOCK();
 *		BLOCK();
 *	}
 *	dxb_data(dxb, chan, data_len);
 *	dxb_write_data_block(dxb, data, data_len);
 *	dxb_write_done(dxb);
 *	DXB_UNLOCK();
 */
int
dxb_data(DXB *dxb, int channel, int data_len)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_DATA	msg;

	if (1) dxb_printf("dxb_data(%d, %d)\n", channel, data_len);

	/*
	 *	Save info so dxb_write_data_block()
	 *	and dxb_write_done() will work properly.
	 */
	CTRL_LEN(q) = 0;
	CTRL_CNT(q) = 0;
	DATA_LEN(q) = data_len;
	DATA_CNT(q) = 0;
	HIGHPRI(q) = 0;
	CHANNEL(q) = channel;

	/*
	 *	Construct message header
	 */
	msg.type = DXB_TYPE_DATA;
	msg.channel = channel;
	msg.data_len = data_len;

	q->in_tmp = 
		dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_DATA_SIZE, dxb);

	return (0);
}

/*
 *	dxb_cdata:	Enqueue a CDATA message header
 *
 *	"channel" is the channel number, "highpri" is 1 if this message
 *	is not subject to flow control. "ctrl_len" and "data_len" are
 *	the length in bytes of the control and data portions that will
 *	be enqueued using dxb_write_data_block().  Space for the message
 *	must have been previously guaranteed using dxb_space_avail(), or by
 *	using dxb_data_payload_space().
 *
 *	Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *	Typical use of API for sending a CDATA message:
 *
 *	for (;;)
 *	{
 *		DXB_LOCK();
 *		if (dxb_space_avail(dxb, chan, 0, data_len, ctrl_len) == DXB_OK)
 *			break;
 *		DXB_UNLOCK();
 *		BLOCK();
 *	}
 *	dxb_cdata(dxb, chan, 0, ctrl_len, data_len);
 *	dxb_write_ctrl_block(dxb, ctrl, ctrl_len);
 *	dxb_write_data_block(dxb, data, data_len);
 *	dxb_write_done(dxb);
 *	DXB_UNLOCK();
 */
int
dxb_cdata(DXB *dxb, int channel, int highpri, int ctrl_len, int data_len)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_CDATA	msg;

	if (1) dxb_printf("dxb_cdata(%x, %d, %d, %d, %d)\n",
		(int) dxb, channel, highpri, ctrl_len, data_len);

	/*
	 *	Save info so dxb_write_data_block()
	 *	and dxb_write_done() will work properly.
	 */
	CTRL_LEN(q) = ctrl_len;
	CTRL_CNT(q) = 0;
	DATA_LEN(q) = data_len;
	DATA_CNT(q) = 0;
	HIGHPRI(q) = highpri;
	CHANNEL(q) = channel;

	/*
	 *	Construct message header
	 */
	msg.type = DXB_TYPE_CDATA;
	msg.channel = channel;
	msg.priority = highpri;
	msg.ctrl_len = ctrl_len;
	msg.data_len = data_len;

	q->in_tmp = 
		dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_CDATA_SIZE, dxb);

	return (0);
}

/*
 *	dxb_write_data_block:
 *
 *	Enqueue a block of data.  This routine may be called more than
 *	once if it is desired to queue the data for a message in several
 *	pieces, such as from fragmented STREAMs mblk's.
 *
 *	Returns 0 if AOK, returns DXB_EINVAL if too much data has been queued,
 *	returns DXB_EACCES if this is being called at the wrong time.
 */
int
dxb_write_data_block(DXB *dxb, const void *data, int data_len)
{
	DXBQ		*q = dxb->outq;

	if (q->in_tmp == BOGUS_VALUE)
		return(DXB_EACCES) ;

	q->in_tmp = dxb_copy_to_buf(&q->buf, q->in_tmp, data, data_len, dxb);

	DATA_CNT(q) += data_len;

	if (DATA_CNT(q) > DATA_LEN(q))
		return(DXB_EINVAL);

	if (DATA_CNT(q) == DATA_LEN(q))
		q->in_tmp = roundup_and_wrap(q->in_tmp, q);

	return (0);
}

void *
dxb_peek_data_write(DXB *dxb, int *max)
{
	DXBQ		*q = dxb->outq ;
	int 		page ;
	int 		pageoffset ;

	if (q->in_tmp == BOGUS_VALUE || CTRL_CNT(q) < CTRL_LEN(q)) {
		*max = 0 ;
		return ((void *) 0) ;
	}

	page = q->in_tmp >> q->buf.pageshift ;
	pageoffset = q->in_tmp & q->buf.pagemask ;

	*max = MYMIN(DATA_LEN(q) - DATA_CNT(q), (int) q->buf.pagesize - pageoffset) ;

	return ((void *) (q->buf.page[page].virt + pageoffset)) ;
}

/*
 *	dxb_write_ctrl_block:
 *
 *	Enqueue a block of control data.  This routine may be called more than
 *	once if it is desired to queue the data for a message in several
 *	pieces, such as from fragmented STREAMs mblk's.
 *
 *	Returns 0 if AOK, returns DXB_EINVAL if too much data has been queued,
 *	returns DXB_EACCES if this is being called at the wrong time.
 */
int
dxb_write_ctrl_block(DXB *dxb, const void *ctrl, int ctrl_len)
{
	DXBQ		*q = dxb->outq;

	if (q->in_tmp == BOGUS_VALUE)
		return(DXB_EACCES) ;

	q->in_tmp = dxb_copy_to_buf(&q->buf, q->in_tmp, ctrl, ctrl_len, dxb);

	CTRL_CNT(q) += ctrl_len;

	if (CTRL_CNT(q) > CTRL_LEN(q))
		return(DXB_EINVAL);

	if (CTRL_CNT(q) == CTRL_LEN(q))
		q->in_tmp = roundup_and_wrap(q->in_tmp, q);

	return (0);
}

void *
dxb_peek_ctrl_write(DXB *dxb, int *max)
{
	DXBQ		*q = dxb->outq ;
	int 		page ;
	int 		pageoffset ;

	if (q->in_tmp == BOGUS_VALUE)
		return ((void *) 0) ;

	page = q->in_tmp >> q->buf.pageshift ;
	pageoffset = q->in_tmp & q->buf.pagemask ;

	*max = MYMIN(CTRL_LEN(q) - CTRL_CNT(q), (int) q->buf.pagesize - pageoffset) ;

	return ((void *) (q->buf.page[page].virt + pageoffset)) ;
}

int
dxb_peek_wconfirm(DXB *dxb, int count)
{
	DXBQ		*q = dxb->outq ;
	int 		amt ;

	if (CTRL_CNT(q) < CTRL_LEN(q)) {
		amt = MYMIN(CTRL_LEN(q) - CTRL_CNT(q), count) ;

		CTRL_CNT(q) += amt ;
		count -= amt ;
		q->in_tmp = (q->in_tmp + amt) & q->buf.mask ;

		if (CTRL_LEN(q) == CTRL_CNT(q))
			q->in_tmp = roundup_and_wrap(q->in_tmp, q) ;
	}

	if (DATA_CNT(q) < DATA_LEN(q)) {
		amt = MYMIN(DATA_LEN(q) - DATA_CNT(q), count) ;

		DATA_CNT(q) += amt ;
		count -= amt ;
		q->in_tmp = (q->in_tmp + amt) & q->buf.mask ;

		if (DATA_LEN(q) == DATA_CNT(q))
			q->in_tmp = roundup_and_wrap(q->in_tmp, q) ;
	}

	if (count)
		return DXB_EACCES ;

	return 0 ;
}

/*
 *	dxb_write_done:
 *
 *	Advance in pointer, marking the end of a data message
 *
 *	Returns 0 if everything is AOK.  Returns DXB_EINVAL and does
 *	not send the message if the amount of control or data information
 *	queued is not the same as what was promised.
 *	Returns DXB_EACCES if this is being called at the wrong time.
 */
int
dxb_write_done(DXB *dxb)
{
	DXBQ		*q = dxb->outq;

	if (q->in_tmp == BOGUS_VALUE)
		return(DXB_EACCES) ;

	/*
	 *	If user didn't write the amount of data he promised,
	 *	blow off this command and return an error.
	 */
	if (CTRL_CNT(q) != CTRL_LEN(q))
		return(DXB_EINVAL);

	if (DATA_CNT(q) != DATA_LEN(q))
		return(DXB_EINVAL);

	/*
	 *	OK, advance pointer for keeps.  Its already aligned.
	 */
	q->in = q->in_tmp;
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	/*
	 *	Account for fixed cost and size of this message
	 */
	if ( !HIGHPRI(q))
	{
		dxb->chan[CHANNEL(q)].credits -= (DATA_LEN(q) + CTRL_LEN(q) +
					dxb->chan[CHANNEL(q)].his_fixed_cost);
	}

	/*
	 *	Now, invalidate in_tmp to prevent future procedural errors.
	 */
	q->in_tmp = BOGUS_VALUE ;

	return (0);
}


/************************************************************************/
/*									*/
/*	API Open/close functions					*/
/*									*/
/************************************************************************/

/*
 *	Zap all the callbacks to NULL
 */
static void
clear_callbacks(DXBCHAN *chan)
{
	chan->openack_cb = NULL;
	chan->data_cb = NULL;
	chan->cdata_cb = NULL;
	chan->close_cb = NULL;
	chan->closeack_cb = NULL;
	chan->credit_cb = NULL;
	chan->push_cb = NULL;
	chan->pushack_cb = NULL;
	chan->pop_cb = NULL;
	chan->popack_cb = NULL;
	chan->link_cb = NULL;
	chan->linkack_cb = NULL;
	chan->unlink_cb = NULL;
	chan->unlinkack_cb = NULL;
	chan->flush_cb = NULL;
	chan->flushack_cb = NULL;
	chan->ioctl_cb = NULL;
	chan->ioctlack_cb = NULL;
	chan->cancel_cb = NULL;
}


#if	defined(DXB_HOST)
/*
 *	dxb_open:	Enqueue an open message
 *
 *	The "devid" is a NULL terminated character string specifying
 *	the protocol/module/whatever to be opened.  The "fixed_cost"
 *	is the cost of sending a message that we want the other side
 *	to use in computing credit requirements when sending to us.
 *	The "data" and "data_len" are additional data to be sent with
 *	the open message.
 *
 *	This routine is called only by the "openside" (host).  The
 *	slave side will get a a callback when it receives this message,
 *	which it can handle as it pleases, but typically by sending
 *	back an open acknowledgement.  The "openside" should call
 *	set_openack_cb() immediately after calling this routine so
 *	that it can get a callback when the open is completed.
 *
 *	Returns a channel number (0-255) on success.  Returns
 *	DXB_EMCHAN if all channels are in use.  Returns DXB_ENOSPACE
 *	if there isn't space to send the message right now (try later).
 *
 *	Typical use:
 *		uint32	minor = 42;
 *		for (;;)
 *		{
 *			DXB_LOCK();
 *			dxbchan = dxb_open(&Dxb, "SERIAL", 0,
 *						&minor, sizeof(minor));
 *			if (dxbchan >= 0) break;
 *			DXB_UNLOCK();
 *			BLOCK();
 *		}
 *
 *		dxb_set_openack_cb(&Dxb, dxbchan, ack_cb, minor);
 *		DXB_UNLOCK();
 *		BLOCK();	// Until ack_cb() gets called
 *		
 */
int
dxb_open(
	DXB		*dxb,
	char		*devid,
	int		fixed_cost,
	void		*data,
	int		data_len
	)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_OPEN	msg;
	int		channel;
	int		rc;
	uint32		in_tmp ;
	int		ctrl_len = strlen(devid) + 1;

	if (1) dxb_printf("dxb_open(%x, %s, %d, (%d), %d)\n",
		(int) dxb, devid, fixed_cost,
		data ? ((int *) data)[0] : 0, data_len);

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, data_len, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Get a channel number to return
	 */
	GET_OPENMAP_LOCK(dxb);
	channel = find_openmap_zero(dxb->openmap);
	if (channel == -1)
	{
		FREE_OPENMAP_LOCK(dxb);
		return (DXB_EMCHAN);
	}

	set_openmap_bit(dxb->openmap, channel);
	find_openmap_limits(dxb);
	FREE_OPENMAP_LOCK(dxb);

	/*
	 *	Save fixed cost info, and zero credits
	 */
	dxb->chan[channel].credits = 0;
	dxb->chan[channel].credits_owed = 0;
	dxb->chan[channel].credit_return_level = 0;
	dxb->chan[channel].my_fixed_cost = fixed_cost;
	dxb->chan[channel].his_fixed_cost = 0;

	/*
	 *	zero all the callbacks
	 */
	clear_callbacks(&dxb->chan[channel]);

	/*
	 *	Construct message header
	 */
	msg.type = DXB_TYPE_OPEN;
	msg.channel = channel;
	msg.fixed_cost = fixed_cost;
	msg.unused = 0 ;
	msg.ctrl_len = ctrl_len;
	msg.data_len = data_len;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_OPEN_SIZE, dxb);

	/*
	 *	Copy in control portion
	 */
	in_tmp = dxb_copy_to_buf(&q->buf, in_tmp, devid, ctrl_len, dxb);
	in_tmp = roundup_and_wrap(in_tmp, q);

	/*
	 *	Copy in data portion
	 */
	if (data_len)
	{
		in_tmp = dxb_copy_to_buf(&q->buf, in_tmp, data, data_len, dxb);
		in_tmp = roundup_and_wrap(in_tmp, q);
	}

	q->in = in_tmp;
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	/*
	 *	Indicate that we are not using the rxbuf or dxbuf features
	 */
	dxb->chan[channel].rxbuf = NULL;
	dxb->chan[channel].dxbuf = NULL;

	return (channel);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_openack:	Enqueue an open acknowledgment message
 *
 *	This routine sends the response to an open message that has been
 *	received.  It is called only by the slave (adapter).  The "fixed_cost"
 *	is the cost of sending a message that we want the other side
 *	to use in computing credit requirements when sending to us.
 *	The "error_code" should be 0 if the open has been performed,
 *	else anything non-zero is an error that is sent to the host.
 *
 *	Returns 0 on success. Returns DXB_ENOSPACE if there isn't space
 *	to send the message right now (try later).
 *
 *	A typical use is in a DXB_OPEN_CB callback routine (note that
 *	in a typical implementation, LOCKs are already held by now):
 *
 *		rc = dxb_openack(&Dxb, channel, 0, 0);
 *		if (rc != 0)
 *			must defer ack until later by some mechanism
 *
 */
int
dxb_openack(
	DXB		*dxb,
	int		channel,
	int		fixed_cost,
	int		error_code
	)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_OPEN	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;
	uint32		in_tmp ;
	int		rc;

	if (1) dxb_printf("dxb_openack(%x, %d, %d,%d)\n",
		(int) dxb, channel, fixed_cost, error_code);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, 0, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Save my fixed cost info
	 */
	dxb->chan[channel].my_fixed_cost = fixed_cost;

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_OPEN;
	msg.channel = channel;
	msg.fixed_cost = fixed_cost;
	msg.ctrl_len = ctrl_len;
	msg.data_len = 0;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_OPEN_SIZE, dxb);

	/*
	 *	Copy in control portion
	 */
	ctrl[0] = error_code ? -1 : 0;
	ctrl[1] = error_code;

	in_tmp = dxb_copy_to_buf(&q->buf, in_tmp, ctrl, ctrl_len, dxb);
	q->in = roundup_and_wrap(in_tmp, q);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	if (error_code)
		return (0);

	/*
	 *	Mark channel open
	 */
	GET_OPENMAP_LOCK(dxb);
	set_openmap_bit(dxb->openmap, channel);
	find_openmap_limits(dxb);
	FREE_OPENMAP_LOCK(dxb);

	/*
	 *	Indicate that we are not using the rxbuf or dxbuf features
	 */
	dxb->chan[channel].rxbuf = NULL;
	dxb->chan[channel].dxbuf = NULL;

	return (0);
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_ADAPTER)
/*
 *      dxb_opennak:    Enqueue a failed open acknowledgment message
 *
 *      This routine is used to queue a negative acknowledgement to
 *      an open request.  The "error_code" must be non-zero.  This
 *      routine is different from dxb_openack in that if the message
 *      cannot be sent, then the dxb library will automatically attempt
 *      to send it each time dxb_rxpoll() is called.
 *
 *      Returns DXB_EPERM if called by the openside, DXB_EINVAL if called
 *      with a zero "error_code", otherwise returns 0.
 *
 *      The reason this function exists is as a convenience for users.
 *      When NAKing an open, the caller typically wants to retain no
 *      state information or data structures for the open.  So as a
 *      convenience the library maintains the state for this one case only.
 *
 */
int
dxb_opennak(
	DXB		*dxb,
	int		channel,
	int		error_code
	)
{
	int	rc;

	if (error_code == 0)
		return (DXB_EINVAL);

	if (dxb->openside)
		return (DXB_EPERM);

	rc = dxb_openack(dxb, channel, 0, error_code);
	if (rc)
		dxb->chan[channel].nak_error_code = error_code;
	else
		dxb->chan[channel].nak_error_code = 0;

	return (0);
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	dxb_close:	Enqueue a close message
 *
 *	"channel" is the DXB channel to close.
 *	Called by "openside" (host) only.
 *
 *	Returns 0 on success, DXB_ENOSPACE if there is no room for message.
 *
 *	Typical use:
 *		for (;;)
 *		{
 *			LOCK();
 *			rc = dxb_close(&Dxb, channel);
 *			UNLOCK();
 *			if (rc == 0) break;
 *			BLOCK();
 *		}
 */
int
dxb_close(DXB *dxb, int channel)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_CLOSE	msg;

	if (1) dxb_printf("dxb_close(%x, %d)\n",
		(int) dxb, channel);

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.
	 */
	if (dxb_bytes_free(q) < DXB_MSG_CLOSE_SIZE)
		return (DXB_ENOSPACE);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_CLOSE;
	msg.channel = channel;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_CLOSE_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	/*
	 *	The channel is still half open until we get the
	 *	close acknowledgement message.
	 */

	return (0);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_closeack:	Enqueue a close acknowledgement message
 *
 *	This routine sends the response to an open message that has
 *	been received.  It is called only by the slave (adapter).
 *
 *	Returns 0 on success. Returns DXB_ENOSPACE if there isn't space
 *	to send the message right now (try later).
 *
 *	A typical use is in a DXB_CLOSE_CB callback routine (note that
 *	in a typical implementation, LOCKs are already held by now):
 *
 *		rc = dxb_closeack(&Dxb, channel, 0, 0);
 *		if (rc != 0)
 *			must defer ack until later by some mechanism
 */
int
dxb_closeack(DXB *dxb, int channel)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_CLOSE	msg;
	DXBCHAN		*chan;

	if (1) dxb_printf("dxb_closeack(%x, %d)\n",
		(int) dxb, channel);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.
	 */
	if (dxb_bytes_free(q) < DXB_MSG_CLOSE_SIZE)
		return (DXB_ENOSPACE);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_CLOSE;
	msg.channel = channel;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_CLOSE_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	/*
	 *	Mark channel closed
	 */
	chan = &dxb->chan[channel];

	GET_OPENMAP_LOCK(dxb);
	clear_openmap_bit(dxb->openmap, channel);
	find_openmap_limits(dxb);
	FREE_OPENMAP_LOCK(dxb);

	chan->credits_owed = 0;

	/*
	 *	If we are using the rxbuf or dxbuf options, free the buffer
	 */
	uninit_rxbuf(chan);
	uninit_dxbuf(chan);

	/*
	 *	zero all the callbacks
	 */
	clear_callbacks(chan);

	return (0);
}
#endif	/* DXB_ADAPTER */


/*
 *	Enqueue a credit award message
 *
 *	This routine is normally not called by the API user.
 */
int
dxb_credit_award(DXB *dxb, int channel, int credits)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_CREDIT	msg;

	if (dxb_bytes_free(q) < DXB_MSG_CREDIT_SIZE)
		return (DXB_ENOSPACE);

	msg.type = DXB_TYPE_CREDIT;
	msg.channel = channel;
	msg.credit_award = credits;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_CREDIT_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	dxb->chan[channel].credits_owed -= credits;

	return (0);
}

/*
 *	dxb_return_credits:
 *
 *	This routine is called to return payload credits for a channel.
 *	"credits" should be the number of control and data message
 *	bytes being credited.  Fixed cost credit returns are handled by
 *	dxb_return_fixed_cost().
 *
 *	This routine is typically called for two reasons:
 *	1) to initially	supply credits after calling dxb_open()/dxb_openack()
 *	2) to replenish credits after processing a received message.
 *
 *	Depending on the setting of dxb_set_credit_return_level(), this
 *	routine may or may not generate an actual DXB message at the
 *	time that it is called.
 *
 *	This routine returns 0 always.  Honest.
 *
 *	Typical use for the first case is:
 *		dxb_return_credits(dxb, channel, 4096);
 *		dxb_return_fixed_cost(dxb, channel, 100);
 *
 *	Typical use for the second case is:
 *		dxb_return_credits(dxb, channel, ctrl_len + data_len);
 *		dxb_return_fixed_cost(dxb, channel, 1);
 *
 */
int
dxb_return_credits(DXB *dxb, int channel, int credits)
{
	DXBCHAN	*ch = &dxb->chan[channel];

	if (credits == 0)
		return (0);

	ch->credits_owed += credits;

	if (1) dxb_printf("dxb_return_credits(%d, %d)\n", channel, credits);

	/*
	 *	If level is zero, always defer returning credits
	 *	until next poll
	 */
	if (ch->credit_return_level == 0)
		return (0);

	if (ch->credits_owed >= ch->credit_return_level)
	{
		dxb_credit_award(dxb, channel, ch->credits_owed);
	}

	return (0);
}

/*
 *	dxb_return_fixed_cost:
 *
 *	Return the fixed credits for a channel.  This is an additional
 *	abstraction on top of the previous routine.  "Count" multiples
 *	of the fixed cost are returned.  This routine should be called
 *	once per data message received and processed.
 *
 *	This routine returns 0 always.  Honest.
 *
 *	NOTE: this abstraction acknowledges the fact that a DXB instance
 *	may be prepared to handle N bytes of received data and/or M
 *	total messages, whichever is lesser.  However, the underlying
 *	DXB implementation doesn't yet understand the difference and
 *	lumps both into a single figure of merit called credits.
 */
int
dxb_return_fixed_cost(DXB *dxb, int channel, int count)
{
	int	credits;

	credits = dxb->chan[channel].my_fixed_cost * count;

	if (1) dxb_printf("dxb_return_fixed_cost(%d, %d) = %d\n",
		channel, count, credits);

	return dxb_return_credits(dxb, channel, credits);
}

/*
 *	dxb_credits_avail :
 *
 *	Return the number of bytes available in the port's data
 *              queue.  It does not account for the dxb queue.
 */
int
dxb_data_credits_avail (DXB *dxb, int channel)
{
	int	space;

	space = dxb->chan[channel].credits - dxb->chan[channel].his_fixed_cost;

	return (space > 0) ? space : 0;
}


/************************************************************************/
/*									*/
/*	API ioctl message sending functions				*/
/*									*/
/************************************************************************/

#if	defined(DXB_HOST)
/*
 *	dxb_ioctl:	Enqueue an ioctl message
 *
 *	"channel" is the channel number for the message.  "command"
 *	is the ioctl command number.  "data_len" is the length of
 *	data to be sent with the command.  Data is passed in a manner
 *	similar to dxb_data() messages, but using readability macros
 *	dxb_ioctl_space_avail() and dxb_write_ioctl_block().
 *
 *	Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *	Typical use of API for sending an ioctl message:
 *
 *	for (;;)
 *	{
 *		DXB_LOCK();
 *		if (dxb_ioctl_space_avail(dxb, chan, data_len) == DXB_OK)
 *			break;
 *		DXB_UNLOCK();
 *		BLOCK();
 *	}
 *	dxb_ioctl(dxb, chan, command, data_len);
 *	dxb_write_ioctl_block(dxb, data, data_len);
 *	dxb_write_done(dxb);
 *	DXB_UNLOCK();
 */
int
dxb_ioctl(DXB *dxb, int channel, int command, int data_len)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_IOCTL	msg;

	if (1) dxb_printf("dxb_ioctl(%d, %d, %d)\n",
		channel, command, data_len);

	msg.type = DXB_TYPE_IOCTL;
	msg.channel = channel;
	msg.command = command;
	msg.ctrl_len = 0;
	msg.data_len = data_len;

	/*
	 *	Save info so dxb_write_data_block()
	 *	and dxb_write_done() will work properly.
	 */
	CTRL_LEN(q) = 0;
	CTRL_CNT(q) = 0;
	DATA_LEN(q) = data_len;
	DATA_CNT(q) = 0;
	HIGHPRI(q) = DXB_PRI_HIGH;
	CHANNEL(q) = channel;

	q->in_tmp = 
		dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_IOCTL_SIZE, dxb);

	return (0);
}
#endif	/* DXB_HOST */


/*
 *	dxb_ioctlack:	Enqueue an ioctl acknowledgement message
 *
 *	"channel" is the channel number for the message.  "command"
 *	is the ioctl command number.  "data_len" is the length of
 *	data to be sent with the command.  "error_code" is the error
 *	indicator (0 is AOK). Data is passed in a manner
 *	similar to dxb_data() messages, but using readability macros
 *	dxb_ioctlack_space_avail() and dxb_write_ioctl_block().
 *
 *	Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *	Typical use of API for sending an ioctl message:
 *
 *	for (;;)
 *	{
 *		DXB_LOCK();
 *		if (dxb_ioctlack_space_avail(dxb, chan, data_len) == DXB_OK)
 *			break;
 *		DXB_UNLOCK();
 *		BLOCK();
 *	}
 *	dxb_ioctlack(dxb, chan, command, data_len);
 *	dxb_write_ioctl_block(dxb, data, data_len);
 *	dxb_write_done(dxb);
 *	DXB_UNLOCK();
 */
int
dxb_ioctlack(DXB *dxb, int channel, int command, int data_len, int error_code)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_IOCTL	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;

	if (1) dxb_printf("dxb_ioctlack(%d, %d, %d, %d)\n",
		channel, command, data_len, error_code);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_IOCTL;
	msg.channel = channel;
	msg.command = command;
	msg.ctrl_len = ctrl_len;
	msg.data_len = data_len;

	/*
	 *	Save info so dxb_write_ioctl_block()
	 *	and dxb_write_done() will work properly.
	 */
	CTRL_LEN(q) = ctrl_len;
	CTRL_CNT(q) = 0;
	DATA_LEN(q) = data_len;
	DATA_CNT(q) = 0;
	HIGHPRI(q) = DXB_PRI_HIGH;
	CHANNEL(q) = channel;

	q->in_tmp = 
		dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_IOCTL_SIZE, dxb);

	ctrl[0] = error_code ? -1 : 0;
	ctrl[1] = error_code;
	dxb_write_ctrl_block(dxb, ctrl, ctrl_len);

	return (0);
}


#if	defined(DXB_HOST)
/*
 *	dxb_cancel:
 *
 *	Enqueue a cancel ioctl message for "channel".  "command" is
 *	the ioctl being canceled.
 */
int
dxb_cancel(DXB *dxb, int channel, int command)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_CANCEL	msg;

	if (dxb_bytes_free(q) < DXB_MSG_CANCEL_SIZE)
		return (DXB_ENOSPACE);

	msg.type = DXB_TYPE_CANCEL;
	msg.channel = channel;
	msg.command = command;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_CANCEL_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_HOST */


/************************************************************************/
/*									*/
/*	API STREAMS support functions					*/
/*									*/
/************************************************************************/


#if	defined(DXB_HOST)
/*
 *	dxb_link:
 *
 *	Enqueue a link message
 */
int
dxb_link(DXB *dxb, int channel, int other_channel)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_LINK	msg;

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	if (dxb_bytes_free(q) < DXB_MSG_LINK_SIZE)
		return (DXB_ENOSPACE);

	msg.type = DXB_TYPE_LINK;
	msg.channel = channel;
	msg.other_channel = other_channel;
	msg.ctrl_len = 0;
	msg.data_len = 0;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_LINK_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_linkack:
 *
 *	Enqueue a link acknowledgement message
 */
int
dxb_linkack(DXB *dxb, int channel, int mux_id, int error_code)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_LINK	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;
	int		rc;
	uint32		in_tmp ;

	if (1) dxb_printf("dxb_linkack(%d, %d, %d)\n",
		channel, mux_id, error_code);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, 0, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_LINK;
	msg.channel = channel;
	msg.ctrl_len = ctrl_len;
	msg.data_len = 0;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_LINK_SIZE, dxb);

	ctrl[0] = error_code ? -1 : mux_id;
	ctrl[1] = error_code;

	q->in = dxb_copy_to_buf(&q->buf, in_tmp, ctrl, ctrl_len, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	dxb_unlink:
 *
 *	Enqueue an UNLINK message
 */
int
dxb_unlink(DXB *dxb, int channel, int mux_id)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_UNLINK	msg;

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	if (dxb_bytes_free(q) < DXB_MSG_UNLINK_SIZE)
		return (DXB_ENOSPACE);

	msg.type = DXB_TYPE_UNLINK;
	msg.channel = channel;
	msg.mux_id = mux_id;
	msg.ctrl_len = 0;
	msg.data_len = 0;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_UNLINK_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_unlinkack:
 *
 *	Enqueue a UNLINK acknowledgement message
 */
int
dxb_unlinkack(DXB *dxb, int channel, int mux_id, int error_code)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_UNLINK	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;
	uint32		in_tmp ;
	int		rc;

	if (1) dxb_printf("dxb_unlinkack(%d, %d, %d)\n",
		channel, mux_id, error_code);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, 0, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_UNLINK;
	msg.channel = channel;
	msg.mux_id = mux_id;
	msg.ctrl_len = ctrl_len;
	msg.data_len = 0;

	in_tmp = 
		dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_UNLINK_SIZE, dxb);

	ctrl[0] = error_code ? -1 : 0;
	ctrl[1] = error_code;

	q->in = dxb_copy_to_buf(&q->buf, in_tmp, ctrl, ctrl_len, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	dxb_push:
 *
 *	Enqueue a push message.  "channel" is the channel number of
 *	the already opened stream.  "module_id" is the name of the
 *	module to be pused onto the channel.
 *
 *	Returns 0 if AOK.  Returns DXB_EPERM if wrong side of DXB interface.
 *	Returns DXB_ENOSPACE if no space for message.
 */
int
dxb_push(DXB *dxb, int channel, const char *module_id,
	const void *data, int data_len)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_PUSH	msg;
	int		rc;
	uint32		in_tmp ;
	int		ctrl_len = (int) (strlen(module_id) + 1);

	if (1) dxb_printf("dxb_push(%d, %s, %x, %d)\n",
		channel, module_id, (int) data, data_len);

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, data_len, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Construct message header
	 */
	msg.type = DXB_TYPE_PUSH;
	msg.channel = channel;
	msg.ctrl_len = ctrl_len;
	msg.data_len = data_len;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_PUSH_SIZE, dxb);

	/*
	 *	Copy in control portion
	 */
	in_tmp = dxb_copy_to_buf(&q->buf, in_tmp, module_id, ctrl_len, dxb);
	in_tmp = roundup_and_wrap(in_tmp, q);

	/*
	 *	Copy in data portion
	 */
	if (data_len)
	{
		in_tmp = dxb_copy_to_buf(&q->buf, in_tmp, data, data_len, dxb);
		in_tmp = roundup_and_wrap(in_tmp, q);
	}

	q->in = in_tmp;
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_pushack:
 *
 *	Enqueue a push acknowledgement message
 */
int
dxb_pushack(DXB	*dxb, int channel, int error_code)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_PUSH	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;
	uint32		in_tmp ;
	int		rc;

	if (1) dxb_printf("dxb_pushack(%x, %d, %d)\n",
		(int) dxb, channel, error_code);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, 0, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_PUSH;
	msg.channel = channel;
	msg.ctrl_len = ctrl_len;
	msg.data_len = 0;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_PUSH_SIZE, dxb);

	ctrl[0] = error_code ? -1 : 0;
	ctrl[1] = error_code;

	q->in = dxb_copy_to_buf(&q->buf, in_tmp, ctrl, ctrl_len, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	dxb_pop:
 *
 *	Enqueue a POP message
 */
int
dxb_pop(DXB *dxb, int channel)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_POP	msg;

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	if (dxb_bytes_free(q) < DXB_MSG_POP_SIZE)
		return (DXB_ENOSPACE);

	msg.type = DXB_TYPE_POP;
	msg.channel = channel;
	msg.ctrl_len = 0;
	msg.data_len = 0;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_POP_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_popack:
 *
 *	Enqueue a POP acknowledgement message
 */
int
dxb_popack(DXB *dxb, int channel, int error_code)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_POP	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;
	uint32		in_tmp ;
	int		rc;

	if (1) dxb_printf("dxb_popack(%d, %d)\n",
		channel, error_code);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, 0, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_POP;
	msg.channel = channel;
	msg.ctrl_len = ctrl_len;
	msg.data_len = 0;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_POP_SIZE, dxb);

	ctrl[0] = error_code ? -1 : 0;
	ctrl[1] = error_code;

	q->in = dxb_copy_to_buf(&q->buf, in_tmp, ctrl, ctrl_len, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	dxb_flush:
 *
 *	Enqueue a flush message
 */
int
dxb_flush(DXB *dxb, int channel, int flush_read, int flush_write)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_FLUSH	msg;

	/*
	 *	Only the openside (host) can call this routine
	 */
	if (!dxb->openside)
		return (DXB_EPERM);

	if (dxb_bytes_free(q) < DXB_MSG_FLUSH_SIZE)
		return (DXB_ENOSPACE);

	msg.type = DXB_TYPE_FLUSH;
	msg.channel = channel;
	msg.flush_read = flush_read;
	msg.flush_write = flush_write;
	msg.ctrl_len = 0;
	msg.data_len = 0;

	q->in = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_FLUSH_SIZE, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_flushack:
 *
 *	Enqueue a flush acknowledgement message
 */
int
dxb_flushack(DXB *dxb, int channel, int error_code)
{
	DXBQ		*q = dxb->outq;
	DXB_MSG_FLUSH	msg;
	dxb_uint16	ctrl[2];
	int		ctrl_len = 4;
	uint32		in_tmp ;
	int		rc;

	if (1) dxb_printf("dxb_flushack(%x, %d, %d)\n",
		(int) dxb, channel, error_code);

	/*
	 *	Only the slaveside (board) can call this routine
	 */
	if (dxb->openside)
		return (DXB_EPERM);

	/*
	 *	See if there is space to send this message.  Since it is
	 *	a high priority message we can lie about the channel #.
	 */
	rc = dxb_space_avail(dxb, 0, ctrl_len, 0, DXB_PRI_HIGH);
	if (rc) return (rc);

	/*
	 *	Construct message and send it
	 */
	msg.type = DXB_TYPE_FLUSH;
	msg.channel = channel;
	msg.ctrl_len = ctrl_len;
	msg.data_len = 0;

	in_tmp = dxb_copy_to_buf(&q->buf, q->in, &msg, DXB_MSG_FLUSH_SIZE, dxb);

	ctrl[0] = error_code ? -1 : 0;
	ctrl[1] = error_code;

	q->in = dxb_copy_to_buf(&q->buf, in_tmp, ctrl, ctrl_len, dxb);
	dxb_syncwrite(&q->in, sizeof(q->in), dxb->syncdata);

	return (0);
}
#endif	/* DXB_ADAPTER */


/************************************************************************/
/*									*/
/*	API functions used when rxbuf option is desired			*/
/*									*/
/************************************************************************/

/*
 *	Internal routine to (possible) allocate an rxbuf ring buffer
 *	and init the buffer variables.
 */
static int
init_rxbuf(DXBCHAN *chan, uint8 *buf, int buf_size)
{
	/* 
	 *	The doc says that buf_size must be a power of two.
	 * 	Let's enforce that early on to prevent programmers from
	 * 	hurting themselves.
	 */
	if (buf_size & (buf_size - 1))
	{
		return (DXB_EINVAL);
	}

	/*
	 *	Allocate the structure itself
	 */
	chan->rxbuf = (DXB_RXBUF *) dxb_malloc(sizeof(DXB_RXBUF));
	if (!chan->rxbuf)
		return (DXB_ENOMEM);

	/*
	 *	If the user didn't give us a buffer, malloc one now
	 */
	if (buf == NULL)
	{
		buf = (uint8 *) dxb_malloc(buf_size);
		if (!buf)
		{
			dxb_free(chan->rxbuf);
			chan->rxbuf = NULL;
			return (DXB_ENOMEM);
		}
		chan->rxbuf->buf = buf;
		chan->rxbuf->free = 1;
	}
	else
	{
#if defined(LINUX)
		/* do explicit memcpy for linux */
		memcpy(&chan->rxbuf->buf, &buf, sizeof(buf));
#else
		chan->rxbuf->buf = buf;
#endif
		chan->rxbuf->free = 0;
	}

	chan->rxbuf->size = buf_size;
	chan->rxbuf->mask = buf_size - 1;
	chan->rxbuf->in = 0;
	chan->rxbuf->out = 0;

	return (0);
}

/*
 *	Internal routine to (possible) free an rxbuf ring buffer
 *	and free the rxbuf structure.
 */
static void
uninit_rxbuf(DXBCHAN *chan)
{
	if (chan->rxbuf == NULL)
		return;

	if (chan->rxbuf->buf && chan->rxbuf->free)
		dxb_free(chan->rxbuf->buf);

	dxb_free(chan->rxbuf);
	chan->rxbuf = NULL;
}


#if	defined(DXB_HOST)
/*
 *	dxb_rxbuf_open:
 *
 *	Just like dxb_open(), except that the user can ask for the
 *	DXB library to manage a circular buffer for data payload
 *	which the user is free to read at his convenience.  This
 *	is a common need for channels with unbounded data, like
 *	async.
 *
 *	"buf_size" is the size of the circular buffer to be used,
 *	which should have some relationship to the number of credits
 *	which will be supplied on this channel (i.e. at least credits+1).
 *	"buf_size" must be a power of two in length.
 *
 *	"buf" is user supplied storage for the circular buffer.  If
 *	it is NULL, the library will use dxb_malloc() to allocate
 *	the space for a buffer of size "buf_size".
 *
 *	The user will get the usual callback when DATA or CDATA messages
 *	are received.  When a CDATA callback occurs, the user should use
 *	dxb_read_ctrl_block() to read the control portion.  In the callback,
 *	the user can use dxb_read_data_block() or dxb_rxbuf_copyin() to
 *	process the received data message.  Use dxb_rxbuf_copyin()
 *	to cause the data to be copied to the allocated rxbuf for
 *	processing at a later time.  Use dxb_read_data_block() to bypass
 *	the rxbuf circular buffer if you are going to handle the data
 *	immediately.  A mix of the two methods may be optimal for some
 *	drivers to avoid the extra copy which occurs when using rxbuf.
 *
 *	If the user has set a non-zero fixed credit cost, then the DATA
 *	or CDATA callbacks should probably call dxb_return_fixed_cost()
 *	to immediately return the fixed cost.  The cost of the bytes
 *	should be credited only after the user has actually used
 *	dxb_rxbuf_getchar() or dxb_rxbuf_read() to remove the data
 *	from the circular buffer.
 *
 *	Returns the same values as for dxb_open().
 */
int
dxb_rxbuf_open(DXB *dxb, char *devid, int fixed_cost, void *data, int data_len,
		uint8 *buf, int buf_size)
{
	int	channel;
	int	rc;
	DXBCHAN	*chan;

	channel = dxb_open(dxb, devid, fixed_cost, data, data_len);
	if (channel < 0) return (channel);

	chan = &dxb->chan[channel];

	/*
	 *	Possibly allocate a buffer and then init it
	 */
	rc = init_rxbuf(chan, buf, buf_size);
	if (rc)
	{
		/* FIXME: close channel */
		return (rc);
	}

	/*
	 *	Clear the copy function so it defaults to memcpy
	 */
	chan->rxbuf->copyout = NULL;

	return (channel);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	dxb_rxbuf_openack():
 *
 *	Just like dxb_openack(), but with the rxbuf option as described
 *	under dxb_rxbuf_open() above.
 *
 *	However, you can get an ENOMEM error return if "error_code" is 0.
 *	If this happens, it is the users responsibility to call
 *	dxb_opennak() with the ENOMEM error code.
 *
 */
int
dxb_rxbuf_openack(DXB *dxb, int channel, int fixed_cost, int error_code,
		uint8 *buf, int buf_size)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	int		rc;
	DXB_RXBUF	*rxbuf;

	/*
	 *	Possibly allocate a buffer and then init it
	 */
	if (error_code == 0)
	{
		rc = init_rxbuf(chan, buf, buf_size);
		if (rc) return (rc);
	}

	/*
	 *	call the normal openack routine
	 */
	rxbuf = chan->rxbuf;	/* save it */

	rc = dxb_openack(dxb, channel, fixed_cost, error_code);
	if (rc != 0)
	{
		uninit_rxbuf(chan);
		return (rc);
	}

	if (error_code)
		return (0);

	chan->rxbuf = rxbuf;	/* restore it */

	/*
	 *	Clear the copy function so it defaults to memcpy
	 */
	chan->rxbuf->copyout = NULL;

	return (0);
}
#endif	/* DXB_ADAPTER */


/*
 *	dxb_rxbuf_copyin:
 *
 *	Copy the data portion of a DXB message into the rxbuf
 *	circular buffer.  Typically called from the data callback.
 *
 *	Returns DXB_ENOSPACE if there is no space to do the copy,
 *	and the copy isn't done.  Else returns 0;
 */
int
dxb_rxbuf_copyin(DXB *dxb, int channel, int data_len)
{
	DXB_RXBUF	*rxbuf = dxb->chan[channel].rxbuf;
	DXBQ		*q = dxb->inq;
	int		in = rxbuf->in;
	int		out = rxbuf->out;
	int		space = (out - in - 1) & rxbuf->mask;
	uint32		out_tmp = OUT_TMP(q);

	if (space < data_len)
		return (DXB_ENOSPACE);

	if (out > in)
	{
		/*
		 *	can't wrap: copy the only part
		 */
		out_tmp = dxb_copy_from_buf(rxbuf->buf + in,
			data_len, &q->buf, out_tmp, data_len);

		rxbuf->in = in + data_len;
	}
	else
	{
		int	contigspace = rxbuf->size - in;
		int	amt = MYMIN(data_len, contigspace);
		int	left = data_len;

		/*
		 *	copy first part
		 */
		out_tmp = dxb_copy_from_buf(rxbuf->buf + in,
			amt, &q->buf, out_tmp, amt);

		left -= amt;
		if (left)
		{
			/*
			 *	copy second part
			 */
			out_tmp = dxb_copy_from_buf(rxbuf->buf + 0,
				left, &q->buf, out_tmp, left);
			rxbuf->in = left;
		}
		else
			rxbuf->in = (rxbuf->in + amt) & rxbuf->mask;
	}

	OUT_TMP(q) = out_tmp;

	return (0);
}

/*
 *	dxb_rxbuf_copyin_with_header:	TODO
 *
 */
int
dxb_rxbuf_copyin_with_header(DXB *dxb, int channel, int data_len,
			void *header, int header_len, int header_align)
{
	return (DXB_EINVAL);
}

/*
 *	dxb_rxbuf_getchar:
 *
 *	Get the next byte out of the rx buffer or return -1 if there
 *	are no more bytes in the buffer.
 */
int
dxb_rxbuf_getchar(DXB *dxb, int channel)
{
	DXB_RXBUF	*rxbuf = dxb->chan[channel].rxbuf;
	int		ch;

	if (rxbuf->in == rxbuf->out)
		return -1;

	ch = rxbuf->buf[rxbuf->out++];
	rxbuf->out &= rxbuf->mask;

	return (ch);
}

/*
 *	dxb_rxbuf_count:
 *
 *	Return the number of bytes available in the rx buffer
 */
int
dxb_rxbuf_count(DXB *dxb, int channel)
{
	DXB_RXBUF	*rxbuf = dxb->chan[channel].rxbuf;

	return (rxbuf->in - rxbuf->out) & rxbuf->mask;
}

/*
 *	dxb_rxbuf_read:
 *
 *	Read data out of the rxbuf circular buffer for "channel".
 *	Put it into "buf".  Limit read to "buf_len" bytes.
 *
 *	Returns the number of bytes actually copied.
 *
 *	If the copyout function is not set, 1 or 2 calls to memcpy
 *	are used to move the data.  Else, the copyout function is
 *	called 1 or 2 times to move the data.
 */
int
dxb_rxbuf_read(DXB *dxb, int channel, void *buf, int buf_len)
{
	DXB_RXBUF	*rxbuf = dxb->chan[channel].rxbuf;
	int		in = rxbuf->in;
	int		out = rxbuf->out;
	int		len;

	len = (in - out) & rxbuf->mask;
	len = MYMIN(len, buf_len);
	if (len == 0)
		return (0);

	/*
	 * Copy the data in one or two parts, depending on whether the
	 * buffer wraps.
	 */
	if (in >= out)
	{
		/*
		 *	copy the only part
		 */
		if (rxbuf->copyout)
			(*rxbuf->copyout)(buf, rxbuf->buf + out, len,
						rxbuf->copyout_arg);
		else
			memcpy(buf, rxbuf->buf + out, len);

		rxbuf->out = out + len;
	}
	else
	{
		int	contiglen = rxbuf->mask - out + 1;
		int	amt = MYMIN(len, contiglen);
		int	left = len;

		/*
		 *	copy first part
		 */
		if (rxbuf->copyout)
		{
			buf = (*rxbuf->copyout)(buf, rxbuf->buf + out, amt,
						rxbuf->copyout_arg);
		}
		else
		{
			memcpy(buf, rxbuf->buf + out, amt);
			buf = (uint8 *) buf + amt;
		}

		left -= amt;
		if (left)
		{
			/*
			 *	copy second part
			 */
			if (rxbuf->copyout)
				(*rxbuf->copyout)(buf, rxbuf->buf + 0, left,
							rxbuf->copyout_arg);
			else
				memcpy(buf, rxbuf->buf + 0, left);

			rxbuf->out = left;
		}
		else
			rxbuf->out = (rxbuf->out + amt) & rxbuf->mask;
	}

	return (len);
}

/*
 *	dxb_rxbuf_flush:
 *
 *	Flush all data out of the rx buffer.  Return the number of bytes
 *	that were flushed.
 */
int
dxb_rxbuf_flush(DXB *dxb, int channel)
{
	DXB_RXBUF	*rxbuf = dxb->chan[channel].rxbuf;
	int		in = rxbuf->in;
	int		amt;

	amt =  (in - rxbuf->out) & rxbuf->mask;
	rxbuf->out = in;

	return (amt);
}

/*
 *	dxb_rxbuf_set_copyout:
 *
 *	Set the copyout function used to copy data out of the rx buffer.
 *	If "copyout" is NULL, then memcpy() is used to copy the data.
 *	You would use a copyout function if you wanted to massage the
 *	data as it was copied to some other place.
 */
int
dxb_rxbuf_set_copyout(DXB *dxb, int channel, DXB_COPYFUNC copyout, DXB_ARG arg)
{
	DXB_RXBUF	*rxbuf = dxb->chan[channel].rxbuf;

	rxbuf->copyout = copyout;
	rxbuf->copyout_arg = arg;

	return (0);
}

/************************************************************************/
/*									*/
/*	API functions used when dxbuf option is desired			*/
/*									*/
/************************************************************************/

/*
 * Notes by Steve on the dxbuf implementation:
 *
 * Once dxb_dxbuf_advance is called, the caller is free to return the
 * credits for that space.  The original implementation kept the message
 * header in the ring buffer, even when the callback routine consumed
 * partial messages, as opposed to whole messages.  It's kind of hard
 * to explain, but if the caller consumes half of a data message and
 * returns the credits for that half-message to the host, those credits
 * say that the space occupied by the header is eligible to be re-written
 * by the host.  This is a losing proposition.
 *
 * The header is now pulled out of the circ. buffer into a structure
 * that persists accross calls to the callback routines.
 *
 * Also note that the dxbuf->out pointer must follow any partial
 * message consumption indicated by dxb_dxbuf_advance(), since the
 * callback routine may return credits for the partial message consumed.
 *
 * However, it is especially important that the "user" does not return
 * credits for the data portion of the message before completely
 * consuming the control portion, because that would give the host
 * permission to write over the control portion that has not yet been read.
 * This is not a problem right now because the only currently known user
 * of dxbufs (Tigger/Hercules S/DXB) does not use messages that contain
 * both control and data portions.
 *
 * The only way I can think of to solve this problem would be to add
 * another layer to the credit mechanism, and return only those credits
 * that wouldn't allow the host to write past the out pointer, holding
 * on to the rest of those credits until the out pointer advances.
 * Not gonna do this now, because like I said, it's not a problem
 * for current use of dxbufs.
 */

/*
 *	Internal routine to (possibly) allocate an dxbuf ring buffer
 *	and init the buffer variables.
 */
static int
init_dxbuf(DXBCHAN *chan, uint8 *buf, int buf_size)
{
	/* 
	 *	The doc says that buf_size must be a power of two.
	 * 	Let's enforce that early on to prevent programmers from
	 * 	hurting themselves.
	 */
	if (buf_size & (buf_size - 1))
	{
		return (DXB_EINVAL);
	}

	/*
	 *	Allocate the structure itself
	 */
	chan->dxbuf = (DXB_DXBUF *) dxb_malloc(sizeof(DXB_DXBUF));
	if (!chan->dxbuf)
		return (DXB_ENOMEM);

	/*
	 *	If the user didn't give us a buffer, malloc one now
	 */
	if (buf == NULL)
	{
		buf = (uint8 *) dxb_malloc(buf_size);
		if (!buf)
		{
			dxb_free(chan->dxbuf);
			chan->dxbuf = NULL;
			return (DXB_ENOMEM);
		}
		chan->dxbuf->buf = buf;
		chan->dxbuf->free = 1;
	}
	else
	{
		chan->dxbuf->buf = buf;
		chan->dxbuf->free = 0;
	}

	chan->dxbuf->size = buf_size;
	chan->dxbuf->mask = buf_size - 1;
	chan->dxbuf->in = 0;
	chan->dxbuf->out = 0;

	chan->dxbuf->ctrl.s = NULL;
	chan->dxbuf->data.s = NULL;

	chan->dxbuf->msgvalid = 0 ;

	return (0);
}

/*
 *	Internal routine to (possible) free an dxbuf ring buffer
 *	and free the dxbuf structure.
 */
static void
uninit_dxbuf(DXBCHAN *chan)
{
	if (chan->dxbuf == NULL)
		return;

	if (chan->dxbuf->buf && chan->dxbuf->free)
		dxb_free(chan->dxbuf->buf);

	dxb_free(chan->dxbuf);
	chan->dxbuf = NULL;
}


#if	defined(DXB_HOST)
/*
 *	dxb_dxbuf_open(), dxb_dxbuf_openack()
 *	
 *	Like dxb_rxbuf_open{*}(), except:
 *
 *	1) A permanent fixed cost of 14 (largest DXB message header plus
 *	   two potential alignments) is automatic. "fixed_cost" is any
 *	   additional cost the user wants to add to any messages sent to us.
 *
 *	2) "buf_size" is the size in bytes of the buffer "buf" supplied,
 *	   or the buffer that is dxb_malloc'ed if "buf" is NULL.  One
 *	   less than that is the credits that are supplied.  There are
 *	   no restrictions, per se, in the library on the size of this
 *	   buffer.
 *
 *	3) You still get the normal callback on every DXB message received.
 *	   This allows you to process messages with no cost (such as
 *	   link, push, pop, etc.) as well as priority cdata messages
 *	   immediately.  These cannot be stored in the local DXB
 *	   circular buffer -- since they have no cost they could
 *	   overflow the storage provided.  Obviously, this limits
 *	   the design of protocols that can use this buffering feature.
 *
 *	4) If you get an ioctl, data, or cdata (with normal priority)
 *	   callback, you need to call dxb_dxbuf_copyin() to copy it
 *	   to the per channel buffer.  The library doesn't do it
 *	   for you automatically because there may be an opportunity
 *	   for you to optimize the case where the dxbuf is empty.
 *	   This way, you have the control to do that, if desired.
 *
 *	5) The actual number of credits sent to the "openside" will
 *	   be buf_size - 4.
 */
int
dxb_dxbuf_open(DXB *dxb, char *devid, int fixed_cost, void *data, int data_len,
		uint8 *buf, int buf_size)
{
	int	channel;
	int	rc;
	DXBCHAN	*chan;

	channel = dxb_open(dxb, devid,
		DXB_MSG_GENERIC_SIZE + 3 + 3 + fixed_cost, data, data_len);
	if (channel < 0) return (channel);

	chan = &dxb->chan[channel];

	/*
	 *	Possibly allocate a buffer and then init it
	 */
	rc = init_dxbuf(chan, buf, buf_size);
	if (rc)
	{
		/* FIXME: close channel */
		return (rc);
	}

	/*
	 *	we automatically return the initial credit allocation.
	 */
 	dxb_return_credits(dxb, channel, buf_size - 4);

	return (channel);
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
int
dxb_dxbuf_openack(DXB *dxb, int channel, int fixed_cost, int error_code,
		uint8 *buf, int buf_size)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	int		rc;
	DXB_DXBUF	*dxbuf;

	/*
	 *	Possibly allocate a buffer and then init it
	 */
	if (error_code == 0)
	{
		rc = init_dxbuf(chan, buf, buf_size);
		if (rc) return (rc);
	}

	/*
	 *	call the normal openack routine
	 */
	dxbuf = chan->dxbuf;	/* save it */

	rc = dxb_openack(dxb, channel,
		DXB_MSG_GENERIC_SIZE + 3 + 3 + fixed_cost, error_code);
	if (rc != 0)
	{
		uninit_dxbuf(chan);
		return (rc);
	}

	if (error_code)
		return (0);

	chan->dxbuf = dxbuf;	/* restore it */

	/*
	 *	we automatically return the initial credit allocation.
	 */
 	dxb_return_credits(dxb, channel, buf_size - 4);

	return (0);
}
#endif	/* DXB_ADAPTER */


/*
 *      dxb_dxbuf_copyin:
 *
 *	Copy the entire *current DXB message* from the main DXB
 *	circular buffer to the per channel circular buffer (for
 *	channels opened with the dxbuf option).  This defers the
 *	message for later processing.
 *
 *	Typically called from the users data, cdata, or ioctl callbacks
 *	(which themselves were called from dxb_rxpoll_one()).  This
 *	routine should be called only on the first callback for a
 *	DXB message when (channel & DXB_DXBUF_CB) == 0 in the callback
 *	routine.
 *
 *      Returns DXB_ENOSPACE if there is no space to do the copy,
 *      and the copy isn't done.  Returns DXB_EINVAL if a copy of
 *	an illegal message type (such as a link message) is attempted.
 *	Else returns 0;
 *
 *	NOTE: if you get DXB_ENOSPACE, something has gone wrong with
 *	the credits mechanism.  This error indicates a fault in your
 *	code.
 */
int
dxb_dxbuf_copyin(DXB *dxb, int channel)
{
	DXB_MSG_GENERIC	*msgp;
	DXBQ		*q = dxb->inq;
	DXB_DXBUF	*dxbuf = dxb->chan[channel].dxbuf;
	int		in = dxbuf->in;
	int		out = dxbuf->out;
	int		space = (out - in - 1) & dxbuf->mask;
	int		len;
	uint32		qout = OUT(q);

	if (dxbuf == NULL)
		return DXB_EINVAL;

	msgp = (DXB_MSG_GENERIC *) dxb_buf_vaddr(&q->buf, qout);

	switch (msgp->type)
	{
	case DXB_TYPE_DATA:
		{
			DXB_MSG_DATA	msg;

			dxb_copy_from_buf(&msg, DXB_MSG_DATA_SIZE,
				&q->buf, qout, DXB_MSG_DATA_SIZE);
			len = DXB_MSG_DATA_SIZE + roundup4(msg.data_len);
		}
		break;

	case DXB_TYPE_CDATA:
		{
			DXB_MSG_CDATA	msg;

			dxb_copy_from_buf(&msg, DXB_MSG_CDATA_SIZE,
				&q->buf, qout, DXB_MSG_CDATA_SIZE);

			if (msg.priority)
				return DXB_EINVAL;

			len = DXB_MSG_CDATA_SIZE + roundup4(msg.data_len)
				+ roundup4(msg.ctrl_len);
		}
		break;

	case DXB_TYPE_IOCTL:
		{
			DXB_MSG_IOCTL	msg;

			dxb_copy_from_buf(&msg, DXB_MSG_IOCTL_SIZE,
				&q->buf, qout, DXB_MSG_IOCTL_SIZE);
			len = DXB_MSG_IOCTL_SIZE + roundup4(msg.data_len)
				+ roundup4(msg.ctrl_len);
		}
		break;

	default:
		return DXB_EINVAL;
	}

	/*
	 *	OK, we know how much to copy. 
	 */

	if (space < len)
	{
		/* 
		 * If this copy won't fit, we know it's becuase we've issued
		 * too many credits to the host.  Perhaps the return of fixed
		 * credits associated with 'lastpiece' wasn't obeyed.
		 */
		dxb_printf("Error: dxblib user has botched credits\n");
		return (DXB_ENOSPACE);
	}

	if (out > in)
	{
		/*
		 *	can't wrap: copy the only part
		 */
		dxb_copy_from_buf(dxbuf->buf + in, len, &q->buf, qout, len);

		dxbuf->in = in + len;
	}
	else
	{
		int	contigspace = dxbuf->size - in;
		int	amt = MYMIN(len, contigspace);
		int	left = len;

		/*
		 *	copy first part
		 */
		qout = dxb_copy_from_buf(dxbuf->buf + in,
						amt, &q->buf, qout, amt);

		left -= amt;
		if (left)
		{
			/*
			 *	copy second part
			 */
			dxb_copy_from_buf(dxbuf->buf + 0,
				left, &q->buf, qout, left);
			dxbuf->in = left;
		}
		else
			dxbuf->in = (dxbuf->in + amt) & dxbuf->mask;
	}

	return (0);
}

/*
 *	dxb_dxbuf_copy_from_buf()
 *
 *	Internal(?) routine to copy a portion of a dxbuf
 *	to a non-wrapping buffer, handling dxbuf buffer wraping.
 *	Very similar to dxb_copy_from_buf; the returned offset
 *	will reflect the fact that we pretended to copy "buf_len"
 *	bytes out of the buffer, even though we will only physically
 *	copy up to "data_size" bytes to the destination buffer.
 *
 *	returns the new buffer offset.
 */
static int
dxb_dxbuf_copy_from_buf(void *data, uint data_size,
			const DXB_DXBUF *dxbuf, int offset, uint buf_len)
{
	uint  len ;
	uint8 *data8 = (uint8 *) data ;	/* compiler fodder */
	uint  partial ;
	int   posn = offset ;

	/*
	 *	copy as much as we have, or have space for
	 */
	len = MYMIN(data_size, buf_len);

	/* number of bytes to buffer wrap point */
	partial = dxbuf->size - posn ;

	if(len > partial) {
		/*
		 * buffer wrap condition
		 *   Copy first part, and set up to copy second part
		 */
		memcpy(data8, dxbuf->buf + posn, partial) ;
		posn = 0 ;
		len -= partial ;
		data8 += partial ;
	}

	memcpy(data8, dxbuf->buf + posn, len) ;

	return (offset + buf_len) & dxbuf->mask ;
}
	

/*
 *      dxb_dxbuf_poll_one:
 *
 *      Processes a single received message from the per-channel
 *	dxb buffer (for channels opened with dxbuf option).  This
 *	routine must be called periodically to process messages
 *	from the buffer which have accumulated.
 *
 *      If an error occurs due to an invalid command recieved, the
 *      routine will return DXB_EINVAL and will stop processing
 *      further messages.
 *
 *	This routine returns 0 if there are no more messages to be
 *	processed, or if the current message was incompletely processed
 *	and the same message will be processed on the next call to
 *	dxb_dxbuf_poll_one.  Otherwise, 1 is returned.  The intent is:
 *	if 1 is returned, there is more usefull work to do right now;
 *	if 0 is returned, there is no usefull work at the moment (either
 *	waiting for input, or blocked on output).
 *
 *	This routine generates a 2nd (or possibly 3rd or Nth) callback
 *	for each DXB message.  To save space, we overload the data and cdata
 *	callbacks by ORing 'DXB_DXBUF_CB' into the channel number.
 *
 *      The typical usage is:
 *
 *              for (;;)
 *              {
 *                      DXB_LOCK;
 *                      rc = dxb_dxbuf_poll_one(dxb, channel);
 *                      DXB_UNLOCK;
 *                      if (rc <= 0) break;
 *              }
 *      
 */
static void
compute_dxbuf_areas(DXB_DXBUF *dxbuf, int hdr_len, int ctrl_len, int data_len)
{
	int	index;

	index = dxbuf->out + hdr_len;

	/*
	 *	Compute start and end pointers for control portion
	 */
	dxbuf->ctrl.s = dxbuf->buf + (index & dxbuf->mask);
	index += ctrl_len;
	dxbuf->ctrl.e = dxbuf->buf + (index & dxbuf->mask);
	index = roundup4(index);

	/*
	 *	Compute start and end pointers for control portion
	 */
	dxbuf->data.s = dxbuf->buf + (index & dxbuf->mask);
	index += data_len;
	dxbuf->data.e = dxbuf->buf + (index & dxbuf->mask);
}

int
dxb_dxbuf_poll_one(DXB *dxb, int channel)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;
	DXB_MSG_GENERIC	*msgp;

	if (dxbuf == NULL)
		return DXB_EINVAL;

	if (dxbuf->in == dxbuf->out)
		return 0;

	if(!dxbuf->msgvalid) {
		msgp = (DXB_MSG_GENERIC *) (dxbuf->buf + dxbuf->out);
		switch(msgp->type)
		{
		case DXB_TYPE_DATA:
			dxb_dxbuf_copy_from_buf(dxbuf->msg,
						DXB_MSG_DATA_SIZE,
						dxbuf,
						dxbuf->out,
						DXB_MSG_DATA_SIZE) ;
			/*
			 *	get pointers to data area if its the 1st time
			 *	we've seen this message in
			 *	dxb_dxbuf_poll_one().
			 */
			if (dxbuf->data.s == NULL) {
				DXB_MSG_DATA *msg = (DXB_MSG_DATA *)dxbuf->msg;
				compute_dxbuf_areas(dxbuf, DXB_MSG_DATA_SIZE,
						    0, msg->data_len);
			}
			dxbuf->msgvalid = 1 ;
			break ;

		case DXB_TYPE_CDATA:
			dxb_dxbuf_copy_from_buf(dxbuf->msg,
						DXB_MSG_CDATA_SIZE,
						dxbuf,
						dxbuf->out,
						DXB_MSG_CDATA_SIZE) ;
			/*
			 *	get pointers to data area if its the 1st time
			 *	we've seen this message in
			 *	dxb_dxbuf_poll_one().
			 */
			if (dxbuf->data.s == NULL) {
				DXB_MSG_CDATA *msg =
					(DXB_MSG_CDATA *)dxbuf->msg ;
				compute_dxbuf_areas(dxbuf, DXB_MSG_CDATA_SIZE,
						    msg->ctrl_len,
						    msg->data_len);
			}
			dxbuf->msgvalid = 1;
			break ;

		/* case DXB_TYPE_IOCTL: */

		default:
			return DXB_EINVAL ;
		}
	}

	msgp = (DXB_MSG_GENERIC *) dxbuf->msg ;

	switch (msgp->type)
	{
	case DXB_TYPE_DATA:
		{
		DXB_DATA_CB	cb = chan->data_cb;
		DXB_ARG		arg = chan->data_cb_arg;
		DXB_MSG_DATA	*pmsg;

		if (!cb)
		{
			dxb_printf("dxb_dxbuf_poll_one: no DATA_CB!\n");
			return DXB_ENOCB;
		}

		pmsg = (DXB_MSG_DATA *)dxbuf->msg ;

		/*
		 *	Call the callback routine
		 */
		dxb_printf("dxb_dxdata_cb(%d, %d, %x) data[0] = 0x%x\n",
			pmsg->channel, (int) pmsg->data_len, (int) arg,
			* (int *) dxbuf->data.s);

		(*cb)(dxb, pmsg->channel | DXB_DXBUF_CB, pmsg->data_len, arg);
		}
		break;

	case DXB_TYPE_CDATA:
		{
		DXB_CDATA_CB	cb = chan->cdata_cb;
		DXB_ARG		arg = chan->cdata_cb_arg;
		DXB_MSG_CDATA	*pmsg;

		if (!cb)
		{
			dxb_printf("dxb_dxbuf_poll_one: no CDATA_CB!\n");
			return DXB_ENOCB;
		}

		pmsg = (DXB_MSG_CDATA *)dxbuf->msg ;

		/*
		 *	Call the callback routine
		 */
		dxb_printf("dxb_dxcdata_cb(%d, %d, %d, %x) ctrl[0] = 0x%x\n",
			   pmsg->channel, (int)pmsg->ctrl_len,
			   (int) pmsg->data_len, (int) arg,
			   * (int *) dxbuf->ctrl.s);

		(*cb)(dxb, pmsg->channel | DXB_DXBUF_CB, pmsg->priority,
		      pmsg->ctrl_len, pmsg->data_len, arg);
		}
		break;


	case DXB_TYPE_IOCTL:
		break;

	default:
		return DXB_EINVAL;
	}

	/* msgvalid is true if the message was incompletely processed */
	/* this time around */
	return (dxbuf->in != dxbuf->out) && !dxbuf->msgvalid ;
}

/*
 *	dxb_dxbuf_ctrl_locations:
 *	dxb_dxbuf_data_locations:
 *
 *	Return the location(s) of the N pieces of the data or ctrl portion
 *	of the current DXB data or cdata message in the per channel dxbuf.
 *
 *	There could be multiple pieces because of buffer wrap and/or
 *	buffer fragmentation.  The piece information is returned in
 *	an array which the user allocates.  Currently, there is no
 *	mechanism in this library to allocate a fragmented buffer,
 *	so the user should always allocate an array of 2 DXB_PIECE's.
 *	
 */
int
dxb_dxbuf_locations(DXB_DXBUF *dxbuf, DXBSE *se,
				int *npieces, DXB_PIECE	*pieces)
{
	if (se->e == se->s)
	{
		*npieces = 0;
		pieces[0].len = 0;	/* For safety only */
		pieces[0].piece = 0;	/* For safety only */
		return (0);
	}
	else if (se->e > se->s)
	{
		*npieces = 1;
		pieces[0].len = se->e - se->s;
		pieces[0].piece = se->s;
		return (0);
	}
	else
	{
		*npieces = 2;
		pieces[0].len = &dxbuf->buf[dxbuf->size] - se->s;
		pieces[0].piece = se->s;
		pieces[1].len = se->e - dxbuf->buf;
		pieces[1].piece = dxbuf->buf;
		return (0);
	}
}

int
dxb_dxbuf_data_locations(DXB *dxb, int channel,
				int *npieces, DXB_PIECE	*pieces)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;

	return dxb_dxbuf_locations(dxbuf, &dxbuf->data, npieces, pieces);
}

int
dxb_dxbuf_ctrl_locations(DXB *dxb, int channel,
				int *npieces, DXB_PIECE	*pieces)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;

	return dxb_dxbuf_locations(dxbuf, &dxbuf->ctrl, npieces, pieces);
}

/*
 *	dxb_dxbuf_advance
 *
 *	Indicate to the library how much of the control and/or
 *	data portions of the current dxbuf DXB message have been
 *	consumed by the caller.  If ctrlp is NULL, it indicates
 *	that the entire control portion has been consumed.  If
 *	datap is NULL, it indicates that the entire data portion
 *	has been consumed.  If either is not NULL, they indicate
 *	the next location in the message that needs to be consumed.
 *	In this case, the library will remember this and you will
 *	get another callback for this message the next time that
 *	dxb_dxbuf_poll_one() is called.
 */
int
dxb_dxbuf_advance(DXB *dxb, int channel, void *ctrlp, void *datap)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;

	if (ctrlp == NULL)
		ctrlp = dxbuf->ctrl.e;
	dxbuf->ctrl.s = (uint8 *) ctrlp;

	if (datap == NULL)
		datap = dxbuf->data.e;
	dxbuf->data.s = (uint8 *) datap;

	if (dxbuf->data.s == dxbuf->data.e && dxbuf->ctrl.s == dxbuf->ctrl.e)
	{
		dxbuf->out = dxbuf->mask & roundup4(dxbuf->data.e - dxbuf->buf);
		/* Force compute_dxbuf_areas() to recalculate the */
		/* buffer info, and re-read the message header */
		dxbuf->data.s = NULL;
		dxbuf->msgvalid = 0 ;
	} else if(dxbuf->ctrl.s == dxbuf->ctrl.e) {
		/* control portion empty; advance out pointer */
		/* to data position -- round *down*, not up */
		dxbuf->out = dxbuf->mask & ~3 & (dxbuf->data.s - dxbuf->buf);
	} else {
		/* still something in the control portion; advance */
		/* to that position, again rounding down. */
		dxbuf->out = dxbuf->mask & ~3 & (dxbuf->ctrl.s - dxbuf->buf);
	}

	return (0);
}

int
dxb_dxbuf_int_advance(DXB *dxb, int channel, int ctrl_cnt, int data_cnt)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;
	int		so, eo; /* start and end offsets */
	int		count;

	/* ---------- control portion ---------- */

	so = dxbuf->ctrl.s - dxbuf->buf;
	eo = dxbuf->ctrl.e - dxbuf->buf;
	count = (eo - so) & dxbuf->mask;

	if( (ctrl_cnt == -1) || (ctrl_cnt > count) )
	{
		dxbuf->ctrl.s = dxbuf->ctrl.e;
	}
	else
	{
		dxbuf->ctrl.s = dxbuf->buf + ((so + ctrl_cnt) & dxbuf->mask);
	}

	/* ---------- data portion ---------- */

	so = dxbuf->data.s - dxbuf->buf;
	eo = dxbuf->data.s - dxbuf->buf;
	count = (eo - so) & dxbuf->mask;

	if( (data_cnt == -1) || (data_cnt > count) )
	{
		dxbuf->data.s = dxbuf->data.e;
	}
	else
	{
		dxbuf->data.s = dxbuf->buf + ((so + data_cnt) & dxbuf->mask);
	}
	
	if (dxbuf->data.s == dxbuf->data.e && dxbuf->ctrl.s == dxbuf->ctrl.e)
	{
		dxbuf->out = dxbuf->mask & roundup4(dxbuf->data.e - dxbuf->buf);
		/* Force compute_dxbuf_areas() to recalculate the */
		/* buffer info, and re-read the message header */
		dxbuf->data.s = NULL;
		dxbuf->msgvalid = 0 ;
	}

	return(0) ;
}    

/*
 *	dxb_dxbuf_read_ctrl_block:
 *	dxb_dxbuf_read_data_block:
 *
 *	Like dxb_read_*_block(), but takes its data from the
 *	per channel circular buffer.  These are use in lieu
 *	of dxb_dxbuf_*_locations() and dxb_dxbuf_advance().
 *
 *	It is possible and allowed to mix both calls, though.
 *	The typical use would be to *read* the control portion,
 *	and *advance* the data portion.
 */
int
dxb_dxbuf_read_ctrl_block(DXB *dxb, int channel,
			  void *buf, int buf_size, int ctrl_len)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;
	int		count ;
	int		rc = 0 ;
	int		so, eo ; /* start and end offsets */

	if(!dxbuf)
		return DXB_EINVAL ;
	
	/* what's the total length of the control portion? */
	so = dxbuf->ctrl.s - dxbuf->buf ;
	eo = dxbuf->ctrl.e - dxbuf->buf ;
	
	count = (eo - so) & dxbuf->mask ; 

	if(count < ctrl_len) {
		rc = DXB_EINVAL ;
		ctrl_len = count ;
	}

	so = dxb_dxbuf_copy_from_buf(buf, buf_size, dxbuf, so, ctrl_len) ;

	return rc ;
}

int
dxb_dxbuf_read_data_block(DXB *dxb, int channel,
			  void *buf, int buf_size, int data_len)
{
	DXBCHAN		*chan = &dxb->chan[channel];
	DXB_DXBUF	*dxbuf = chan->dxbuf;
	int		count ;
	int		rc = 0 ;
	int		so, eo ; /* start and end offsets */

	if(!dxbuf)
		return DXB_EINVAL ;
	
	/* what's the total length of the control portion? */
	so = dxbuf->data.s - dxbuf->buf ;
	eo = dxbuf->data.e - dxbuf->buf ;
	
	count = (eo - so) & dxbuf->mask ; 

	if(count < data_len) {
		rc = DXB_EINVAL ;
		data_len = count ;
	}

	so = dxb_dxbuf_copy_from_buf(buf, buf_size, dxbuf, so, data_len) ;

	return rc ;
}

/*
 *	dxb_dxbuf_empty
 *
 *	Returns true if there is no data queued in dxbuf
 */
int
dxb_dxbuf_empty(DXB *dxb, int channel)
{
	DXB_DXBUF	*dxbuf = dxb->chan[channel].dxbuf;

	if (dxbuf == NULL)
		return 0 ;

	return (dxbuf->in == dxbuf->out);
}


/************************************************************************/
/*                                                                      */
/*      API Receive side functions and poller				*/
/*                                                                      */
/************************************************************************/

/*
 *	dxb_read_data_block:
 *
 *	Read part or all of the control or data portion of a message.
 *
 *	"buf_size" is the size of the buffer to put it in.  "data_len"
 *	is the amount of data to be handled.  If buf_size is less than
 *	data_len, only buf_size bytes will be moved, and the rest will
 *	be skipped.
 *
 *	Always returns 0; Honest.
 *
 *	Typical use is from DXB_DATA_CB, DXB_CDATA_CB, DXB_IOCTL_CB,
 *	or DXB_IOCTLACK_CB callback routines:
 *
 *	int
 *	data_cb(DXB *dxb, int channel, int data_len, DXB_ARG action)
 *	{
 *		char    data[512];
 *
 *		dxb_read_data_block(dxb, data, sizeof(data), data_len);
 *		dxb_read_done(dxb);
 *		// ... do something with data[] ...
 *		dxb_return_fixed_cost(dxb, channel, 1);
 *		dxb_return_credits(dxb, channel, data_len);
 *		return (0);
 *	}
 *
 *	int
 *	cdata_cb(DXB *dxb, int channel, int priority,
 *			int ctrl_len, int data_len, DXB_ARG action)
 *	{
 *		char    ctrl[512];
 *		char    data[512];
 *
 *		dxb_read_ctrl_block(dxb, ctrl, sizeof(ctrl), ctrl_len);
 *		dxb_read_data_block(dxb, data, sizeof(data), data_len);
 *		dxb_read_done(dxb);
 *		// ... do something with ctrl[], data[] ...
 *		if (!priority)
 *		{
 *			dxb_return_fixed_cost(dxb, channel, 1);
 *			dxb_return_credits(dxb, channel, ctrl_len + data_len);
 *		}
 *		return (0);
 *	}
 */
int
dxb_read_ctrl_block(DXB *dxb, void *buf, int buf_size, int ctrl_len)
{
	DXBQ		*q = dxb->inq;
	int		cnt;
	int		rc = 0;
	uint32		out_tmp = OUT_TMP(q);

	/*
	 *	Figure out what the new control count is, and check to
	 *	see if the user is trying to read more control data than
	 *	is available.
	 */
	cnt = CTRL_CNT(q) + ctrl_len;
	if (cnt > CTRL_LEN(q))
	{
		ctrl_len = CTRL_LEN(q) - CTRL_CNT(q);
		cnt = CTRL_CNT(q);
		rc = DXB_EINVAL;
	}

	/*
	 *	Copy the data to the user's buffer
	 */
	out_tmp = dxb_copy_from_buf(buf, buf_size,
				&q->buf, out_tmp, ctrl_len);

	CTRL_CNT(q) = cnt;

	if (cnt == CTRL_LEN(q))
		out_tmp = roundup_and_wrap(out_tmp, q);

	OUT_TMP(q) = out_tmp;

	return (rc);
}

void *
dxb_peek_ctrl_read(DXB *dxb, int *max)
{
	DXBQ		*q = dxb->inq;
	int		page ;
	int		pageoffset ;
	uint32		out_tmp = OUT_TMP(q);

	page = out_tmp >> q->buf.pageshift ;
	pageoffset = out_tmp & q->buf.pagemask ;
	*max = MYMIN(CTRL_LEN(q) - CTRL_CNT(q), (int) q->buf.pagesize - pageoffset) ;

	return ((void *) (q->buf.page[page].virt + pageoffset)) ;
}

int
dxb_read_data_block(DXB *dxb, void *buf, int buf_size, int data_len)
{
	DXBQ		*q = dxb->inq;
	int		cnt;
	int		rc = 0;
	uint32		out_tmp = OUT_TMP(q);

	/*
	 *	Insure that all the control data has been read
	 */
	if (CTRL_CNT(q) != CTRL_LEN(q))
	{
		out_tmp = dxb_copy_from_buf(NULL, 0, &q->buf, out_tmp,
				CTRL_LEN(q) - CTRL_CNT(q));
		out_tmp = roundup_and_wrap(out_tmp, q);
		CTRL_CNT(q) = CTRL_LEN(q);
		rc = DXB_EINVAL;
	}

	/*
	 *	Figure out what the final data count will be, and check to
	 *	see if the user is trying to read more data than available.
	 */
	cnt = DATA_CNT(q) + data_len;
	if (cnt > DATA_LEN(q))
	{
		data_len = DATA_LEN(q) - DATA_CNT(q);
		cnt = DATA_CNT(q);
		rc = DXB_EINVAL;
	}

	/*
	 *	Copy the data to the user's buffer
	 */
	out_tmp = dxb_copy_from_buf(buf, buf_size,
				&q->buf, out_tmp, data_len);

	DATA_CNT(q) = cnt;

	if (cnt == DATA_LEN(q))
		out_tmp = roundup_and_wrap(out_tmp, q);

	OUT_TMP(q) = out_tmp;

	return (rc);
}

void *
dxb_peek_data_read(DXB *dxb, int *max)
{
	DXBQ		*q = dxb->inq;
	int		page ;
	int		pageoffset ;
	uint32		out_tmp = OUT_TMP(q);

	if (CTRL_CNT(q) < CTRL_LEN(q)) {
		*max = 0 ;
		return ((void *) 0) ;
	}

	page = out_tmp >> q->buf.pageshift ;
	pageoffset = out_tmp & q->buf.pagemask ;
	*max = MYMIN(DATA_LEN(q) - DATA_CNT(q), (int) q->buf.pagesize - pageoffset) ;

	return ((void *) (q->buf.page[page].virt + pageoffset)) ;
}

int
dxb_peek_rconfirm(DXB *dxb, int count)
{
	DXBQ		*q = dxb->inq;
	int amt ;
	uint32	out_tmp = OUT_TMP(q);

	if (CTRL_CNT(q) < CTRL_LEN(q)) {
		amt = MYMIN(CTRL_LEN(q) - CTRL_CNT(q), count) ;

		CTRL_CNT(q) += amt ;
		count -= amt ;
		out_tmp = (out_tmp + amt) & q->buf.mask ;

		if (CTRL_LEN(q) == CTRL_CNT(q))
			out_tmp = roundup_and_wrap(out_tmp, q) ;
	}

	if (DATA_CNT(q) < DATA_LEN(q)) {
		amt = MYMIN(DATA_LEN(q) - DATA_CNT(q), count) ;

		DATA_CNT(q) += amt ;
		count -= amt ;
		out_tmp = (out_tmp + amt) & q->buf.mask ;

		if (DATA_LEN(q) == DATA_CNT(q))
			out_tmp = roundup_and_wrap(out_tmp, q) ;
	}

	OUT_TMP(q) = out_tmp;

	if (count)
		return DXB_EACCES ;

	return 0 ;
}

/*
 *	dxb_read_done:
 *
 *	Indicate that the caller has read all of the data and it is safe
 *	to advance the out pointer.
 *
 *	Always returns 0; Honest.
 */
int
dxb_read_done(DXB *dxb)
{
	DXBQ		*q = dxb->inq;

	OUT_TMP(q) = roundup_and_wrap(OUT_TMP(q), q);

	/* q->out will actually be updated in the poller */
	
	return (0);
}


#if	defined(DXB_ADAPTER)
static void
process_open(DXB *dxb, DXBQ *q)
{
	DXB_MSG_OPEN	msg;
	char		devid[16];
	DXB_OPEN_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_OPEN_SIZE,
				&q->buf, OUT(q), DXB_MSG_OPEN_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(devid, sizeof(devid),
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

	/*
	 *	Save his fixed cost information and zero credits
	 */
	dxb->chan[msg.channel].credits = 0;
	dxb->chan[msg.channel].credits_owed = 0;
	dxb->chan[msg.channel].credit_return_level = 0;
	dxb->chan[msg.channel].his_fixed_cost = msg.fixed_cost;

	/*
	 *	Save info so dxb_read_data_block()
	 *	and dxb_read_done() will work properly.
	 */
	DATA_LEN(q) = msg.data_len;
	DATA_CNT(q) = 0;
	CTRL_LEN(q) = msg.ctrl_len;
	CTRL_CNT(q) = msg.ctrl_len;

	/*
	 *	zero all the callbacks
	 */
	clear_callbacks(&dxb->chan[msg.channel]);

	/*
	 *	Call the callback routine
	 */
	cb = dxb->open_cb;
	arg = dxb->open_cb_arg;
	if (cb)
	{
		dxb_printf("dxb_open_cb(%d, %s, %d, %x) data[0] = %d\n",
			msg.channel, devid, (int) msg.data_len, (int) arg,
			dxb_data0(&q->buf, out_tmp) );
			
		(*cb)(dxb, msg.channel, devid, msg.data_len, arg);
	}

	/*
	 *	Regardless of whether or not the user read any data,
	 *	advance the q->out_tmp pointer the proper amount.
	 */
	out_tmp = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;
	OUT_TMP(q) = out_tmp;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	Process a received OPEN acknowledgement
 */
static void
process_openack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_OPEN	msg;
	short		ctrl[2];
	DXBCHAN		*chan;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_OPEN_SIZE,
				&q->buf, OUT(q), DXB_MSG_OPEN_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);

	/*
	 *	Now skip past any data portion
	 */
	out_tmp = dxb_copy_from_buf(NULL, 0,
				&q->buf, out_tmp, msg.data_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		return;
#endif

	if (1) dxb_printf("process_openack (%d)\n", msg.channel);

	/*
	 *	Save his fixed cost information
	 */
	chan = &dxb->chan[msg.channel];
	chan->his_fixed_cost = msg.fixed_cost;

	/*
	 *	Save info so dxb_read_data_block()
	 *	and dxb_read_done() will work properly.
	 */
	DATA_LEN(q) = msg.data_len;
	DATA_CNT(q) = 0;
	CTRL_LEN(q) = msg.ctrl_len;
	CTRL_CNT(q) = msg.ctrl_len;

	/*
	 *	Call the callback routine
	 */
	if (chan->openack_cb)
	{
		dxb_printf("dxb_openack_cb(%d, %d, %d)\n",
			msg.channel,
			ctrl[0] == -1 ? ctrl[1] : 0,
			(int) chan->openack_cb_arg);

		(*chan->openack_cb)(dxb, msg.channel,
			ctrl[0] == -1 ? ctrl[1] : 0, chan->openack_cb_arg);
	}

	/*
	 *	If there was an error, "close" the channel
	 */
	if (ctrl[0] == -1)
	{
		GET_OPENMAP_LOCK(dxb);
		clear_openmap_bit(dxb->openmap, msg.channel);
		find_openmap_limits(dxb);
		FREE_OPENMAP_LOCK(dxb);

		chan->credits_owed = 0;

		/*
		 *	If we are using the rxbuf or dxbuf options, free them
		 */
		uninit_rxbuf(chan);
		uninit_dxbuf(chan);
	}

	return;
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
/*
 *	Process a received CLOSE message
 */
static void
process_close(DXB *dxb, DXBQ *q)
{
	DXB_MSG_CLOSE	msg;
	DXB_CLOSE_CB	cb;
	DXB_ARG		arg;
	DXBCHAN		*chan;

	/*
	 *	Get the whole message header
	 */
	OUT_TMP(q) = dxb_copy_from_buf(&msg, DXB_MSG_CLOSE_SIZE,
				&q->buf, OUT(q), DXB_MSG_CLOSE_SIZE);

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		return;
#endif

	if (1) dxb_printf("process_close(%d)\n", msg.channel);

	/*
	 *	Call the callback routine.  If there is no callback
	 *	routine, we attempt to send the closeack (once) on behalf
	 *	of the user.
	 */
	chan = &dxb->chan[msg.channel];
	cb = chan->close_cb;
	arg = chan->close_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, arg);
	else
	{
		dxb_closeack(dxb, msg.channel);
	}

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
/*
 *	Process a received CLOSE acknowledgement message
 */
static void
process_closeack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_CLOSE	msg;
	DXB_CLOSEACK_CB	cb;
	DXB_ARG		arg;
	DXBCHAN		*chan;

	/*
	 *	Get the whole message header
	 */
	OUT_TMP(q) = dxb_copy_from_buf(&msg, DXB_MSG_CLOSE_SIZE,
				&q->buf, OUT(q), DXB_MSG_CLOSE_SIZE);

	if (1) dxb_printf("process_closeack(%d)\n", msg.channel);

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		return;
#endif

	/*
	 *	Call the callback routine
	 */
	chan = &dxb->chan[msg.channel];
	cb = chan->closeack_cb;
	arg = chan->closeack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, arg);

	/*
	 *	Mark channel closed
	 */
	GET_OPENMAP_LOCK(dxb);
	clear_openmap_bit(dxb->openmap, msg.channel);
	find_openmap_limits(dxb);
	FREE_OPENMAP_LOCK(dxb);

	chan->credits_owed = 0;

	/*
	 *	If we are using the rxbuf or dxbuf options, free them
	 */
	uninit_rxbuf(chan);
	uninit_dxbuf(chan);

	/*
	 *	zero all the callbacks
	 */
	clear_callbacks(chan);

	return;
}
#endif	/* DXB_HOST */


static void
process_data(DXB *dxb, DXBQ *q)
{
	DXB_MSG_DATA	msg;
	DXB_DATA_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_DATA_SIZE,
				&q->buf, OUT(q), DXB_MSG_DATA_SIZE);

	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	/*
	 *	Save info so dxb_read_data_block()
	 *	and dxb_read_done() will work properly.
	 */
	DATA_LEN(q) = msg.data_len;
	DATA_CNT(q) = 0;
	CTRL_LEN(q) = 0;
	CTRL_CNT(q) = 0;

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].data_cb;
	arg = dxb->chan[msg.channel].data_cb_arg;
	if (cb)
	{
		dxb_printf("dxb_data_cb(%d, %d, %x) data[0] = 0x%x\n",
			msg.channel, (int) msg.data_len, (int) arg,
			dxb_data0(&q->buf, out_tmp) );

		(*cb)(dxb, msg.channel, msg.data_len, arg);
	}

	/*
	 *	Regardless of whether or not the user read any data,
	 *	advance the q->out_tmp pointer the proper amount.
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}

static void
process_cdata(DXB *dxb, DXBQ *q)
{
	DXB_MSG_CDATA	msg;
	DXB_CDATA_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	if (1) dxb_printf("process_cdata\n");

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_CDATA_SIZE,
				&q->buf, OUT(q), DXB_MSG_CDATA_SIZE);

	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	/*
	 *	Save info so dxb_read_data_block()
	 *	and dxb_read_done() will work properly.
	 */
	DATA_LEN(q) = msg.data_len;
	DATA_CNT(q) = 0;
	CTRL_LEN(q) = msg.ctrl_len;
	CTRL_CNT(q) = 0;

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].cdata_cb;
	arg = dxb->chan[msg.channel].cdata_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.priority,
				msg.ctrl_len, msg.data_len, arg);

	/*
	 *	Regardless of whether or not the user read any data,
	 *	advance the q->out_tmp pointer the proper amount.
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len) + roundup4(msg.ctrl_len))
				& q->buf.mask;

	return;
}


/*
 *	Process a received credit award message
 */
static void
process_credit(DXB *dxb, DXBQ *q)
{
	DXB_MSG_CREDIT	msg;
	DXBCHAN		*chan;

	/*
	 *	Get the whole message header
	 */
	OUT_TMP(q) = dxb_copy_from_buf(&msg, DXB_MSG_CREDIT_SIZE,
				&q->buf, OUT(q), DXB_MSG_CREDIT_SIZE);

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		return;
#endif

	chan = &dxb->chan[msg.channel];
	chan->credits += msg.credit_award;
	chan->need_credit_cb = 1;

	if (1) dxb_printf("process_credit %d credits\n",
		(int) msg.credit_award);

#if 0	/* Now done from txpoll */

	/*
	 *	Call the callback routine
	 */
	if (chan->credit_cb)
	{
		(*chan->credit_cb)(dxb, msg.channel,
			chan->credits, chan->credit_cb_arg);
	}
#endif

	return;
}


#if	defined(DXB_ADAPTER)
static void
process_push(DXB *dxb, DXBQ *q)
{
	DXB_MSG_PUSH	msg;
	char		devid[16];
	DXB_PUSH_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_PUSH_SIZE,
				&q->buf, OUT(q), DXB_MSG_PUSH_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(devid, sizeof(devid),
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_push\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].push_cb;
	arg = dxb->chan[msg.channel].push_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, devid, msg.data_len, arg);

	/*
	 *	Regardless of whether or not the user read any data,
	 *	advance the q->out_tmp pointer the proper amount.
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
static void
process_pushack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_PUSH	msg;
	DXB_PUSHACK_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;
	short		ctrl[2];

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_PUSH_SIZE,
				&q->buf, OUT(q), DXB_MSG_PUSH_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_pushack\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].pushack_cb;
	arg = dxb->chan[msg.channel].pushack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, ctrl[0] == -1 ? ctrl[1] : 0, arg);

	/*
	 *	Data length is supposed to be zero, but skip any data
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
static void
process_pop(DXB *dxb, DXBQ *q)
{
	DXB_MSG_POP	msg;
	DXB_POP_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_POP_SIZE,
				&q->buf, OUT(q), DXB_MSG_POP_SIZE);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_pop\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].pop_cb;
	arg = dxb->chan[msg.channel].pop_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, arg);

	/*
	 *	Data & control len are supposed to be 0, but skip if not
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)
			+ roundup4(msg.ctrl_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
static void
process_popack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_POP	msg;
	DXB_POPACK_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;
	short		ctrl[2];

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_POP_SIZE,
				&q->buf, OUT(q), DXB_MSG_POP_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_popack\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].popack_cb;
	arg = dxb->chan[msg.channel].popack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, ctrl[0] == -1 ? ctrl[1] : 0, arg);

	/*
	 *	Data length is supposed to be zero, but skip any data
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
static void
process_link(DXB *dxb, DXBQ *q)
{
	DXB_MSG_LINK	msg;
	DXB_LINK_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_LINK_SIZE,
				&q->buf, OUT(q), DXB_MSG_LINK_SIZE);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_link\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].link_cb;
	arg = dxb->chan[msg.channel].link_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.other_channel, arg);

	/*
	 *	Data & control len are supposed to be 0, but skip if not
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)
			+ roundup4(msg.ctrl_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
static void
process_linkack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_LINK	msg;
	DXB_LINKACK_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;
	short		ctrl[2];

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_LINK_SIZE,
				&q->buf, OUT(q), DXB_MSG_LINK_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_linkack\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].linkack_cb;
	arg = dxb->chan[msg.channel].linkack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, ctrl[0],
				ctrl[0] == -1 ? ctrl[1] : 0, arg);

	/*
	 *	Data length is supposed to be zero, but skip any data
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
static void
process_unlink(DXB *dxb, DXBQ *q)
{
	DXB_MSG_UNLINK	msg;
	DXB_UNLINK_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_UNLINK_SIZE,
				&q->buf, OUT(q), DXB_MSG_UNLINK_SIZE);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_unlink\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].unlink_cb;
	arg = dxb->chan[msg.channel].unlink_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.mux_id, arg);

	/*
	 *	Data & control len are supposed to be 0, but skip if not
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)
			+ roundup4(msg.ctrl_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
static void
process_unlinkack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_UNLINK	msg;
	DXB_UNLINKACK_CB cb;
	DXB_ARG		arg;
	uint32		out_tmp;
	short		ctrl[2];

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_UNLINK_SIZE,
				&q->buf, OUT(q), DXB_MSG_UNLINK_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_unlinkack\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].unlinkack_cb;
	arg = dxb->chan[msg.channel].unlinkack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.mux_id,
				ctrl[0] == -1 ? ctrl[1] : 0, arg);

	/*
	 *	Data length is supposed to be zero, but skip any data
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
static void
process_flush(DXB *dxb, DXBQ *q)
{
	DXB_MSG_FLUSH	msg;
	DXB_FLUSH_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_FLUSH_SIZE,
				&q->buf, OUT(q), DXB_MSG_FLUSH_SIZE);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_flush\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].flush_cb;
	arg = dxb->chan[msg.channel].flush_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.flush_read, msg.flush_write, arg);

	/*
	 *	Data & control len are supposed to be 0, but skip if not
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)
			+ roundup4(msg.ctrl_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
static void
process_flushack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_FLUSH	msg;
	DXB_FLUSHACK_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;
	short		ctrl[2];

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_FLUSH_SIZE,
				&q->buf, OUT(q), DXB_MSG_FLUSH_SIZE);

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);
	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_flushack\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].flushack_cb;
	arg = dxb->chan[msg.channel].flushack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, ctrl[0] == -1 ? ctrl[1] : 0, arg);

	/*
	 *	Data length is supposed to be zero, but skip any data
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_HOST */


#if	defined(DXB_ADAPTER)
static void
process_ioctl(DXB *dxb, DXBQ *q)
{
	DXB_MSG_IOCTL	msg;
	DXB_IOCTL_CB	cb;
	DXB_ARG		arg;
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_IOCTL_SIZE,
				&q->buf, OUT(q), DXB_MSG_IOCTL_SIZE);

	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_ioctl\n");

	/*
	 *	Save info so dxb_read_data_block()
	 *	and dxb_read_done() will work properly.
	 */
	DATA_LEN(q) = msg.data_len;
	DATA_CNT(q) = 0;
	CTRL_LEN(q) = 0;
	CTRL_CNT(q) = 0;

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].ioctl_cb;
	arg = dxb->chan[msg.channel].ioctl_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.command, msg.data_len, arg);

	/*
	 *	Regardless of whether or not the user read any data,
	 *	advance the q->out_tmp pointer the proper amount.
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)
			+ roundup4(msg.ctrl_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_ADAPTER */


#if	defined(DXB_HOST)
static void
process_ioctlack(DXB *dxb, DXBQ *q)
{
	DXB_MSG_IOCTL	msg;
	DXB_IOCTLACK_CB	cb;
	DXB_ARG		arg;
	short		ctrl[2];
	uint32		out_tmp;

	/*
	 *	Get the whole message header
	 */
	out_tmp = dxb_copy_from_buf(&msg, DXB_MSG_IOCTL_SIZE,
				&q->buf, OUT(q), DXB_MSG_IOCTL_SIZE);

	/*
	 *	Save info so dxb_read_data_block()
	 *	and dxb_read_done() will work properly.
	 */
	DATA_LEN(q) = msg.data_len;
	DATA_CNT(q) = 0;
	CTRL_LEN(q) = 0;
	CTRL_CNT(q) = 0;

	/*
	 *	Now get the control portion
	 */
	out_tmp = dxb_copy_from_buf(ctrl, 4,
				&q->buf, out_tmp, msg.ctrl_len);
	out_tmp = roundup_and_wrap(out_tmp, q);

	OUT_TMP(q) = out_tmp;

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		goto skip;
#endif

	if (1) dxb_printf("process_ioctlack\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].ioctlack_cb;
	arg = dxb->chan[msg.channel].ioctlack_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.command, msg.data_len,
			ctrl[0] == -1 ? ctrl[1] : 0, arg);

	/*
	 *	Regardless of whether or not the user read any data,
	 *	advance the q->out_tmp pointer the proper amount.
	 */
#ifdef	DXB_MP
skip:
#endif
	OUT_TMP(q) = (out_tmp + roundup4(msg.data_len)) & q->buf.mask;

	return;
}
#endif	/* DXB_HOST */


static void
process_cancel(DXB *dxb, DXBQ *q)
{
	DXB_MSG_CANCEL	msg;
	DXB_CANCEL_CB	cb;
	DXB_ARG		arg;

	/*
	 *	Get the whole message header
	 */
	OUT_TMP(q) = dxb_copy_from_buf(&msg, DXB_MSG_CANCEL_SIZE,
				&q->buf, OUT(q), DXB_MSG_CANCEL_SIZE);

#ifdef	DXB_MP
	if (CHECK_ASSIGN(msg.channel, CPUNumber) == 0)
		return;
#endif

	if (1) dxb_printf("process_cancel\n");

	/*
	 *	Call the callback routine
	 */
	cb = dxb->chan[msg.channel].cancel_cb;
	arg = dxb->chan[msg.channel].cancel_cb_arg;
	if (cb)
		(*cb)(dxb, msg.channel, msg.command, arg);

	return;
}


/*
 *	Internal routine called by dxb_rxpoll_{one,all}().
 */
int
dxb_rxpoll_internal(DXB *dxb, DXBQ *q)
{
	DXB_MSG_GENERIC	*msgp;
	uint32	qout = OUT(q);
	uint32	qin = q->in;

	/*
	 *	Process one incoming message.
	 */
	if (qin >= q->buf.size || qin & 3)
	{
		dxb_printf("dxb_rxpoll: q->in=%x\n", qin);
#ifdef	DXB_MP
		dxb->poll_error = CPUNumber + 1;
#else
		dxb->poll_error = 1;
#endif
		return DXB_EINVAL;
	}

	if (qout >= q->buf.size || qout & 3)
	{
		dxb_printf("dxb_rxpoll: q->out=%x\n", qout);
#ifdef	DXB_MP
		dxb->poll_error = CPUNumber + 1;
#else
		dxb->poll_error = 1;
#endif
		return DXB_EINVAL;
	}

	/* 
	 * invalidate cache for the entire read buffer
	 */
	dxb_syncread(q->buf.page[0].virt, q->buf.size, dxb->syncdata); 

	msgp = (DXB_MSG_GENERIC *) dxb_buf_vaddr(&q->buf, qout);

	switch (msgp->type)
	{
	case DXB_TYPE_OPEN:
		PROCESS_SIDE(process_openack, process_open, (dxb, q));
		break;

	case DXB_TYPE_CLOSE:
		PROCESS_SIDE(process_closeack, process_close, (dxb, q));
		break;

	case DXB_TYPE_DATA:
		process_data(dxb, q);
		break;

	case DXB_TYPE_CDATA:
		process_cdata(dxb, q);
		break;

	case DXB_TYPE_START:
		/* Deprecated???? */
		break;

	case DXB_TYPE_CREDIT:
		process_credit(dxb, q);
		break;

	case DXB_TYPE_CANCEL:
		process_cancel(dxb, q);
		break;

	case DXB_TYPE_LINK:
		PROCESS_SIDE(process_linkack, process_link, (dxb, q));
		break;

	case DXB_TYPE_UNLINK:
		PROCESS_SIDE(process_unlinkack, process_unlink, (dxb, q));
		break;

	case DXB_TYPE_PUSH:
		PROCESS_SIDE(process_pushack, process_push, (dxb, q));
		break;

	case DXB_TYPE_POP:
		PROCESS_SIDE(process_popack, process_pop, (dxb, q));
		break;

	case DXB_TYPE_FLUSH:
		PROCESS_SIDE(process_flushack, process_flush, (dxb, q));
		break;

	case DXB_TYPE_IOCTL:
		PROCESS_SIDE(process_ioctlack, process_ioctl, (dxb, q));
		break;

	default:
		dxb_printf("Unknown DXB message 0x%x in=%x out=%x\n",
			msgp->type, qin, qout);
#ifdef	DXB_MP
		dxb->poll_error = CPUNumber + 1;
#else
		dxb->poll_error = 1;
#endif
		return DXB_EINVAL;
	}

	/*
	 *	Advance index permanently
	 */
	OUT(q) = OUT_TMP(q);
	dxb_syncwrite(&OUT(q), sizeof(OUT(q)), dxb->syncdata);

	return (0);
}


#ifndef	DXB_MP

/*
 *      dxb_rxpoll_one:
 *
 *      Processes a single received message.  This routine is
 *      called by dxb_rxpoll_all().  It is exposed to the user
 *      of the DXB library to allow fine control on how many
 *      DXB messages are processed on each poll.
 *
 *      All DXB library callbacks except for the credit_cb are
 *      initiated from this routine.
 *
 *      If an error occurs due to an invalid command recieved, the
 *      routine will return DXB_EINVAL and will stop processing
 *      further messages.
 *
 *	This routine returns 1 if more messages remain to be processed
 *	and 0 if there are no more messages to be processed.
 *
 *      If you chose to use the lower level dxb_rxpoll_one() and
 *      friends, the typical usage is:
 *
 *              for (;;)
 *              {
 *                      DXB_LOCK;
 *                      rc = dxb_rxpoll_one(dxb);
 *                      DXB_UNLOCK;
 *                      if (rc <= 0) break;
 *              }
 *              DXB_LOCK;
 *              dxb_txpoll_all();
 *              DXB_UNLOCK;
 */
int
dxb_rxpoll_one(DXB *dxb)
{
	DXBQ	*q = dxb->inq;
	int	rc;

	/*
	 *	If we got an error in this poller, we will stop
	 *	processing any more messages, giving the user a chance
	 *	to debug the fault.  This is a catastrophic situation.
	 */
	if (dxb->poll_error)
		return 0;	/* Only error on first occurrance */

	/*
	 *	Process all incoming messages
	 */
	dxb_syncread(&q->in, sizeof(q->in), dxb->syncdata);

	if (q->in == q->out)
		return (0);

	rc = dxb_rxpoll_internal(dxb, q);
	if (rc)
		return (rc);
	else
		return (1);
}

/*
 *      dxb_rxpoll_all:
 *
 *      Processes all received messages.  Repeatedly calls
 *      dxb_rxpoll_one() until received message queue is
 *      empty.  Any errors from that routine are propogated
 *      back to the caller.
 */
int
dxb_rxpoll_all(DXB *dxb)
{
	DXBQ	*q = dxb->inq;
	int	rc;

	/*
	 *	If we got an error in this poller, we will stop
	 *	processing any more messages, giving the user a chance
	 *	to debug the fault.  This is a catastrophic situation.
	 */
	if (dxb->poll_error)
		return 0;	/* Only error on first occurrance */

	/*
	 *	Process all incoming messages
	 */
	dxb_syncread(&q->in, sizeof(q->in), dxb->syncdata);

	while (q->in != q->out)
	{
		rc = dxb_rxpoll_internal(dxb, q);
		if (rc) return (rc);
	}

	return (0);
}
#endif	/* DXB_MP */


/*
 *      dxb_txpoll_all:
 *
 *      Must be called periodically to perform housekeeping
 *      chores on each DXB channel, such as generation of
 *      credit messages and opennak messages.  We also do
 *      the credit_cb callback for credit returns here, so
 *      they are scheduled fairly.
 */
void
dxb_txpoll_internal(DXB *dxb, int channel, DXBCHAN *chan)
{
#if	defined(DXB_ADAPTER)
	if (chan->nak_error_code)
		dxb_opennak(dxb, channel, chan->nak_error_code);
#endif

	if (chan->credits_owed)
		dxb_credit_award(dxb, channel, chan->credits_owed);

	if (chan->need_credit_cb
		|| (chan->credit_cb_on_space
			&& dxb_data_payload_space(dxb, channel)
				>= chan->credit_cb_on_space))
	{
		chan->need_credit_cb = 0;

		if (chan->credit_cb)
			(*chan->credit_cb)(dxb, channel,
					chan->credits, chan->credit_cb_arg);
	}
}

void
dxb_txpoll_all(DXB *dxb)
{
	int	i;
	int	channel;
	DXBCHAN	*chan;
#ifdef	DXB_MP
	DXB_MPSTATE	*mps = &(dxb->mpstate[CPUNumber]);
#endif

	/*
	 *	Loop thru each channel using fair algorithm.
	 */
	channel = dxb->first_channel;
	chan = dxb->chan + channel;
	for (i = dxb->min_channel; i <= dxb->max_channel; ++i)
	{
#ifdef	DXB_MP
		if (mps->assigned_channels[channel >> 5]
			& (1 << (channel & 31)))
#endif
		dxb_txpoll_internal(dxb, channel, chan);

		if (++channel > dxb->max_channel)
		{
			channel = dxb->min_channel;
			chan = dxb->chan + channel;
		}
		else
			++chan;
	}

#ifdef	DXB_MP
	if (CPUNumber == (CPUCount - 1))
#endif
	if (--dxb->first_channel < dxb->min_channel)
		dxb->first_channel = dxb->max_channel;
}

#ifndef	DXB_MP
/*
 *	dxb_poll:
 *
 *	Poll routine.  Must be called at the poll rate, which
 *	is typically every 10 msecs or as fast as you can stand it.
 *	Processes received message and does housekeeping chores
 *	like generation of transmitted credit and nak messages.
 *
 *	This is a convenience routine that simply calls
 *	dxb_rxpoll_all() and dxb_txpoll_all().  Any errors
 *	from those routines are propogated back to the caller.
 *
 *	Some library users may prefer the finer control of directly
 *	calling dxb_rxpoll_all(), dxb_rxpoll_one(), and dxb_txpoll_all().
 *
 *	Typical use:
 *		timer_tick()
 *		{
 *			DXB_LOCK;
 *			dxb_poll(&Dxb);
 *			DXB_UNLOCK;
 *		}
 */
int
dxb_poll(DXB *dxb)
{
	int	rc;

	/*
	 *	If we got an error in this poller, we will stop
	 *	processing any more messages, giving the user a chance
	 *	to debug the fault.  This is a catastrophic situation.
	 */
	if (dxb->poll_error)
		return 0;	/* Only error on first occurrance */

	/*
	 *	Process all incoming messages
	 */
	rc = dxb_rxpoll_all(dxb);
	if (rc) return (rc);

	/*
	 *	Generate outgoing messages
	 */
	dxb_txpoll_all(dxb);

	return (0);
}
#endif


/************************************************************************/
/*                                                                      */
/*      Initialization and cleanup					*/
/*                                                                      */
/************************************************************************/

/*
 *	dxb_dxbbuf_parms:
 *
 *	Given a DXBBUF buf pointer with buf->npages and buf->pagesize
 *	filled in, compute the buf->pagemask, buf->pageshift, buf->size,
 *	buf->mask, and buf->mask4 paramters which are used in the DXB
 *	library for performance reasons.
 *
 *	This routine is exposed external to the DXB library so that it
 *	may be called by the adapter side when it receives the physical
 *	buffer information at initialization time.
 */
void
dxb_dxbbuf_parms(DXBBUF *buf)
{
	buf->pagemask = buf->pagesize - 1;
	buf->pageshift = rightmost_one(buf->pagesize);
	buf->size = buf->npages * buf->pagesize;
	buf->mask = buf->size - 1;
	buf->mask4 = buf->mask & ~3;
}

/*
 *	Allocate space for a single DXB buffer
 *
 *	This routine is prepared to allocate either a single virtually
 *	contiguous buffer, or a buffer allocated in virtually
 *	discontinuous pieces.  We allow this because some operating
 *	systems may have trouble allocating large hunks of virtually
 *	contiguous space.  Using the most pessimistic assumptions
 *	of 120 ports, 8000 bytes/sec/port, 4x compression, and a 50HZ
 *	polling rate, we need 76800 bytes/buffer.  Rounding up to the
 *	next power of two makes it a 128kb buffer.
 *	
 */
static int
dxb_buf_alloc(DXBBUF *buf, int npages, int virtpagesize, int physpagesize)
{
	int	i;

	buf->npages = npages;
	buf->pagesize = virtpagesize;

	dxb_dxbbuf_parms(buf);

	if (npages == 1)
	{
		/*
		 *	We can allocate a single virtually contiguous buffer
		 *
		 *	Our abstraction of memory allocation assumes we can
		 *	get memory, but not necessarily aligned on a
		 *	physical page boundary.  We will allocate more
		 *	than we need, then find the page boundary.  But,
		 *	we also need to keep the original buffer address
		 *	around so we can later free it when the driver
		 *	shuts down.  As a somewaht gross hack, we'll stash
		 *	the original address in page[1], which won't otherwise
		 *	be used.
		 */
		uint8	*virt;
		long	addr;

		virt = (uint8 *) dxb_malloc(buf->size + physpagesize);
		if (!virt)
			return (-1);
		addr = (long) virt;
		addr += physpagesize - 1;
		addr &= ~(physpagesize - 1);
		buf->page[0].virt = (uint8 *) addr;
		buf->page[1].virt = virt;
	}
	else
	{
		/*
		 *	We must allocate a virtually discontinuous buffer
		 */
		for (i = 0; i < npages; ++i)
		{
			uint8	*virt;

			virt = (uint8 *) dxb_alloc_page();
			if (!virt)
				return (-1);
			buf->page[i].virt = virt;
		}
	}

	dxb_printf("dxb_buf: page[0].virt = %x\n", (int) buf->page[0].virt);

	return (0);
}

void
dxb_buf_free(DXBBUF *buf)
{
	uint	i;

	if (buf->npages == 1)
	{
		/*
		 *	Single buffer.  page[1] is not an error: we
		 *	need to free the original unaligned buffer
		 */
		if (buf->page[1].virt)
			dxb_free(buf->page[1].virt);
	}
	else
	{
		/*
		 *	Fragmented buffer composed of aligned pages
		 */
		for (i = 0; i < buf->npages; ++i)
		{
			if (buf->page[i].virt)
				dxb_free_page(buf->page[i].virt);
		}
	}

	/*
	 *	Insure we don't free anything twice if this routine is
	 *	called twice during driver unloading/error exit.
	 */
	buf->npages = 0;
}

/*
 *	dxb_uninit:	Uninitialize a DXB instance.  Free resources.
 */
void
dxb_uninit(DXB	*dxb)
{
	dxb_printf("dxb_uninit: freeing resources\n");

	dxb_buf_free(&dxb->a2h.buf);

	dxb_buf_free(&dxb->h2a.buf);

}

/*
 *	Conditionally swap a long.
 *
 *	Assumes there is a local variable called "doswap".
 */
#define	DXB_SWAPLONG(X)	( doswap ? dxb_swaplong(X) : (X) )

/*
 *	dxb_buf_copy:	Copy (potentially byteswapped) DXBBUF. 
 *
 * 	This is an internal helper function.  Do a member-by-member
 *	copy of a DXBBUF, swapping things as we go if doswap is set.
 * 			
 */
static void
dxb_buf_copy(DXBBUF *dst, const DXBBUF *src, uint doswap)
{
	unsigned int i;

	dst->npages	= DXB_SWAPLONG(src->npages);
	dst->pagesize	= DXB_SWAPLONG(src->pagesize);

	dst->pagemask	= DXB_SWAPLONG(src->pagemask);
	dst->pageshift	= DXB_SWAPLONG(src->pageshift);

	dst->size	= DXB_SWAPLONG(src->size);
	dst->mask	= DXB_SWAPLONG(src->mask);
	dst->mask4	= DXB_SWAPLONG(src->mask4);

	for (i = 0 ; i <= dst->npages ; i++)
	{
		dst->page[i].bus  = DXB_SWAPLONG(src->page[i].bus);
	}

}

/*
 *	dxb_phys_copy:	Copy the dxbphys structure
 *
 *	This routine is called by the board to copy the DXBPHYS structure
 *	in from the host, potentially byteswapping as we go.  This is
 *	typically used by the SLAVESIDE (adapter), before using the DXBPHYS
 *	structure that the OPENSIDE (host) has passed to it.
 */
void
dxb_phys_copy(DXBPHYS *dst, const DXBPHYS *src, uint doswap)
{
	/* 
	 * This militant approach will enforce people to visit this
	 * function when adding or changing members of the dxb physical 
	 * structures.  Any elements that aren't explicitly swapped
	 * will get zeroed, which hopefully will cause some kind of fault.
 	 */
	memset(dst, 0x0, sizeof(*dst));

	/* Copy and swap physical queue descriptors */

	dst->qphys.host2adapter_in  = DXB_SWAPLONG(src->qphys.host2adapter_in);
	dst->qphys.host2adapter_out = DXB_SWAPLONG(src->qphys.host2adapter_out);
	dst->qphys.adapter2host_in  = DXB_SWAPLONG(src->qphys.adapter2host_in);
	dst->qphys.adapter2host_out = DXB_SWAPLONG(src->qphys.adapter2host_out);

	dxb_buf_copy(&dst->host2adapter_q, &src->host2adapter_q, doswap);
	dxb_buf_copy(&dst->adapter2host_q, &src->adapter2host_q, doswap);
}


#if	defined(DXB_HOST)
/*
 *	dxb_set_phys:	Fill in the phys structure
 *
 *	This routine is called by the host, after dxb_init(), and
 *	just before firing up the firmware.  This prepares a DXBPHYS
 *	structure with physical buffer addresses which the host
 *	then communicates to the firmware (see triple boot protocol)
 *	using a global pointer in shared memory.
 *
 *	Each page of each circular buffer is stamped with a magic
 *	number and its page number.  This allows the FEP to determine
 *	that the page is really there at initialization.  The FEP
 *	will die if the pages aren't marked correctly.
 */
void
dxb_set_phys(DXB *dxb, DXBPHYS *phys)
{
	uint	i;
	uint32	*virt;

	dxb_printf("dxb_set_phys: dxb=%x phys=%x\n", (int) dxb, (int) phys);

	phys->h2a = dxb->h2a.buf;
	for (i = 0; i < dxb->h2a.buf.npages; ++i)
	{
		virt = (uint32 *) dxb->h2a.buf.page[i].virt;
		virt[0] = MAGIC_HOSTBUF;
		virt[1] = i;

		phys->h2a.page[i].bus = dxb_virt2bus(virt);
	}

	phys->a2h = dxb->a2h.buf;
	for (i = 0; i < dxb->a2h.buf.npages; ++i)
	{
		virt = (uint32 *) dxb->a2h.buf.page[i].virt;
		virt[0] = MAGIC_HOSTBUF;
		virt[1] = i;

		phys->a2h.page[i].bus = dxb_virt2bus(virt);
	}

	phys->qphys.host2adapter_in = dxb_virt2bus(&dxb->h2a.in);
	phys->qphys.host2adapter_out = dxb_virt2bus(&dxb->h2a.out);
	phys->qphys.adapter2host_in = dxb_virt2bus(&dxb->a2h.in);
	phys->qphys.adapter2host_out = dxb_virt2bus(&dxb->a2h.out);
}
#endif	/* DXB_HOST */


/*
 *	dxb_init:	Initialize a DXB instance
 *
 *	This function must be called before any DXB activity can take place.
 *
 *	The number of pages and the size of each page specifies
 *	the size of the two buffers we will allocate.  Yes, both the
 *	host to adapter and adapter to host queues are set to the
 *	same size.
 *
 *	The "openside" boolean specifies if we are the side
 *	that opens connections (host==1==DXB_OPENSIDE) or the
 *	slave side (adapter==0==DXB_SLAVESIDE).
 *
 *	Returns 0 on success, DXB_ENOMEM on allocation failure.
 *
 *	Typical host usage:
 *		DXB	Dxb;
 *		rc = dxb_init(&Dxb, 32, 4096, DXB_OPENSIDE);
 *
 *	Typical adapter usage:
 *		DXB	Dxb;
 *		rc = dxb_init(&Dxb, 1, 32*4096, DXB_SLAVESIDE);
 *
 *		NOTE: the adapter side usually uses the info in the phys
 *		structure to set the size of its buffer the same as the
 *		host.  So the call is more like:
 *
 *		rc = dxb_init(&Dxb, 1, dxbphys.host2adapter_q.size,
 *				DXB_SLAVESIDE);
 */
int
dxb_init(DXB *dxb, int npages, int pagesize, int openside)
{
	int	rc;

	dxb_printf("dxb_init: dxb=%x, npages=%d pagesize=%d openside=%d\n",
		(int) dxb, npages, pagesize, openside);

	LibdxbDxb = dxb;
	LibdxbChan = dxb->chan;
	LibdxbChanOffset = (char *) dxb->chan - (char *) dxb;

	/*
	 *	Structure starts off all zeros
	 */
	memset(dxb, 0, sizeof(DXB));

	dxb->openside = openside;
	find_openmap_limits(dxb);

	/*
 	 *	Get convenient pointers to *our* in and out Q's
	 */
	if (dxb->openside)
	{
		dxb->inq = &dxb->adapter2host_q;
		dxb->outq = &dxb->host2adapter_q;
	}
	else
	{
		dxb->inq = &dxb->host2adapter_q;
		dxb->outq = &dxb->adapter2host_q;
	}

#ifdef	DXB_MP
	dxb_mp_init(dxb);
#endif	/* DXB_MP */

	/*
	 *	Initialize outq's in_tmp to BOGUS_VALUE
	 */
	dxb->outq->in_tmp = BOGUS_VALUE ;

	/*
	 *	Allocate buffers
	 */
	rc = dxb_buf_alloc(&dxb->h2a.buf, npages, pagesize,
				dxb_phys_page_size() );
	if (rc) return (DXB_ENOMEM);

	rc = dxb_buf_alloc(&dxb->a2h.buf, npages, pagesize,
				dxb_phys_page_size() );
	if (rc) return (DXB_ENOMEM);

	return (0);
}

