
#
# Copyright 1999 by Digi International (www.digi.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
#	$Id: dm_misc.tcl,v 1.11 2001/03/23 22:16:10 jamesp Exp $
#
package provide dm_gui 1.0

#
#	Open the management channel
#
proc open_mgmt {bn} {
	global mgmt

	osdep_open_mgmt $bn
}

proc close_mgmt {bn} {
	global mgmt

	closefd $mgmt($bn)
}

proc check_all_elements { swig_func swig_struct bound value } {
	# Check all elements of an array for 'value', return 1 if
	# all elements contain that value
	for { set i 0 } { $i < $bound } { incr i } { 
		if { [$swig_func $swig_struct $i] != $value } {
			return 0
		}
	}

	return 1
}

proc uptimeString {uptime_seconds} {
	set a_uptime $uptime_seconds 
	set uptime_str ""
	set a_uptime_days [ expr $a_uptime / 86400 ]
		set uptime_str "\n$a_uptime_days Days,"
		set a_uptime [ expr $a_uptime - ($a_uptime_days * 86400) ]
	set a_uptime_hours [ expr $a_uptime / 3600 ] 
		set uptime_str "$uptime_str $a_uptime_hours Hours,\n"
		set a_uptime [ expr $a_uptime - ($a_uptime_hours * 3600) ]
	set a_uptime_minutes [ expr $a_uptime / 60 ]
		set uptime_str "$uptime_str $a_uptime_minutes Minutes,"
		set a_uptime [ expr $a_uptime - ($a_uptime_minutes * 60) ]
	set a_uptime_seconds $a_uptime
		set uptime_str "$uptime_str $a_uptime_seconds Seconds."

	return $uptime_str
}

proc connectTimeString {uptime_seconds} {
	set a_uptime $uptime_seconds 
	set uptime_str ""
	set a_uptime_hours [ expr $a_uptime / 3600 ] 
		set uptime_str "${a_uptime_hours}h"
		set a_uptime [ expr $a_uptime - ($a_uptime_hours * 3600) ]
	set a_uptime_minutes [ expr $a_uptime / 60 ]
		set uptime_str "$uptime_str ${a_uptime_minutes}m"
		set a_uptime [ expr $a_uptime - ($a_uptime_minutes * 60) ]
	set a_uptime_seconds $a_uptime
		set uptime_str "$uptime_str ${a_uptime_seconds}s"

	return $uptime_str
}

#
# old_notifier (no longer needed)
#
proc old_notifier {name title text { width 10c } } {

    if {[winfo exists $name]} { 
		destroy $name
    }

	toplevel $name

	wm title $name $title
    
	buttonFrame $name
    
	packedButton $name.buttons .quit "Dismiss" \
	 "destroy $name ; global $name ; set $name 0" left
    
	message $name.msg -aspect 300 -text $text -width $width

	lappend FontWidgets $name.msg
	pack append $name $name.msg {top expand}

	return
}

#
# notifier
#
proc notifier {name title text { width 60 } { height 15 } } {

    if {[winfo exists $name]} { 
		destroy $name
    }

	toplevel $name

	wm title $name $title

	tixNoteBook $name.help_nb -ipadx 6
	$name config -bg gray
	$name.help_nb subwidget nbframe config -backpagecolor gray
    
	set t [Scrolled_Text $name.help_nb.txt -width $width -height $height]

	pack $name.help_nb -expand yes -fill both -padx 5 -pady 5 -side top

	$t insert end $text

	buttonFrame $name.help_nb
    
	packedButton $name.help_nb.buttons .quit "Dismiss" \
	 "destroy $name ; global $name ; set $name 0" bottom

	return
}

#
# 
#
proc Scrolled_Text { f args } {

	frame $f

	eval {text $f.text -yscrollcommand [list $f.yscroll set]} $args

	scrollbar $f.yscroll -orient vertical -command [list $f.text yview]

	grid $f.text $f.yscroll -sticky news
	grid columnconfigure $f 0 -weight 1

	pack $f

	return $f.text
}

#
# buttonFrame creates a frame that is designed to hold a row of buttons
#
proc buttonFrame { parent {name .buttons} {border 2} } {

    set self [selfName $parent $name]
    set color [format #%02x%02x%02x 240 128 0]
    frame $self -borderwidth $border \
	-relief raised
    pack append $parent $self {bottom fillx}
    return $self
}

#
# packedButton adds a button to a row of buttons
#
proc packedButton { parent name label command {position left} } {

    set self [selfName $parent $name]
    button $self -text $label -command $command

    lappend FontWidgets $self
    pack append $parent $self $position

    return $self
}

#
# selfName - determine the name of a nested widget
#	parent is either "." or ".foo.bar"
#	name is ".zork"
#
proc selfName { parent name } {
    if {[string compare $parent "."] == 0} {
		set self $name
    } else {
		set self $parent$name
    }
    return $self
}

#
#	Get the adapter_info structure
#
proc get_adapter_info {bn} {
	global mgmt
	global ioc_adapter_info
	global adapter_info
	global GET_ADAPTER_INFO

	set ioc_adapter_info($bn) [new_get_adapter_info_t]

	set adapter_info($bn) \
		[ get_adapter_info_t_info_get $ioc_adapter_info($bn) ]

	ioctl $mgmt($bn) $GET_ADAPTER_INFO $ioc_adapter_info($bn)
}

proc get_adapter_state { bn }  {
	global GET_ADAPTER_STATE
	global ioc_adapter_state
	global adapter_state 
	global mgmt

	set ioc_adapter_state($bn) [new_get_adapter_state_t]
	set adapter_state($bn) \
		[get_adapter_state_t_state_get $ioc_adapter_state($bn)]

	ioctl $mgmt($bn) $GET_ADAPTER_STATE $ioc_adapter_state($bn)
}

proc get_serial_status {bn cn} {
	global GET_SERIAL_STATUS
	global ioc_serial_status
	global serial_status
	global mgmt
	
	set ioc_serial_status($bn,$cn) [new_get_serial_status_t]
	set serial_status($bn,$cn) \
		[get_serial_status_t_status_get \
			$ioc_serial_status($bn,$cn) ]

	get_serial_status_t_channel_set $ioc_serial_status($bn,$cn) $cn
	ioctl $mgmt($bn) $GET_SERIAL_STATUS $ioc_serial_status($bn,$cn) 

}

proc get_serial_config {bn cn} {
    global GET_SERIAL_CONFIG
    global ioc_serial_config
    global serial_config
    global mgmt

    set ioc_serial_config($bn,$cn) [getset_serial_config_t]
    set serial_config($bn,$cn) [$ioc_serial_config($bn,$cn) cget -config]
    serial_config_t -this $serial_config($bn,$cn)

    $ioc_serial_config($bn,$cn) configure -channel $cn
    ioctl $mgmt($bn) $GET_SERIAL_CONFIG $ioc_serial_config($bn,$cn)
}

proc get_serial_channel_test {bn cn} {
	global GET_SERIAL_CHANNEL_MODE
	global ioc_serial_channel_test
	global serial_channel_test
	global mgmt

	set ioc_serial_channel_test($bn,$cn) [new_getset_serial_channel_test_t]
	set serial_channel_test($bn,$cn) [getset_serial_channel_test_t_mode_get $ioc_serial_channel_test($bn,$cn)]

	getset_serial_channel_test_t_channel_set \
	     $ioc_serial_channel_test($bn,$cn) $cn
	ioctl $mgmt($bn) $GET_SERIAL_CHANNEL_MODE $ioc_serial_channel_test($bn,$cn)
}

proc get_serial_ext_status {bn cn} {
	global GET_SERIAL_EXT_STATUS
	global ioc_serial_ext_status
	global serial_ext_status
	global mgmt
	
	set ioc_serial_ext_status($bn,$cn) [new_get_serial_ext_status_t]
	set serial_ext_status($bn,$cn) \
		[get_serial_ext_status_t_status_get \
			$ioc_serial_ext_status($bn,$cn) ]

	get_serial_ext_status_t_channel_set $ioc_serial_ext_status($bn,$cn) $cn
	ioctl $mgmt($bn) $GET_SERIAL_EXT_STATUS $ioc_serial_ext_status($bn,$cn) 
}

proc get_ds1_status {bn pn} {
	global GET_DS1_STATUS
	global ioc_ds1_status
	global ds1_status
	global mgmt

	set ioc_ds1_status($bn,$pn) [new_get_ds1_status_t]
	set ds1_status($bn,$pn) \
		[get_ds1_status_t_status_get \
			$ioc_ds1_status($bn,$pn) ]

	get_ds1_status_t_port_set $ioc_ds1_status($bn,$pn) $pn
	ioctl $mgmt($bn) $GET_DS1_STATUS $ioc_ds1_status($bn,$pn)
}

proc get_ds1_ext_status {bn pn} {
	global GET_DS1_EXT_STATUS
	global ioc_ds1_ext_status
	global ds1_ext_status
	global mgmt

	set ioc_ds1_ext_status($bn,$pn) [new_get_ds1_ext_status_t]
	set ds1_ext_status($bn,$pn) \
		[get_ds1_ext_status_t_status_get \
			$ioc_ds1_ext_status($bn,$pn) ]

	get_ds1_ext_status_t_port_set $ioc_ds1_ext_status($bn,$pn) $pn
	ioctl $mgmt($bn) $GET_DS1_EXT_STATUS $ioc_ds1_ext_status($bn,$pn)
}

proc get_bri_config {bn pn} {
	global GET_BRI_CONFIG
	global ioc_bri_config
	global bri_config
	global mgmt

	set ioc_bri_config($bn,$pn) [new_getset_bri_config_t]
	set bri_config($bn,$pn) \
		[getset_bri_config_t_config_get \
			$ioc_bri_config($bn,$pn) ]

	getset_bri_config_t_port_set $ioc_bri_config($bn,$pn) $pn
	ioctl $mgmt($bn) $GET_BRI_CONFIG $ioc_bri_config($bn,$pn)
}

proc get_bri_status {bn pn} {
	global GET_BRI_STATUS
	global ioc_bri_status
	global bri_status
	global mgmt

	set ioc_bri_status($bn,$pn) [new_get_bri_status_t]
	set bri_status($bn,$pn) \
		[get_bri_status_t_status_get \
			$ioc_bri_status($bn,$pn) ]

	get_bri_status_t_port_set $ioc_bri_status($bn,$pn) $pn
	ioctl $mgmt($bn) $GET_BRI_STATUS $ioc_bri_status($bn,$pn)
}

proc get_dchan_ext_status {bn pn} {
	global GET_DCHAN_EXT_STATUS
	global ioc_dchan_ext_status
	global dchan_ext_status
	global mgmt

	set ioc_dchan_ext_status($bn,$pn) [new_get_dchan_ext_status_t]
	set dchan_ext_status($bn,$pn) \
		[get_dchan_ext_status_t_status_get \
			$ioc_dchan_ext_status($bn,$pn) ]

	get_dchan_ext_status_t_port_set $ioc_dchan_ext_status($bn,$pn) $pn
	ioctl $mgmt($bn) $GET_DCHAN_EXT_STATUS $ioc_dchan_ext_status($bn,$pn)
}

proc get_call_status { bn pn cn } {
	global GET_CALL_STATUS
	global ioc_call_status
	global call_status
	global mgmt

	set ioc_call_status($bn,$pn,$cn) [new_get_call_status_t]
	set call_status($bn,$pn,$cn) \
		[get_call_status_t_status_get \
			$ioc_call_status($bn,$pn,$cn) ]

	get_call_status_t_port_set $ioc_call_status($bn,$pn,$cn) $pn
	get_call_status_t_channel_set $ioc_call_status($bn,$pn,$cn) $cn
	ioctl $mgmt($bn) $GET_CALL_STATUS $ioc_call_status($bn,$pn,$cn)
}

proc get_all_call_status { bn pn } {
	global GET_ALL_CALL_STATUS
	global ioc_all_call_status
	global all_call_status
	global mgmt

	set ioc_all_call_status($bn,$pn) [new_get_all_call_status_t]
	set all_call_status($bn,$pn) \
		[get_all_call_status_t_status_get \
			$ioc_all_call_status($bn,$pn) ]

	get_all_call_status_t_port_set $ioc_all_call_status($bn,$pn) $pn
	ioctl $mgmt($bn) $GET_ALL_CALL_STATUS $ioc_all_call_status($bn,$pn)
}

proc get_modem_status { bn cn } {
	global GET_MODEM_STATUS
	global ioc_modem_status
	global modem_status
	global mgmt

	set ioc_modem_status($bn,$cn) [new_get_modem_status_t]
	set modem_status($bn,$cn) \
		[get_modem_status_t_status_get \
			$ioc_modem_status($bn,$cn) ]

	get_modem_status_t_channel_set $ioc_modem_status($bn,$cn) $cn
	ioctl $mgmt($bn) $GET_MODEM_STATUS $ioc_modem_status($bn,$cn)
}

proc get_bchan_status { bn cn } {
	global GET_BCHAN_STATUS
	global ioc_bchan_status
	global bchan_status
	global mgmt

	set ioc_bchan_status($bn,$cn) [new_get_bchan_status_t]
	set bchan_status($bn,$cn) \
		[get_bchan_status_t_status_get \
			$ioc_bchan_status($bn,$cn) ]

	get_bchan_status_t_channel_set $ioc_bchan_status($bn,$cn) $cn
	ioctl $mgmt($bn) $GET_BCHAN_STATUS $ioc_bchan_status($bn,$cn)
}

proc reset_modem { bn cn } {
	global SET_MODEM_CONFIG
	global ioc_getset_modem_config
	global ioc_modem_config
	global mgmt

	set ioc_getset_modem_config($bn) [new_getset_modem_config_t]
	set ioc_modem_config($bn) [new_modem_config_t]

	modem_config_t_answer_rings_set $ioc_modem_config($bn) 0
	modem_config_t_clid_enable_set $ioc_modem_config($bn) 0
	modem_config_t_max_rate_set $ioc_modem_config($bn) 0
	modem_config_t_min_rate_set $ioc_modem_config($bn) 0
	modem_config_t_hangup_set $ioc_modem_config($bn) 1
	modem_config_t_reset_set $ioc_modem_config($bn) 1

	getset_modem_config_t_channel_set $ioc_getset_modem_config($bn) $cn
	getset_modem_config_t_config_set $ioc_getset_modem_config($bn) \
		$ioc_modem_config($bn)

	ioctl $mgmt($bn) $SET_MODEM_CONFIG $ioc_getset_modem_config($bn)

	delete_modem_config_t $ioc_modem_config($bn)
	delete_getset_modem_config_t $ioc_getset_modem_config($bn)
}

#
#	Reset the RS-232 port on a ap2k
#
proc reset_rs232 { bn cn } {
	global SET_SERIAL_EXT_CONFIG
	global ioc_getset_serial_ext_config
	global ioc_serial_ext_config
	global mgmt

	set ioc_getset_serial_ext_config($bn) [new_getset_serial_ext_config_t]
	set ioc_serial_ext_config($bn) [new_serial_ext_config_t]

	serial_ext_config_t_unpause_rx_set $ioc_serial_ext_config($bn) 0
	serial_ext_config_t_unpause_tx_set $ioc_serial_ext_config($bn) 0
	serial_ext_config_t_reset_set $ioc_serial_ext_config($bn) 1

	getset_serial_ext_config_t_channel_set \
		$ioc_getset_serial_ext_config($bn) $cn
	getset_serial_ext_config_t_config_set \
		$ioc_getset_serial_ext_config($bn) \
		$ioc_serial_ext_config($bn)

	ioctl $mgmt($bn) $SET_SERIAL_EXT_CONFIG \
		$ioc_getset_serial_ext_config($bn)

	delete_serial_ext_config_t $ioc_serial_ext_config($bn)
	delete_getset_serial_ext_config_t $ioc_getset_serial_ext_config($bn)
}

#
#  Handle changes to the "loopback test" state for Xp adapters
#
proc manage_loopback_test { bn cn action } {
	global SET_SERIAL_CHANNEL_MODE
	global TEST_MODE
	global EXT_LOOP_AUTO
	global STOP_TEST
	global TEST_EXIT
	global mgmt

	set ioc_getset_test [new_getset_serial_channel_test_t]
	set ioc_test [new_serial_channel_test_t]

	if { $action == 0 } {
		#
		#  Stop the test
		#
		serial_channel_test_t_mode_set $ioc_test $STOP_TEST
		getset_serial_channel_test_t_channel_set $ioc_getset_test $cn
		getset_serial_channel_test_t_mode_set    $ioc_getset_test $ioc_test

		ioctl $mgmt($bn) $SET_SERIAL_CHANNEL_MODE $ioc_getset_test

		serial_channel_test_t_mode_set $ioc_test $TEST_EXIT
		getset_serial_channel_test_t_channel_set $ioc_getset_test $cn
		getset_serial_channel_test_t_mode_set    $ioc_getset_test $ioc_test

		ioctl $mgmt($bn) $SET_SERIAL_CHANNEL_MODE $ioc_getset_test
	} \
	elseif { $action == 1 } {
		#
		#  Start an auto external loopback test
		#
		serial_channel_test_t_mode_set $ioc_test $TEST_MODE
		getset_serial_channel_test_t_channel_set $ioc_getset_test $cn
		getset_serial_channel_test_t_mode_set    $ioc_getset_test $ioc_test

		ioctl $mgmt($bn) $SET_SERIAL_CHANNEL_MODE $ioc_getset_test

		serial_channel_test_t_mode_set $ioc_test $EXT_LOOP_AUTO
		getset_serial_channel_test_t_channel_set $ioc_getset_test $cn
		getset_serial_channel_test_t_mode_set    $ioc_getset_test $ioc_test

		ioctl $mgmt($bn) $SET_SERIAL_CHANNEL_MODE $ioc_getset_test
	}

	delete_serial_channel_test_t $ioc_test
	delete_getset_serial_channel_test_t $ioc_getset_test
}

proc modem_compression2text { value } {
	# From mgmthlc.h
	global MODEM_NO_COMP
	global MODEM_MNP5
	global MODEM_V42BIS
	global MODEM_V24BIS
	global MODEM_T4_1D
	global MODEM_T4_2D
	global MODEM_T4_2D_UNCOMP
	global MODEM_T6

	switch -exact $value \
		$MODEM_NO_COMP { \
			set text "None"
		} \
		$MODEM_MNP5 { \
			set text "MNP5"
		} \
		$MODEM_V42BIS { \
			set text "V.42 bis"
		} \
		$MODEM_V24BIS { \
			set text "V.24 bis"
		} \
		$MODEM_T4_1D { \
			set text "T4 1D"
		} \
		$MODEM_T4_2D { \
			set text "T4 2D"
		} \
		$MODEM_T4_2D_UNCOMP { \
			set text "T4 2D Uncompressed"
		} \
		$MODEM_T6 { \
			set text "T6"
		} \
		default { \
			set text "Unknown"
		}

	return $text
}

proc modem_error_correction2text { value } {
	# From mgmthlc.h
	global MODEM_NO_EC
	global MODEM_LAPM
	global MODEM_MNP2
	global MODEM_MNP3
	global MODEM_MNP4
	global MODEM_V42
	global MODEM_FAXECM64
	global MODEM_FAXECM256

	switch -exact $value \
		$MODEM_NO_EC { \
			set text "None"
		} \
		$MODEM_LAPM { \
			set text "LAPM"
		} \
		$MODEM_MNP2 { \
			set text "MNP2"
		} \
		$MODEM_MNP3 { \
			set text "MNP3"
		} \
		$MODEM_MNP4 { \
			set text "MNP4"
		} \
		$MODEM_V42 { \
			set text "V.42"
		} \
		$MODEM_FAXECM64 { \
			set text "FAX ECM 64"
		} \
		$MODEM_FAXECM256 { \
			set text "FAX ECM 256"
		} \
		default { \
			set text "Unknown"
		}

	return $text
}

proc modem_modulation2text { value } {
	# From mgmthlc.h
	global MODEM_103
	global MODEM_212A
	global MODEM_V21
	global MODEM_V22
	global MODEM_V22BIS
	global MODEM_V23
	global MODEM_V32
	global MODEM_V32BIS
	global MODEM_V34
	global MODEM_V34BIS
	global MODEM_K56FLEX
	global MODEM_V90
	global MODEM_V17
	global MODEM_V21CH2
	global MODEM_V27TER
	global MODEM_V29
	global MODEM_V33
	global MODEM_V34FAX

	switch -exact $value \
		$MODEM_103 { \
			set text "Bell 103"
		} \
		$MODEM_212A { \
			set text "Bell 212A"
		} \
		$MODEM_V21 { \
			set text "V.21"
		} \
		$MODEM_V22 { \
			set text "V.22"
		} \
		$MODEM_V22BIS { \
			set text "V.22 bis"
		} \
		$MODEM_V23 { \
			set text "V.23"
		} \
		$MODEM_V32 { \
			set text "V.32"
		} \
		$MODEM_V32BIS { \
			set text "V.32 bis"
		} \
		$MODEM_V34 { \
			set text "V.34"
		} \
		$MODEM_V34BIS { \
			set text "V.34 bis"
		} \
		$MODEM_K56FLEX { \
			set text "K56flex"
		} \
		$MODEM_V90 { \
			set text "V.90"
		} \
		$MODEM_V17 { \
			set text "V.17"
		} \
		$MODEM_V21CH2 { \
			set text "V.21 CH2"
		} \
		$MODEM_V27TER { \
			set text "V.27 TER"
		} \
		$MODEM_V29 { \
			set text "V.29"
		} \
		$MODEM_V33 { \
			set text "V.33"
		} \
		$MODEM_V34 { \
			set text "V.34 FAX"
		} \
		default { \
			set text "Unknown"
		}

}

proc bchan_compression2text { value } {
	#  mgmthlc.h
	global BCHAN_NO_COMP
	global BCHAN_STAC

	switch -exact $value \
		$BCHAN_NO_COMP { \
			set text "Not in Use"
		} \
		$BCHAN_STAC { \
			set text "STAC"
		} \
		default { \
			set text "Unknown"
		}

	return $text
}

proc bchan_mode2text { value } {
	#  mgmthlc.h
	global BCHAN_IDLE
	global BCHAN_INBOUND
	global BCHAN_OUTBOUND

	switch -exact $value \
		$BCHAN_IDLE { \
			set text "Not in Use"
		} \
		$BCHAN_INBOUND { \
			set text "Inbound"
		} \
		$BCHAN_OUTBOUND { \
			set text "Outbound"
		} \
		default { \
			set text "Unknown"
		}

	return $text
}

proc modem_mode2text { value } {
	#  mgmthlc.h
	global MODEM_IDLE
	global MODEM_INBOUND
	global MODEM_OUTBOUND

	switch -exact $value \
		$MODEM_IDLE { \
			set text "Not in Use"
		} \
		$MODEM_INBOUND { \
			set text "Inbound"
		} \
		$MODEM_OUTBOUND { \
			set text "Outbound"
		} \
		default { \
			set text "Unknown"
		}

	return $text
}

#
# Generate the total logical channel number, given the port number,
# channel number, and channels-per-port.
#
proc total_channel_number { pn cn c_p_p } {
	return [expr (($pn - 1) * $c_p_p) + $cn]
}


#
#       Fill in the Adapter Info details for a single board
#
proc create_adapter_info_detail {f bn} {
	global	adapter_info
	# Let's set the adapter's common name
	global  adapter_cn
	global	board_type
	global	board_family
	global	ADAPTER_HOU30
	global	DIGI_PCIHOUSTON30_NAME
	global	ADAPTER_HOU60
	global	DIGI_PCIHOUSTON60_NAME
	global	ADAPTER_HOU24
	global	DIGI_PCIHOUSTON24_NAME
	global	ADAPTER_HOU48
	global	DIGI_PCIHOUSTON48_NAME
	global	ADAPTER_HOU30cpci
	global	DIGI_CPCIHOUSTON30_NAME
	global	ADAPTER_HOU60cpci
	global	DIGI_CPCIHOUSTON60_NAME
	global	ADAPTER_HOU24cpci
	global	DIGI_CPCIHOUSTON24_NAME
	global	ADAPTER_HOU48cpci
	global	DIGI_CPCIHOUSTON48_NAME
	global	ADAPTER_LONU
	global	DIGI_LONDONU_NAME
	global	ADAPTER_LONST
	global	DIGI_LONDONST_NAME
	global	ADAPTER_CHASKA80
	global	DIGI_CHASKA80_NAME
	global	ADAPTER_CHASKA81
	global	DIGI_CHASKA81_NAME
	global	ADAPTER_CHASKA82
	global	DIGI_CHASKA82_NAME
	global	ADAPTER_CHASKA83
	global	DIGI_CHASKA83_NAME
	global	ADAPTER_CHASKA40
	global	DIGI_CHASKA40_NAME
	global	ADAPTER_CHASKA41
	global	DIGI_CHASKA41_NAME
	global	ADAPTER_CHASKA42
	global	DIGI_CHASKA42_NAME
	global	ADAPTER_CHASKA43
	global	DIGI_CHASKA43_NAME
	global	ADAPTER_AP2K2
	global	DIGI_AP2K_NAME
	global	ADAPTER_AP2K4
	global	DIGI_AP4K_NAME
	global	ADAPTER_AP2K8
	global	DIGI_AP8K_NAME
	global	ADAPTER_AP2K16
	global	DIGI_AP16K_NAME
#	global	ADAPTER_AP2K32
#	global	DIGI_AP32K_NAME

	set ai $adapter_info($bn)

	frame $f.tbl

	create_info_label $f.tbl $bn "Adapter Type" board_type

	set bt [adapter_info_t_board_type_get $ai]

	set daughter no
	set pump yes

	switch -exact $bt \
		$ADAPTER_HOU30 { \
			set name $DIGI_PCIHOUSTON30_NAME; \
			set board_family($bn) dfrp; \
			set board_type($bn) dfrpe1 } \
		$ADAPTER_HOU60 { \
			set daughter yes; \
			set name $DIGI_PCIHOUSTON60_NAME; \
			set board_family($bn) dfrp; \
			set board_type($bn) dfrpe2 } \
		$ADAPTER_HOU24 { \
			set name $DIGI_PCIHOUSTON24_NAME; \
			set board_family($bn) dfrp; \
			set board_type($bn) dfrpt1 } \
		$ADAPTER_HOU48 { \
			set daughter yes; \
			set name $DIGI_PCIHOUSTON48_NAME; \
			set board_family($bn) dfrp; \
			set board_type($bn) dfrpt2 } \
		$ADAPTER_HOU30cpci { \
			set name $DIGI_CPCIHOUSTON30_NAME; \
			set board_family($bn) dfdp; \
			set board_type($bn) dfdpe1 } \
		$ADAPTER_HOU60cpci { \
			set daughter yes; \
			set name $DIGI_CPCIHOUSTON60_NAME; \
			set board_family($bn) dfdp; \
			set board_type($bn) dfdpe2 } \
		$ADAPTER_HOU24cpci { \
			set name $DIGI_CPCIHOUSTON24_NAME; \
			set board_family($bn) dfdp; \
			set board_type($bn) dfdpt1 } \
		$ADAPTER_HOU48cpci { \
			set daughter yes; \
			set name $DIGI_CPCIHOUSTON48_NAME; \
			set board_family($bn) dfdp; \
			set board_type($bn) dfdpt2 } \
		$ADAPTER_LONU { \
			set name $DIGI_LONDONU_NAME; \
			set board_family($bn) dfrb; \
			set board_type($bn) dfrb4u } \
		$ADAPTER_LONST { \
			set name $DIGI_LONDONST_NAME; \
			set board_family($bn) dfrb; \
			set board_type($bn) dfrb4st } \
		$ADAPTER_CHASKA80 { \
			set name $DIGI_CHASKA80_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr80 } \
		$ADAPTER_CHASKA40 { \
			set name $DIGI_CHASKA40_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr41 } \
		$ADAPTER_CHASKA81 { \
			set name $DIGI_CHASKA81_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr81 } \
		$ADAPTER_CHASKA41 { \
			set name $DIGI_CHASKA41_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr41 } \
		$ADAPTER_CHASKA82 { \
			set name $DIGI_CHASKA82_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr82 } \
		$ADAPTER_CHASKA42 { \
			set name $DIGI_CHASKA42_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr42 } \
		$ADAPTER_CHASKA83 { \
			set name $DIGI_CHASKA83_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr83 } \
		$ADAPTER_CHASKA43 { \
			set name $DIGI_CHASKA43_NAME; \
			set board_family($bn) apr; \
			set board_type($bn) apr43 } \
		$ADAPTER_AP2K2 { \
			set pump no; \
			set name $DIGI_AP2K_NAME; \
			set board_family($bn) ap2k; \
			set board_type($bn) ap2k2 } \
		$ADAPTER_AP2K4 { \
			set pump no; \
			set name $DIGI_AP4K_NAME; \
			set board_family($bn) ap2k; \
			set board_type($bn) ap2k4 } \
		$ADAPTER_AP2K8 { \
			set pump no; \
			set name $DIGI_AP8K_NAME; \
			set board_family($bn) ap2k; \
			set board_type($bn) ap2k8 } \
		$ADAPTER_AP2K16 { \
			set pump no; \
			set name $DIGI_AP16K_NAME; \
			set board_family($bn) ap2k; \
			set board_type($bn) ap2k16 } \
		default { \
			set name "Unknown"; \
			set board_family($bn) unknown; \
			set board_type($bn) unknown }

#
#  The following would be added to the above if/when a 32 port
#  adapter is made available.
#
#		$ADAPTER_AP2K32 { \
#			set pump no; \
#			set name $DIGI_AP32K_NAME; \
#			set board_family($bn) ap2k; \
#			set board_type($bn) ap2k32 } \
#
	
	$f.tbl.value_board_type config -text "$name"
	set adapter_cn $name

	create_info_label $f.tbl $bn "Port Count" port_count
	create_info_label $f.tbl $bn "Channels/Port" channels_per_port
	create_info_label $f.tbl $bn "Memory Size" memory_size

	create_info_label $f.tbl $bn "FEP Version" fep_version
	set ver [adapter_info_t_fep_version_get $ai]
	if { $ver < 0x40000000 } {
		# ad-hoc FEP: build date
		$f.tbl.value_fep_version config -text \
			[format "%x" $ver]
	} else {
		# released FEP: part number suffix
		$f.tbl.value_fep_version config -text \
			[format "%c%c%c%c" \
				[expr ($ver >> 24) & 0xff] \
				[expr ($ver >> 16) & 0xff] \
				[expr ($ver >> 8) & 0xff] \
				[expr ($ver >> 0) & 0xff] \
			]
	}

	if { $pump == "yes" } {

		create_info_label $f.tbl $bn "CSC Version" csc_version
		set ver [adapter_info_t_csc_version_get $ai]
		$f.tbl.value_csc_version config -text \
				[format "%c.%c%c%c" \
					[expr ($ver >> 24) & 0xff] \
					[expr ($ver >> 16) & 0xff] \
					[expr ($ver >> 8) & 0xff] \
					[expr ($ver >> 0) & 0xff] \
				]

		create_info_label $f.tbl $bn "PUMP Version" pump_version
		set ver [adapter_info_t_pump_version_get $ai]
		$f.tbl.value_pump_version config -text \
				[format "%x.%x%c" \
					[expr ($ver >> 24) & 0xff] \
					[expr ($ver >> 16) & 0xff] \
					[expr ($ver >> 0) & 0xff] \
				]
	}

	create_info_label $f.tbl $bn "Part Number" part_number
	set ver [any2str [adapter_info_t_part_number_get $ai]]
	$f.tbl.value_part_number config -text $ver

	create_info_label $f.tbl $bn "Main Board Serial Number" main_board_serial
	set ver [any2str [adapter_info_t_main_board_serial_get $ai]]
	$f.tbl.value_main_board_serial config -text $ver

	create_info_label $f.tbl $bn "Main Adapter Revision" main_board_revision

	if { $daughter == "yes" } {

		create_info_label $f.tbl $bn "Daughter Card Serial Number" daughter_card_serial
		set ver [any2str [adapter_info_t_daughter_card_serial_get $ai]]
		$f.tbl.value_daughter_card_serial config -text $ver

		create_info_label $f.tbl $bn "2nd Adapter Revision" daughter_card_revision
	}
	# missing serial #'s

	pack $f.tbl -anchor w
}


#
#       Create the Adapter Info page by adding a tab and then filling in the
#	frame for that tab using the "detail" helper.
#
proc create_adapter_info {w bn} {

	$w add adapter_info -label "Adapter Info" -underline 8

	set f [$w subwidget adapter_info]

	create_adapter_info_detail $f $bn
}

