
/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stddef.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>

#ifdef STREAMS

#include <sys/conf.h>
#include <sys/stream.h>
#include <sys/stropts.h>

#endif


#define TRUE 1
#define FALSE 0

/*
 *	This family mask is used with the adapter_info_t->board_type variable and
 *	its defined valid values (from pci_ids.h).  As new values are added
 *	to pci_ids.h, make sure this mask and its later usage in this
 *	program continue to be valid.
 */

#define FAMILY_MASK 0xfff0

#include <time.h>
#include "tracelog.h"

#if defined(SCO)
#include "mgmtioc1.h"
#include "rasmon.h"
#endif

#include "trace_os_dep.h"

/*
 *	Function prototypes
 */

void usage (char *pgm);

void decode_error_codes( int n );

int SendCMD(int Mgmtfd, mgmt_req_t *req, void  *resp_ptr );

void dump_packet (time_t seconds, trace_header_t *tracehdr);

int write_header (void );

int check_ascii (int value);

void print_data(trace_header_t *tracehdr, int transmitted);

void report(trace_header_t *hdr, uchar *data);

void report_status(trace_header_t *hdr, uchar *data);

/*
 * Macros and defines
 */

#define MAX_TRACE_IN 4096 + sizeof(trace_header_t)

#define DO_SWAP4(l) (l)

#define DCHAN_MODE 1
#define BCHAN_MODE 2
#define SER_MODE   3
int modetype  = DCHAN_MODE;

#define RX_ONLY 1
#define TX_ONLY 2
#define RXTX    3
int direction = RXTX;

/*
 * Global variables
 */

int Mgmtfd;

#if !defined(LINUX)
FILE	*Outfp = stdout, *Rawfp = stderr;
#else
FILE	*Outfp, *Rawfp;
#endif

char trace_in_buf[MAX_TRACE_IN];

#ifdef STREAMS
struct strbuf cbuf, dbuf;
#endif

#ifdef STREAMS
get_adapter_info_response_t 
#else ifdef IOCTL
get_adapter_info_t 
#endif
 adapter_info_r;

char outbuf[200] = "/dev/tty";
char rawbuf[200] = "/tmp/raw.ras";

int raw_write_to_file = 0;

int outfile_option = FALSE;  /* tracks if '-o' option invoked */

char	SerialMode = 'x';
int	TimeStamp = 1;
int	ModemSignals = 0;
int	FlowControl = 0;
char	LastDir = 0;
char	Dir = 0;
int	Column = 0;
int	LastStamp;

void
usage(char *pgm) {
	fprintf(stderr,
"Usage: %s\n"
"\t-S\t\tTrace serial\n"
"\t-B\t\tTrace B-channel (ISDN adapters only)\n"
"\t-D\t\tTrace D-channel (ISDN adapters only, default trace mode)\n"
"\t-b adapter\t(default 0)\n"
"\t-p port\t\t(default 1, valid only w/ -D)\n"
"\t-c channel\t(default 0, valid only w/ -S or -B)\n"
"\t-o outputfile\t(default /dev/tty)\n"
"\t-R rawfile\tRaw output written to rawfile in record format\n"
"\t-a\t\tPrint Serial or B-Channel trace in ASCII\n"
"\t-e\t\tPrint Serial or B-Channel trace in ASCII (with escapes)\n"
"\t-f\t\tPrint Serial or B-Channel trace in Happy FAX mode\n"
"\t-x\t\tPrint Serial or B-Channel trace in hex\n"
"\t-s\t\tPrint Serial or B-Channel trace in verbose side-by-side format\n"
"\t-T\t\tSuppress Serial or B-Channel timestamp display\n"
#ifndef OLD_HEADERS
"\t-m\t\tInclude Serial Trace modem signal changes\n"
"\t-F\t\tInclude Serial Trace flow control changes\n"
#endif
"\t-t\t\tTransmit tracing only\n"
"\t-r\t\tReceive  tracing only\n", pgm);
}

int
main(int argc, char *argv[])
{
	int counter=0;
	int board_family;
	int total_chans;
	int mode_error=0;
	int flags=0;
	int	channel=0;
	int port=1;
	int board=0;
	int rc;
	time_t seconds;
	char *outfile = NULL;
	char *rawfile = NULL;
	char device[128];
	char mesg[128];
	char prog[128];
	int command;
	mgmt_req_t      req;
	mgmt_hd_t       mgmt_hdr;
	mgmt_hd_t		mgmt_hdr_resp;
	int send_len = 0;
	int n;
	int retval;
	char c;
	trace_header_t *tracehdr;

#ifdef IOCTL
	int len;
	int hdrlen;
#endif

#if defined(LINUX)
	Outfp = stdout;
	Rawfp = stderr;
#endif

	if (getuid() != 0)
	{
		fprintf(stderr,
		 "\nYou must be the super-user to execute this program!\n\n");
		exit(1);
	}

	strcpy(prog, argv[0]);
	
	while ((c = getopt(argc, argv, "aefFmTxsSBDtrb:p:o:c:R:?")) != EOF) {
		switch(c) {
		case 'S':
			modetype = SER_MODE;
			break ;

		case 'B':
			modetype = BCHAN_MODE;
			break ;

		case 'D':
			modetype = DCHAN_MODE;
			break ;

		case 't':
			direction = TX_ONLY;
			break ;

		case 'r':
			direction = RX_ONLY;
			break ;

		case 'b':
			board = strtol(optarg, NULL, 0) ;
			break ;

		case 'p':
			port = strtol(optarg, NULL, 0) ;
			break ;

		case 'c':
			channel = strtol(optarg, NULL, 0) ;
			break ;

		case 'o':
			outfile_option = TRUE;
			outfile = optarg ;
			strcpy(outbuf, outfile);
			break;

		case 'R':
			/* 
			 * NOTE on raw formatting 
			 * This is currently the old DataFire BRI ISDN
			 * trace format with padding added which includes 
			 * directional information.  Eventually, we may want
			 * to migrate this to an equivalent DXBLIB MGMT
			 * format which uses the standard management headers
			 * It would be hopeful that direction information
			 * would be available at that time as well.
			 */
			raw_write_to_file = 1;
			rawfile = optarg ;
			strcpy(rawbuf, rawfile);

			break ;

		case 'a':
			SerialMode = 'a';
			break;

		case 'e':
			SerialMode = 'e';
			break;

		case 'f':
			SerialMode = 'f';
			break;

		case 'x':
			SerialMode = 'x';
			break;

		case 's':
			SerialMode = 's';
			break;

#ifndef OLD_HEADERS
		case 'm':
			ModemSignals = 1;
			break;

		case 'F':
			FlowControl = 1;
			break;
#endif

		case 'T':
			TimeStamp = 0;
			break;

		case '?':
		case ':':
		default:
			usage(prog);
			exit(0);
		}
	}

	if (outfile == NULL) {
		Outfp = stdout;
	}
	else if((Outfp=fopen(outbuf, "w")) == NULL) {
		sprintf(mesg, "Cannot open %s for output", outbuf);
		perror(mesg);
		exit(1);
	}

	for (n = FIRST_TRACE_CHAN; n < TRACE_CHAN_COUNT + FIRST_TRACE_CHAN;
	 n++ ) {

		/* try all devices */

		MK_DEV_NAME (device, DEVDIR, n, board);

#if 0
		fprintf(stdout, "\nTrying to open %s...\n", device);
#endif

		if((Mgmtfd=open(device, O_RDWR)) != -1) {
			break;
		}	
	}

	if (n >= TRACE_CHAN_COUNT + FIRST_TRACE_CHAN) {
		sprintf(mesg, "Cannot open device %s", device);
		perror(mesg);
		exit(1);
	}
#if 0
	else
		fprintf(stdout, "Open on %s succeeded\n", device);
#endif

	setbuf(Outfp, NULL);	/* Set to non-buffered for now */

	if (raw_write_to_file) { 
		/* See NOTE on raw formatting above */
		if (rawfile == NULL) {
			Rawfp = stderr;
		}
		else if((Rawfp=fopen(rawbuf, "w")) == NULL) {
			sprintf(mesg, "Can not open %s for output", rawbuf);
			perror(mesg);
			close (Mgmtfd);
			exit(1);
		}
		setbuf(Rawfp, NULL);	/* Set to non-buffered for now */

		/* write output to raw file */
		write_header();
	}

	/*
	 * Get adapter info so we can verify port/channel stuff
	 */

	memset (&req, 0, sizeof (req));

	req.hd.command	= GET_ADAPTER_INFO;
	req.hd.mpkt_len	= 0; /* Data size */
	req.hd.rpkt_len	= sizeof(adapter_info_r);

	if ((retval = SendCMD(Mgmtfd , &req, &adapter_info_r )) < 0) {
		fprintf(stderr, "\nCommand has been executed with error.\n");
		decode_error_codes(retval);
		close (Mgmtfd);
		exit (1);
	}

	board_family = adapter_info_r.info.board_type & FAMILY_MASK;
	total_chans = adapter_info_r.info.port_count *
	 adapter_info_r.info.channels_per_port;

	memset (&req, 0, sizeof (req));

	switch (modetype) {
		case DCHAN_MODE:

			// This mode valid only for ISDN adapters

			if ((board_family != ADAPTER_HOUXX) &&
			 (board_family != ADAPTER_HOUXXcpci) &&
			 (board_family != ADAPTER_LONXX)) {
				fprintf(stderr, 
					"D-Channel tracing valid only for ISDN adapters\n");
				close (Mgmtfd);
				usage(prog);

				exit(1);
			}

			if (port < 1 || port > adapter_info_r.info.port_count) {
				fprintf(stderr,
				 "D-Channel port must be 1 - %ld for this adapter\n",
				 adapter_info_r.info.port_count);
				close (Mgmtfd);
				usage(prog);

				exit(1);
			}

			req.rq.q_gs_dchan_data_trc.port = port;
			req.rq.q_gs_dchan_data_trc.trace_on = 1;

			send_len = sizeof(req.rq.q_gs_dchan_data_trc);

			switch (direction) {
			case RX_ONLY:
				command  = SET_DCHAN_RX_DATA_TRC;
				break;
			case TX_ONLY:
				command  = SET_DCHAN_TX_DATA_TRC;
				break;
			case RXTX:
				command  = SET_DCHAN_DATA_TRC;
				break;
			default:
				command  = SET_DCHAN_DATA_TRC;
				break;
			}

			break;

		case BCHAN_MODE:

			// This mode valid only for ISDN adapters

			if ((board_family != ADAPTER_HOUXX) &&
			 (board_family != ADAPTER_HOUXXcpci) &&
			 (board_family != ADAPTER_LONXX)) {
				fprintf(stderr, 
					"B-Channel tracing valid only for ISDN adapters\n");
				close (Mgmtfd);
				usage(prog);

				exit(1);
			}

			if (channel < 0 || channel >= total_chans) {
				fprintf(stderr, 
				 "B-channel must be 0 - %d for this adapter\n",
				 total_chans - 1);
				close (Mgmtfd);
				usage(prog);

				exit(1);
			}

			req.rq.q_gs_bchan_data_trc.channel = channel;
			req.rq.q_gs_bchan_data_trc.trace_on = 1;

			send_len = sizeof(req.rq.q_gs_bchan_data_trc);

			switch (direction) {
			case RX_ONLY:
				command = SET_BCHAN_RX_DATA_TRC;
				break;
			case TX_ONLY:
				command = SET_BCHAN_TX_DATA_TRC;
				break;
			case RXTX:
				command = SET_BCHAN_RXTX_DATA_TRC;
				break;
			default:
				command = SET_BCHAN_RXTX_DATA_TRC;
				break;
			}

			break;

		case SER_MODE:

			if (channel < 0 || channel >= total_chans) {
				fprintf(stderr, 
				 "Serial channel must be 0 - %d for this adapter\n",
				 total_chans - 1);
				close (Mgmtfd);
				usage(prog);

				exit(1);
			}

			req.rq.q_gs_serial_trc.channel = channel;
			req.rq.q_gs_serial_trc.trace_on = 1;

			send_len = sizeof(req.rq.q_gs_serial_trc);
 
			switch (direction) {
			case RX_ONLY:
				command = SET_SERIAL_RX_DATA_TRC;
				break;
			case TX_ONLY:
				command = SET_SERIAL_TX_DATA_TRC;
				break;
			case RXTX:
				command = SET_SERIAL_RXTX_DATA_TRC;
				break;
			default:
				command = SET_SERIAL_RXTX_DATA_TRC;
				break;
			}
			break;

		default:
			mode_error = 1;
			break;
	}

	if (mode_error) {
		fprintf(stderr, "Bad Tracing Mode");
		usage(prog);
		close (Mgmtfd);
		exit(1);
	}

	/* GET/SET_MGMT_XX_T of request */
	/*=============================*/

	req.hd.command      = command ;
	req.hd.mpkt_len     = send_len; /* Data size */
	req.hd.rpkt_len     = 0; /* Response size */

	if ((retval = SendCMD (Mgmtfd, &req, &mgmt_hdr)) != 0 ) {
		fprintf(stderr, "\nCommand has been executed with error.\n");
		decode_error_codes(retval);
		close (Mgmtfd);
		exit(1);
	}

#ifndef OLD_HEADERS
	if (ModemSignals || FlowControl)
	{
		memset (&req, 0, sizeof (req));

		req.hd.command      = SET_SERIAL_STATUS_TRC ;
		req.hd.mpkt_len     = sizeof(req.rq.q_gs_serial_status_trc);
		req.hd.rpkt_len     = 0; /* Response size */
		req.rq.q_gs_serial_status_trc.channel = channel;

		if (ModemSignals)
			req.rq.q_gs_serial_status_trc.status.mdm_sig_state[0] =
			 0xff;

		if (FlowControl)
		{
			req.rq.q_gs_serial_status_trc.status.in_flow_type[0] =
			 0xff;
			req.rq.q_gs_serial_status_trc.status.out_flow_type[0] =
			 0xff;
			req.rq.q_gs_serial_status_trc.status.in_flow_state[0] =
			 0xff;
			req.rq.q_gs_serial_status_trc.status.out_flow_state[0]=
			 0xff;
		}

		if ((retval = SendCMD (Mgmtfd, &req, &mgmt_hdr)) != 0 ) {
			fprintf(stderr,
			 "\nCommand has been executed with error.\n");
			decode_error_codes(retval);
			close (Mgmtfd);
			exit(1);
		}
	}
#endif /* OLD_HEADERS */

#ifdef STREAMS

	dbuf.maxlen = sizeof(trace_in_buf);
	dbuf.buf = trace_in_buf;

	cbuf.buf=(void *)&mgmt_hdr_resp;
	cbuf.len=sizeof(mgmt_hd_t);
	cbuf.maxlen = 29; /* FIXME */

#else ifdef IOCTL

	hdrlen = offsetof(trace_header_t, payload);

#endif

	tracehdr = (trace_header_t *) trace_in_buf;

	for (;;){

		/* Clear the buffer */
		memset (trace_in_buf, 0, sizeof(trace_in_buf));

#ifdef STREAMS

		dbuf.len = 0;

		if ((( rc = getmsg(Mgmtfd, &cbuf, &dbuf,&flags))) < 0) {
			fprintf(stderr, "getmsg error code=%d to device\n", rc);
			perror("getmsg");
			close(Mgmtfd);
			exit(1);
		}

		if (rc & MORECTL) {

			fprintf(stderr, "getmsg got MORECTL, invalid\n");
			close(Mgmtfd);
			exit(1);
		}

#else ifdef IOCTL

		/*
		 *	Read header
		 */
		len = read(Mgmtfd, trace_in_buf, hdrlen);

		if (len != hdrlen) {
			perror("read hdr");
			close(Mgmtfd);
			exit(1);
		}

		if (tracehdr->length > MAX_TRACE_IN) {

			fprintf (stderr, "tracehdr.length=%d greater than MAX of %d\n",
			 tracehdr->length, MAX_TRACE_IN);
			close(Mgmtfd);

			exit (1);
		}

		/*
		 *	Read payload
		 */
		len = read(Mgmtfd, trace_in_buf + hdrlen, tracehdr->length);

		if (len != tracehdr->length) {
			perror("read data");
			close(Mgmtfd);
			exit(1);
		}
#endif

		time(&seconds);

		dump_packet(seconds, tracehdr);
		++counter;

		if (raw_write_to_file) 
			fprintf(stderr, "%d\r", counter);

#ifdef STREAMS

		if (rc & MOREDATA) {

			fprintf(stderr,"Truncated trace info\n");

			while (1) {  
				/* read to the end and dump all */
				rc = getmsg(Mgmtfd, &cbuf, &dbuf, &flags);

				if (rc & MORECTL) {

					fprintf(stderr, "getmsg got MORECTL, invalid\n");
					close(Mgmtfd);
					exit(1);
				}

				if (! (rc & MOREDATA)){
					/* go back to reading */
					break;
				}
			}
		} 
#endif

	}

	/* NOT REACHED */
}

char padbuf[8];
char obuf[4096];
char ibuf[4096];

void
dump_packet(time_t seconds, trace_header_t *tracehdr)
{
	static int seqnum;
	static int firstrun = TRUE;
	static int previous_seqnum;
	unsigned int prim;
	TLF_DCH_RECORD rec;
	/* TLF_ASCII_RECORD arec; */
	int i, padbytes = 0;
	int rawdatasize;
	struct tm *tm;
	int transmitted = 0;
	char *iptr;

	tm = localtime(&seconds);

	if (firstrun == TRUE) {
		firstrun = FALSE;
		previous_seqnum = tracehdr->sequence - 1;
	}

	previous_seqnum++;

	if (tracehdr->sequence != previous_seqnum) {
		if (outfile_option == FALSE)
			fprintf(Outfp, 
			 "\n**LOST %i sequence numbers (try -o option)**\n", 
			 (tracehdr->sequence - previous_seqnum)); 
		else
			fprintf(Outfp, 
			 "\n**LOST %i sequence numbers**\n", 
			 (tracehdr->sequence - previous_seqnum));    
		previous_seqnum = tracehdr->sequence;
	}



	tracehdr->time_stamp.seconds = DO_SWAP4(tracehdr->time_stamp.seconds);	
	tracehdr->time_stamp.mseconds = DO_SWAP4(tracehdr->time_stamp.mseconds);

	switch (tracehdr->data_type) {
	case DCHAN_TX_DATA:
		transmitted = 1;
	case DCHAN_RX_DATA:
	case DCHAN_RXTX_DATA:
		rawdatasize = tracehdr->length;

		if (rawdatasize%4) {
                        padbytes = (rawdatasize%4);
                        padbytes = sizeof(uint) - padbytes;
                }
		rec.Header.Len = sizeof(rec) + rawdatasize + padbytes +
			sizeof(unsigned int);
		rec.Header.TypeId = TLF_TYPE_DCH;
		rec.Header.SeqNum = tracehdr->sequence;
		if(seqnum > 999)
			seqnum = 0;

		rec.Header.TimeStamp.Hour = tracehdr->time_stamp.seconds/3600;
		rec.Header.TimeStamp.Minute = 
			(tracehdr->time_stamp.seconds/60)%60;
		rec.Header.TimeStamp.Second = tracehdr->time_stamp.seconds%60;
		rec.Header.TimeStamp.CentiSecond = 
			tracehdr->time_stamp.mseconds / 100;
	
		rec.Header.DeviceId = tracehdr->channel;
		rec.Header.LineId = tracehdr->port;
	
		rec.IsdnStandard = ISDN_STANDARD_NI1;
		rec.Flags = (transmitted ? 0 : DCH_FLAG_DIR_RX );
		rec.DataLen = rawdatasize;
	
		if (raw_write_to_file) {  /* write output to raw file */
			if (fwrite(&rec, sizeof(rec), 1, Rawfp) != 1) {
				perror("fwrite record");
				exit(1);
			}

			if ((i = fwrite( tracehdr->payload, 1, rawdatasize, 
					Rawfp)) != rawdatasize) {
				fprintf(stderr,"fwrite raw %d != %d", 
						i, rawdatasize);
				if (ferror(Rawfp)) {
					exit(1);
				}
			}

			if (padbytes) {
				if (fwrite(padbuf, padbytes, 1, Rawfp) != 1) {
					perror("fwrite padbytes");
					exit(1);
				}	 
			}
	 
			if (fwrite(&rec.Header.Len, sizeof(rec.Header.Len), 
				1, Rawfp) != 1) {
				perror("fwrite datalen");
				exit(1);
			}
		}
		else {
			iptr = ibuf;

			bcopy(&rec, iptr, sizeof(rec));
			iptr += sizeof(rec);

			bcopy(tracehdr->payload, iptr, rawdatasize);
			iptr += rawdatasize;

			bcopy(padbuf, iptr, padbytes);
			iptr += padbytes;

			bcopy(&rec.Header.Len, iptr, sizeof(rec.Header.Len));
			iptr += sizeof(rec.Header.Len);

			/* Send it off to formatter */
			if ( isdn_dump((TLF_HEADER *)ibuf, obuf, sizeof(obuf),
				IDF_MSG_HEADER+IDF_INFO_HEADER+IDF_HEX_DUMP+
				IDF_WHITESPACE+IDF_INFO_ELEMENTS+
				IDF_NO_ASCII_HEX_DUMP) )
			{
				fprintf(stderr,
				"Failed to format raw trace event record\n");
				break;
			}

			/* Send the formatted buffer to destination */
			if (fputs ( obuf, Outfp ) == EOF )
			{
				fprintf(stderr,
				"Failed to output formatted trace event!\n");
				break;
			}
		}
		break;

	case BCHAN_TX_DATA:
		transmitted = 1;
	case BCHAN_RXTX_DATA:
	case BCHAN_RX_DATA:

		if (SerialMode == 's')
			print_data(tracehdr, transmitted);
		else
			report(tracehdr, &tracehdr->payload[0]);

		break;

	case SERIAL_TX_DATA:
		transmitted = 1;
	case SERIAL_RX_DATA:
	case SERIAL_RXTX_DATA:

		if (SerialMode == 's')
			print_data(tracehdr, transmitted);
		else
			report(tracehdr, &tracehdr->payload[0]);

		break;

#ifndef OLD_HEADERS
	case SERIAL_STATUS:
		if ((SerialMode != 's') && (ModemSignals || FlowControl))
			report_status(tracehdr, &tracehdr->payload[0]);
		break;
#endif

	default:
		fprintf(stderr,"Bad message type! (0x%x)\n", prim);
		break;
	}
}

int
write_header(void)
{
	TLF_LOG_FILE_HEADER hdr;
	time_t secs;
	struct tm *tm;

	time(&secs);
	tm = localtime(&secs);

	hdr.TraceFileVersion = TLF_FILE_VERSION;
	
 	sprintf(hdr.Title, TLF_LOG_TITLE, tm->tm_mon+1, tm->tm_mday, tm->tm_year,
					tm->tm_hour, tm->tm_min, tm->tm_sec);
	hdr.TraceEnginePlatform = TLF_OS_UNIX;

	if(fwrite(&hdr, sizeof(hdr), 1, Rawfp) != 1) {
		perror("fwrite header");
		exit(1);
	}
	return(0);
}

/*
 * decode_error_codes(): Decode error code according to
 * the type defined in the "mgmtdflt.h" header file.
 */
void decode_error_codes( int n ) {
	switch (n) {
	/*
	 * MGMT IOCTL error codes
	 */
	    case MGMT_OK:
		fprintf(stderr,"Error code is MGMT_OK");
		break;
	    case BAD_COMMAND:
		fprintf(stderr,"Error code is BAD_COMMAND");
		break;
	    case BAD_DFC:
		fprintf(stderr,"Error code is BAD_DFC");
		break;

	    case BAD_PARAMETER:
		fprintf(stderr,"Error code is BAD_PARAMETER");
		break;

	    case BAD_DEVICE:
		fprintf(stderr,"Error code is BAD_DEVICE");
		break;

	    case OUT_OF_MEMORY:
		fprintf(stderr,"Error code is OUT_OF_MEMORY");
		break;

	    case MGMT_SHIM_ERROR_UNKNOWN:
		fprintf(stderr,"Error code is MGMT_SHIM_ERROR_UNKNOWN");
		break;

	    case MGMT_SHIM_ERROR_BAD_PACKET_SIZE:
		fprintf(stderr,"Error code is MGMT_SHIM_ERROR_BAD_PACKET_SIZE");
		break;

	    case MGMT_SHIM_ERROR_NO_BOARDS:
		fprintf(stderr,"Error code is MGMT_SHIM_ERROR_NO_BOARDS");
		break;

	    case MGMT_SHIM_ERROR_NOT_INITIALIZED:
		fprintf(stderr,"Error code is MGMT_SHIM_ERROR_NOT_INITIALIZED");
		break;

	    case MGMT_SHIM_ERROR_OPERATION_TIMEOUT:
		fprintf(stderr,
			"Error code is MGMT_SHIM_ERROR_OPERATION_TIMEOUT ");
		break;

	    case MGMT_SHIM_ERROR_INTERNAL:
		fprintf(stderr,"Error code is MGMT_SHIM_ERROR_INTERNAL");
		break;

	    case MGMT_SHIM_ERROR_SERIALIZE_FAIL:
		fprintf(stderr,"Error code is MGMT_SHIM_ERROR_SERIALIZE_FAIL");
		break;

	    case MGMT_SHIM_ERROR_BOARD_NON_FUNCTIONAL:
		fprintf(stderr,
			"Error code is MGMT_SHIM_ERROR_BOARD_NON_FUNCTIONAL");
		break;

	}
	fprintf(stderr,"\n\n");
}

#ifdef STREAMS

int
SendCMD(int Mgmtfd , mgmt_req_t *req, void *resp_ptr )
{
	struct strbuf 	ctrl, hdp;
	char		buffer[MAX_TRACE_IN]; 
	mgmt_hd_t	mgmt_hdr;
	union mgmt_response_primitives *new;
	int		retval, flag=0;

	/*
	 * On the putmsg, we only use the contol portion (and not the data
	 * potion).  The control length should be sizeof(mgmt_hd_t) +
	 * sizeof(the request structure).
	 *
	 * On the getmsg, the control portion is the mgmt_hd_t and the data
	 * portion is the response structure.
	 */

	req->hd.hd_size	= sizeof(mgmt_hd_t) ;
	req->hd.boardno	= 1;
	req->hd.errorcode	= 0;

	ctrl.buf = (char*) req;
	ctrl.len = sizeof(mgmt_hd_t) + req->hd.mpkt_len;
	ctrl.maxlen = 0;		/* Not used by putmsg */

	if ((( retval = putmsg(Mgmtfd, &ctrl, NULL,0))) != 0) {
		fprintf(stderr,"putmsg error code=%d to device\n", retval);
		perror("putmsg ERROR");
	}

	mgmt_hdr.errorcode = 0;

	ctrl.buf = (char*) &mgmt_hdr;
	ctrl.maxlen = sizeof(mgmt_hdr);
	ctrl.len = 0;

	hdp.buf		= (char*)&buffer;
	hdp.maxlen	= sizeof(buffer);
	hdp.len		= 0;
	flag		= 0;

	if ((( retval = getmsg(Mgmtfd, &ctrl, &hdp,&flag))) < 0) {
		fprintf(stderr,"getmsg error code=%d to device\n", retval);
		errno	=	EIO;
		perror("getmsg ERROR");
	}

	if ( retval != 0 ) {
		fprintf(stderr,"\nCommand has been executed with error.\n");
		decode_error_codes(retval);
		exit(1);
	}

	if ((mgmt_hdr.rpkt_len == req->hd.rpkt_len) && (mgmt_hdr.rpkt_len > 0)) {
		new = (union mgmt_response_primitives *) hdp.buf;

		switch(req->hd.command)
		{
			case GET_MGMT_PROTOCOL_VERSION:
			memcpy(resp_ptr, &new->r_gainfo, req->hd.rpkt_len);
			break;

			case GET_ADAPTER_INFO:
			memcpy(resp_ptr, &new->r_gainfo, req->hd.rpkt_len);
			break;

			case GET_MGMT_BOARD_NO:
			memcpy(resp_ptr, &new->r_gmresp, req->hd.rpkt_len);
			break;

			case GET_MGMT_BOARD_INFO:
			memcpy(resp_ptr, &new->r_gminfo, req->hd.rpkt_len);
			break;
		} 
	}

	return (mgmt_hdr.errorcode);
}

#else ifdef IOCTL

int
SendCMD(int Mgmtfd , mgmt_req_t *req, void *resp_ptr )
{
	int	rc;
	char err_msg[256];

	/*
	 * Send the command
	 */

	rc = ioctl (Mgmtfd, req->hd.command, &(req->rq));

	if (rc < 0) {

		sprintf (err_msg, "SendCMD ioctl: Command = 0x%x", req->hd.command);
		perror (err_msg);

		exit (1);
	}

	/*
	 * Copy any expected response
	 */

	if (req->hd.rpkt_len > 0) {

		memcpy (resp_ptr, &(req->rq), req->hd.rpkt_len);
	}

	return (0);
}

#endif

void
print_data(trace_header_t *tracehdr, int transmitted)
{
	int i,tmp_hold, j, k, Column, hours, minutes, seconds, mseconds;

	hours = tracehdr->time_stamp.seconds/3600 ;
	minutes = tracehdr->time_stamp.seconds/60;
	minutes %= 60;
	seconds = tracehdr->time_stamp.seconds%60;
	mseconds = tracehdr->time_stamp.mseconds;

	fprintf(Outfp, "%d: %s : %04d:%02d:%02d:%04d\n", 
		tracehdr->sequence, transmitted ? "TX":"RX",
		hours, minutes, seconds, mseconds);

	j=0;
	if (tracehdr->length >= 1) {
		tmp_hold = tracehdr->length;
		for (i = 0; i < tmp_hold; i+=16) {
			if ((i+16) <= tmp_hold) {
				fprintf(Outfp, "%02x%02x%02x%02x "
					"%02x%02x%02x%02x "
					"%02x%02x%02x%02x "
					"%02x%02x%02x%02x "
					"%c%c%c%c%c%c%c%c"
					"%c%c%c%c%c%c%c%c\n",
				tracehdr->payload[i],
				tracehdr->payload[i+1],
				tracehdr->payload[i+2],
				tracehdr->payload[i+3],
				tracehdr->payload[i+4],
				tracehdr->payload[i+5],
				tracehdr->payload[i+6],
				tracehdr->payload[i+7],
				tracehdr->payload[i+8],
				tracehdr->payload[i+9],
				tracehdr->payload[i+10],
				tracehdr->payload[i+11],
				tracehdr->payload[i+12],
				tracehdr->payload[i+13],
				tracehdr->payload[i+14],
				tracehdr->payload[i+15],
				check_ascii(tracehdr->payload[i]),
				check_ascii(tracehdr->payload[i+1]),
				check_ascii(tracehdr->payload[i+2]),
				check_ascii(tracehdr->payload[i+3]),
				check_ascii(tracehdr->payload[i+4]),
				check_ascii(tracehdr->payload[i+5]),
				check_ascii(tracehdr->payload[i+6]),
				check_ascii(tracehdr->payload[i+7]),
				check_ascii(tracehdr->payload[i+8]),
				check_ascii(tracehdr->payload[i+9]),
				check_ascii(tracehdr->payload[i+10]),
				check_ascii(tracehdr->payload[i+11]),
				check_ascii(tracehdr->payload[i+12]),
				check_ascii(tracehdr->payload[i+13]),
				check_ascii(tracehdr->payload[i+14]),
				check_ascii(tracehdr->payload[i+15]));
			} else {
				Column=0;
				k=0;
				j=i;
				while (i < tmp_hold ) {
					if (k >= 4) {
						Column+=1;
						fprintf(Outfp, " ");
						k=0;
					}
					k++;
					fprintf(Outfp, 
						"%02x", tracehdr->payload[i]);
					Column+=2;
					i++;
				}
				for (k = Column; k < 37; k++) {
					fprintf(Outfp, " ");
				}
				i=j;
				while (i < tmp_hold ) {
					fprintf(Outfp, "%c", 
					check_ascii(tracehdr->payload[i]));
					i++;
				}
				fprintf(Outfp, "\n");
			}
		}
		fprintf(Outfp, "\n");
	}
}

int
check_ascii(int value)
{
	if ((value < ' ') || (value > 0x7e)) {
		return('.');
	} else {
		return(value);
	}
}

void
escaped_char(uchar c)
{
	switch (c)
	{
	case '\n':
		putc('\\', Outfp); putc('n', Outfp); Column += 2;
		break;
	case '\r':
		putc('\\', Outfp); putc('r', Outfp); Column += 2;
		break;
	case '\t':
		putc('\\', Outfp); putc('t', Outfp); Column += 2;
		break;
	default:
		if (c >= ' ' && c < 0x7f)
		{
			putc(c, Outfp);
			++Column;
		}
		else
		{
			fprintf(Outfp, "\\x%02x", c);
			Column += 4;
		}
	}
}

void
start_header(trace_header_t *hdr)
{
	fprintf(Outfp, "\n");
	Column = 0;

	if (TimeStamp)
	{
		fprintf(Outfp, "%10d.%03d ",
			hdr->time_stamp.seconds,
			hdr->time_stamp.mseconds);
		Column += 15;
	}
}

void
header(trace_header_t *hdr)
{
	start_header(hdr);

	fprintf(Outfp, "%c", Dir);
	++Column;

	if (SerialMode != 'x')
	{
		fprintf(Outfp, " ");
		++Column;
	}

	LastDir = Dir;
}

void
happy_fax_char(trace_header_t *hdr, uchar c)
{
	static int	state = 0;
	static uchar	*cp;
	static int	happy_stamp;
	static int	happy_cnt;
	uchar		*connect = "CONNECT\r\n";

	switch (state)
	{
	case 0: /* normal ascii mode: look for CONNECT */
		escaped_char(c);
		if (c == connect[0])
		{
			state = 1;
			cp = connect + 1;
		}
		break;
	case 1:	/* Find letters in CONNECT */
		escaped_char(c);
		if (c != *cp++)
			state = 0;
		else if (*cp == 0)
		{
			header(hdr);
			state = 2;
			happy_stamp = LastStamp;
			happy_cnt = 0;
		}
		break;
	case 2:	/* Page data */
		if (c == 0x10)
			state = 0x10;
		else
		{
			fprintf(Outfp, " %02x", c);
			Column += 3;
			++happy_cnt;
		}
		break;
	case 0x10:	/* Saw a DLE */
		if (c == 0x10)
		{
			fprintf(Outfp, " 10 10");
			Column += 6;
			state = 2;
			++happy_cnt;	/* Only one data char! */
		}
		else if (c == '.' || c == 0x03 || c == ',' || c == ';')
		{
			int	msecs;

			start_header(hdr);
			msecs = LastStamp - happy_stamp;
			fprintf(Outfp, "[ %.3f CPS; %d chars in %d.%03d secs]",
				happy_cnt * 1000.0 / msecs,
				happy_cnt, msecs/1000, msecs % 1000);
			header(hdr);
			escaped_char(0x10);
			escaped_char(c);
			state = 0;
		}
		else
		{
			fprintf(Outfp, " 10 %02x", c);
			Column += 6;
			state = 2;
		}
		break;
	}
}

void
report(trace_header_t *hdr, uchar *data)
{
	char		dir;
	int		i;
	int		stamp;

	// printf("hdr = 0x%04x\n", hdr->data_type);

	if ((hdr->data_type == SERIAL_TX_DATA) ||
	 (hdr->data_type == BCHAN_TX_DATA))
		Dir = '>';
	else
		Dir = '<';

	stamp = hdr->time_stamp.seconds * 1000 + hdr->time_stamp.mseconds;
	if (Dir != LastDir || (TimeStamp && (stamp - LastStamp > 100)) )
		header(hdr);
	LastStamp = stamp;

	switch (SerialMode)
	{
	case 'a':
		for (i = 0; i < hdr->length; ++i)
		{
			int	c = *data++;

			if (c < ' ' || c >= 0x7f)
				c = '.';
			putc(c, Outfp);
			++Column;
			if (Column >= 75)
				header(hdr);
		}
		break;
	case 'e':
		for (i = 0; i < hdr->length; ++i)
		{
			int	c = *data++;

			escaped_char(c);

			if (Column >= 75)
				header(hdr);
		}
		break;
	case 'f':
		for (i = 0; i < hdr->length; ++i)
		{
			int	c = *data++;

			happy_fax_char(hdr, c);

			if (Column >= 75)
				header(hdr);
		}
		break;
	case 'x':
		for (i = 0; i < hdr->length; ++i)
		{
			fprintf(Outfp, " %02x", *data++);
			Column += 3;
			if (Column >= 75)
				header(hdr);
		}
		break;
	}
	fflush(Outfp);
	
}

void
print_sig(int val, char *hiname, char *loname)
{
	char *name;
	char *extra = "";

	switch (val)
	{
	case SERIAL_SIGNAL_DEASSERTED:	name = loname; break;
	case SERIAL_SIGNAL_ASSERTED:	name = hiname; break;
	default:			extra = "?"; break;
	}
	fprintf(Outfp, " %s%s", extra, name);
}

void
report_status(trace_header_t *hdr, uchar *data)
{
	serial_status_t *status = (serial_status_t *) data;
	int		i;
	int		fc;
	int		stamp;

	stamp = hdr->time_stamp.seconds * 1000 + hdr->time_stamp.mseconds;
	Dir = 'C';
	header(hdr);
	LastStamp = stamp;

	if (ModemSignals)
	{
		print_sig(status->mdm_sig_state[SERIAL_SIGNAL_RTS],
						"+RTS", "-rts");
		print_sig(status->mdm_sig_state[SERIAL_SIGNAL_CTS],
						"+CTS", "-cts");
		print_sig(status->mdm_sig_state[SERIAL_SIGNAL_DTR],
						"+DTR", "-dtr");
		print_sig(status->mdm_sig_state[SERIAL_SIGNAL_DSR],
						"+DSR", "-dsr");
		print_sig(status->mdm_sig_state[SERIAL_SIGNAL_DCD],
						"+DCD", "-dcd");
	}

	if (FlowControl)
	{
		char	buf[32];

		sprintf(buf, "+IFC?");
		fc = SERIAL_SIGNAL_DEASSERTED;
		for (i = 0; i < SERIAL_IN_FLOW_MAX; ++i)
			if (status->in_flow_state[i] == SERIAL_FLOW_ASSERTED)
			{
				fc = SERIAL_SIGNAL_ASSERTED;
				sprintf(buf, "+IFC:%d", i);
				break;
			}
		print_sig(fc, buf, "-ifc");

		sprintf(buf, "+OFC?");
		fc = SERIAL_SIGNAL_DEASSERTED;
		for (i = 0; i < SERIAL_OUT_FLOW_MAX; ++i)
			if (status->out_flow_state[i] == SERIAL_FLOW_ASSERTED)
			{
				fc = SERIAL_SIGNAL_ASSERTED;
				sprintf(buf, "+OFC:%d", i);
				break;
			}
		print_sig(fc, buf, "-ofc");
	}

	fflush(Outfp);
}

