/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /eng/cvs/triboot/s2k_hw.c,v 1.2 2001/01/05 14:43:31 dickl Exp $
 */

#include "types.h"
#include "tribl.h"
#include "triglbl.h"


extern uint16 swap_le16(uint16, BOARD*);
extern uint32 swap_le32(uint32, BOARD*);
extern uint32 swap32(uint32, BOARD*);

void s2k_post_board_int(BOARD *);
int s2k_clear_board_int(BOARD *);
void s2k_unmask_board_int(BOARD *);
void s2k_mask_board_int(BOARD *);
void s2k_shutdown_board(BOARD *);
void s2k_start(BOARD *,uint32);
int32 s2k_postmap(BOARD *);
void s2k_reset(BOARD *,uint32);
void s2k_release_reset(BOARD *);

void s2k_post_board_int(BOARD *brd)
{ return; }

int s2k_clear_board_int(BOARD *brd)
{ return 0; }

void s2k_unmask_board_int(BOARD *brd)
{ return; }

void s2k_mask_board_int(BOARD *brd)
{ return; }

int32 s2k_postmap(BOARD *brd)
{ return 0; }

void s2k_release_reset(BOARD *brd)
{ return; }

void s2k_start(BOARD *brd, uint32 address)
{ 
 uint32 unmap;
 caddr_t vaddr, xb_vaddr;

/* First get to dualport area */
	if(brd->vram == NULL)
	{
		unmap = 1;
		vaddr = (void *)brd->ram;
		brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
	}
	else
	{
		unmap = 0;
		vaddr = (caddr_t)brd->vram;
	}

	if(vaddr == 0)
		return;

	xb_vaddr = vaddr + swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + brd->gco,
			brd->id, TRIBL_MEM), brd);

	xb_vaddr = vaddr + 0x2e00;

/* BUS/memory tranfer mode */
#define BUS_MASTER_DXBV2 5
	*(xb_vaddr + 0x53) = BUS_MASTER_DXBV2;

/* Start addr. of fep and GO instruction */
	TRIBL_PCI_WRITE32((vaddr+0x18),swap_le32(0x4000,brd),brd->id,TRIBL_MEM);
	TRIBL_PCI_WRITE32((vaddr+0x1c),swap_le32(0x1,brd),   brd->id,TRIBL_MEM);

/* then go instruction for FEP to proceed to read hostinfo */
	*(xb_vaddr + 0x51) = 0; 	/* rsp */
	*(xb_vaddr + 0x50) = 1; 	/* ctl */

    if(unmap)
		brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

	return; 
}

/* DBU: interpretation for S2k is vram=basedualport, vflash=PLX eeprom,
        vreg=controlarea (also dualport).  Reset instruction is WRITTEN to
        controlarea, but response is read from plxEEPROM.
*/
void
s2k_reset(BOARD *brd, uint32 force)
{
 caddr_t vaddr;
 uint32 unmap;
 tribl_uintptr_t delay;

/* First get to PLX area to clear response */
	if(brd->vflash == NULL)
	{
		unmap = 1;
		vaddr = (void *)brd->flash;
		brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->flashlen, NULL, NULL);
	}
	else
	{
		unmap = 0;
		vaddr = (caddr_t)brd->vflash;
	}

	if(vaddr == 0)
		return;

/* At offset 0x40 from the base of PLXeeprom is where 'DONE' will be reported
   so to assure a clean reset, first clear this area.
*/
	vaddr += 0x40;
	TRIBL_PCI_WRITE32(vaddr, 0, brd->id, TRIBL_FLASH);

    if(unmap)
		brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->flashlen, NULL, NULL);

/* The reset is written to the control area:in triboot-world, vreg */
	if(brd->vreg == NULL)
	{
		unmap = 1;
		vaddr = (void *)brd->reg;
		brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
	}
	else
	{
		unmap = 0;
		vaddr = brd->vreg;
    }

    if(vaddr == 0)
		return;

/* Write EB, delay, then EF to clear it */
	delay = 1;
	*((uint8 *)vaddr) = 0xEB;
	brd->cb(DELAY,&delay,NULL,NULL,NULL);
	*((uint8 *)vaddr) = 0xEF;

	if(unmap)
	    brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}


void
s2k_shutdown_board(BOARD *brd)
{
    s2k_reset(brd, 0);
    return;
}
